package no.ntnu.item.mam.utility.actors.nrg;

import java.util.HashMap;

import se.ericsson.eto.norarc.ejbactorframe.ActorSM;

import no.ntnu.item.mam.actors.ccnrgedgeactors.nrgcallcontrolfeature.NRGCallControlFeatureSM;
import no.ntnu.item.mam.signals.ccnrgedgeservice.HandleCall;

import com.ericsson.hosasdk.api.P_INVALID_CRITERIA;
import com.ericsson.hosasdk.api.TpAddress;
import com.ericsson.hosasdk.api.TpAddressPlan;
import com.ericsson.hosasdk.api.TpAddressPresentation;
import com.ericsson.hosasdk.api.TpAddressRange;
import com.ericsson.hosasdk.api.TpAddressScreening;
import com.ericsson.hosasdk.api.cc.TpAdditionalCallEventCriteria;
import com.ericsson.hosasdk.api.cc.TpCallAppInfo;
import com.ericsson.hosasdk.api.cc.TpCallEventRequest;
import com.ericsson.hosasdk.api.cc.TpCallEventType;
import com.ericsson.hosasdk.api.cc.TpCallMonitorMode;
import com.ericsson.hosasdk.api.cc.TpCallNotificationInfo;
import com.ericsson.hosasdk.api.cc.TpCallNotificationRequest;
import com.ericsson.hosasdk.api.cc.TpCallNotificationScope;
import com.ericsson.hosasdk.api.cc.TpReleaseCause;
import com.ericsson.hosasdk.api.cc.mpccs.IpAppCallLegAdapter;
import com.ericsson.hosasdk.api.cc.mpccs.IpAppMultiPartyCallAdapter;
import com.ericsson.hosasdk.api.cc.mpccs.IpAppMultiPartyCallControlManager;
import com.ericsson.hosasdk.api.cc.mpccs.IpAppMultiPartyCallControlManagerAdapter;
import com.ericsson.hosasdk.api.cc.mpccs.IpMultiPartyCallControlManager;
import com.ericsson.hosasdk.api.cc.mpccs.TpAppMultiPartyCallBack;
import com.ericsson.hosasdk.api.cc.mpccs.TpCallLegIdentifier;
import com.ericsson.hosasdk.api.cc.mpccs.TpMultiPartyCallIdentifier;
import com.ericsson.hosasdk.utility.sync.ThreadPool;

public class NRGCCProcessor extends IpAppMultiPartyCallControlManagerAdapter implements IpAppMultiPartyCallControlManager {
	
	private IpMultiPartyCallControlManager itsMPCCManager;
    private NRGCallControlFeatureSM theFeature;
    private IpAppMultiPartyCallAdapter theCallCallback = new IpAppMultiPartyCallAdapter();
    private IpAppCallLegAdapter theLegCallback = new IpAppCallLegAdapter();
    private ThreadPool itsThreadPool = new ThreadPool();

    /**
     * Creates a new instance.
     * 
     * @param aMPCCManager a Multi Party Call Control manager
     * @param aFeature the feature that will receive events
     */
    public NRGCCProcessor(IpMultiPartyCallControlManager manager, ActorSM featureSM) {
    	this.itsMPCCManager = manager;
        this.theFeature = (NRGCallControlFeatureSM)featureSM;
	}

	/**
     * Disposes of this instance.
     */

    public void dispose() {
        super.dispose();
        theLegCallback.dispose();
        theCallCallback.dispose();
    }

    /**
     * Requests to be notified when a subscriber in the network dials a given
     * destination number. The Feature will be notified via handleCall.
     * 
     * @returns the assignment Ids to be used for stopping notifications
     * @param destinations the destination numbers that subscribe to the SBCC service
     */

    public HashMap startNotifications(String[] destinations) {
        
    	HashMap notificationIDs = new HashMap();
        
        //need to create notifications for all the destinations
    	for (int i=0;i<destinations.length;i++) {
    		// current destination
    		String aDestination = destinations[i];
    		// the scope of the notification: 
    		// every call to current destination from any source
    		TpCallNotificationScope scope = new TpCallNotificationScope(createAddressRange(aDestination),
            createAddressRange("*"));
	
    		// request to be notified when the destination number has just been dialled
    		TpCallEventType eventType = TpCallEventType.P_CALL_EVENT_ADDRESS_COLLECTED;
    		TpAdditionalCallEventCriteria additionalCriteria = new TpAdditionalCallEventCriteria();

    		additionalCriteria.MinAddressLength(aDestination.length());
	
    		// when the notification is received, this application will take control the call
    		TpCallMonitorMode mode = TpCallMonitorMode.P_CALL_MONITOR_MODE_INTERRUPT;
	
    		// compose the notification request with the above data
    		TpCallEventRequest request = new TpCallEventRequest(eventType,
            additionalCriteria, mode);
    		TpCallEventRequest[] requests = {request};
    		
            // activate the notification request
    		try {
    			Integer notificationID = new Integer(itsMPCCManager.createNotification(this,
        	            new TpCallNotificationRequest(scope, requests)));
    			// store the different notificationIDs for future stopping of the notifications
        		notificationIDs.put(notificationID, null); 
    		}
    		catch (P_INVALID_CRITERIA e) {
    			//a notification request with the same criteria (same originating and terminating 
    			//address + same number plan) is already requested and overlaps with the current 
    			//notification request 
    			e.printStackTrace();
    		}
    		
    	}
        // return the notificationIDs to the NRGCallControlFeature 
        return notificationIDs;
    }
    
    /**
     * Deactivates the trigger that was set with <code>startNotifications</code>.
     */

    public void stopNotifications(int anAssignmentId) {
        itsMPCCManager.destroyNotification(anAssignmentId);
    }

    public TpAppMultiPartyCallBack reportNotification(
        TpMultiPartyCallIdentifier aCall, 
        TpCallLegIdentifier[] aLegs, 
        TpCallNotificationInfo anInfo, 
        int assignmentID)
    {
        // we want to return the callback for the call to Ericsson Network Resource Gateway as soon as possible,
        // without blocking this method until the call has been handled.
        // This requires the use of a thread. For reasons of efficiency, 
        // we don not want to create a new thread for each and every new call. 
        // A ThreadPool allows us the handle the call in a separate, re-usable thread.
    	
    	//System.out.println("In reportNotification()!!!!!!!!!!!!!!!!!!!!!!");
        //System.out.println("Number of legs: " + aLegs.length);
        //System.out.println("!!!!!!!!!!!!!!!!!!!!!!");
    	
        final TpMultiPartyCallIdentifier call = aCall;
        final TpCallLegIdentifier firstLeg = aLegs[0];
        final TpAddress originatingAddress = anInfo.CallNotificationReportScope.OriginatingAddress;
        final TpAddress destinationAddress = anInfo.CallNotificationReportScope.DestinationAddress;
        itsThreadPool.start(new Runnable()
        {
            public void run()
            {
            	//theFeature.handleCall(call, firstLeg, originatingAddress);
            	theFeature.sendMessage(new HandleCall(call, firstLeg, originatingAddress, destinationAddress), theFeature.getMyActorAddress());
            }
        });
        TpAppMultiPartyCallBack callback = new TpAppMultiPartyCallBack();
        callback.AppMultiPartyCall(theCallCallback);
        return callback;
    }

    /**
     * Creates a new leg and routes it towards a given destination.
     * 
     * @param aCall the call for which to create and route a new leg
     * @param aSource the calling party (can be used for number 
     *     presentation in the network)
     * @param aDestination the address of the called party
     */

    public void route(TpMultiPartyCallIdentifier aCall, TpAddress aSource, String aDestination) {
    	TpAddress destination = createAddress(aDestination);
    	TpAddress source = aSource;
        TpCallLegIdentifier newLeg = aCall.CallReference.createAndRouteCallLegReq(aCall.CallSessionID,
            new TpCallEventRequest[0], destination,
            source, new TpCallAppInfo[0], theLegCallback);
    }

    /**
     * Returns control of a leg over to the network
     * 
     * @param aLeg the leg to be continued
     */

    public void continueProcessing(TpCallLegIdentifier aLeg) {
        aLeg.CallLegReference.continueProcessing(aLeg.CallLegSessionID);
    }

    /**
     * Disconnects the association between Ericsson Network Resource Gateway and the network of the call; the call is terminated.
     * 
     * @param aCall the call to terminate
     */

    public void release(TpMultiPartyCallIdentifier aCall) {
        aCall.CallReference.release(aCall.CallSessionID,
            TpReleaseCause.P_UNDEFINED);
    }

    /**
     * Disconnects the association between Ericsson Network Resource Gateway and the network of the call; the call continues.
     * 
     * @param aCall the call to deassign
     */

    public void deassign(TpMultiPartyCallIdentifier aCall) {
        aCall.CallReference.deassignCall(aCall.CallSessionID);
    }

    private static TpAddress createAddress(String anE164Number) {
        TpAddress address = new TpAddress();
        address.Plan = TpAddressPlan.P_ADDRESS_PLAN_E164;
        address.AddrString = anE164Number;
        address.Name = "";
        address.SubAddressString = "";
        address.Presentation = TpAddressPresentation.P_ADDRESS_PRESENTATION_UNDEFINED;
        address.Screening = TpAddressScreening.P_ADDRESS_SCREENING_UNDEFINED;
        return address;
    }

    private static TpAddressRange createAddressRange(String anE164Number) {
        TpAddressRange range = new TpAddressRange();
        range.Plan = TpAddressPlan.P_ADDRESS_PLAN_E164;
        range.AddrString = anE164Number;
        range.Name = "";
        range.SubAddressString = "";
        return range;
    }

}
