package no.ntnu.item.contextengine.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.PropertyResourceBundle;
import java.util.StringTokenizer;

import no.ntnu.item.contextengine.contextengine.ContextEngineSM;
import no.ntnu.item.contextengine.model.OntologyOperator;
import no.ntnu.item.contextengine.model.ontologyclasses.Absolute;
import no.ntnu.item.contextengine.model.ontologyclasses.ContextInformation;
import no.ntnu.item.contextengine.model.ontologyclasses.Humidity;
import no.ntnu.item.contextengine.model.ontologyclasses.InstantStatus;
import no.ntnu.item.contextengine.model.ontologyclasses.Latitude;
import no.ntnu.item.contextengine.model.ontologyclasses.Location;
import no.ntnu.item.contextengine.model.ontologyclasses.Longitude;
import no.ntnu.item.contextengine.model.ontologyclasses.LuxConditions;
import no.ntnu.item.contextengine.model.ontologyclasses.Metric;
import no.ntnu.item.contextengine.model.ontologyclasses.Noise;
import no.ntnu.item.contextengine.model.ontologyclasses.PersonEntity;
import no.ntnu.item.contextengine.model.ontologyclasses.Pressure;
import no.ntnu.item.contextengine.model.ontologyclasses.Relative;
import no.ntnu.item.contextengine.model.ontologyclasses.ScheduledActivity;
import no.ntnu.item.contextengine.model.ontologyclasses.Temperature;
import no.ntnu.item.contextengine.model.ontologyclasses.Time;
import no.ntnu.item.contextengine.sensors.activitysensor.ActivitySensor;
import no.ntnu.item.contextengine.sensors.timesensor.TimeSensor;
import no.ntnu.item.contextengine.signals.ContextUpdate;
import edu.stanford.smi.protegex.owl.jena.JenaOWLModel;
import edu.stanford.smi.protegex.owl.model.OWLIndividual;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;

public class ContextReasoner {

	private PropertyResourceBundle userProperties = null;

	private PropertyResourceBundle individualProperties = null;

	private JenaOWLModel owlModel = null;

	private OntologyOperator operator = null;

	private String location = new String();

	private String time = new String();

	private boolean activityRegistered = false;

	private ContextEngineSM myContextEngine = null;

	private TimeSensor timeSensor = null;

	private String personIndividualName = null;

	public ContextReasoner(ContextEngineSM myContextEngine) {
		this.myContextEngine = myContextEngine;
		try {
			this.userProperties = new PropertyResourceBundle(
					new FileInputStream("userConfiguration.properties"));
			this.individualProperties = new PropertyResourceBundle(
					new FileInputStream("individualConfiguration.properties"));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// start up the ontology operator
		this.operator = new OntologyOperator();
		// create the owl model based on the Ontology file
		this.owlModel = operator.fetchStructure("finalContextOntology.owl");
		// 1. PersonEntity ()
		this.personIndividualName = userProperties.getString("PersonEntity");
		String personIndividualID = userProperties.getString("personID");
		owlModel = operator.assignIndividual(owlModel, "PersonEntity",
				personIndividualName);
		owlModel = operator.assignDataProperty(owlModel, personIndividualName,
				"hasIdentity", personIndividualID);
		// 2. Metrics
		String homeAvailabilityMetric = individualProperties
				.getString("homeAvailabilityMetric");
		String homeAvailabilityMetricType = individualProperties
				.getString("homeAvailabilityMetricType");
		owlModel = operator.assignIndividual(owlModel, "Metric",
				homeAvailabilityMetric);
		owlModel = operator.assignDataProperty(owlModel,
				homeAvailabilityMetric, "hasType", homeAvailabilityMetricType);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				homeAvailabilityMetric, "hasValue", "undefinedValue");
		String officeAvailabilityMetric = individualProperties
				.getString("officeAvailabilityMetric");
		String officeAvailabilityMetricType = individualProperties
				.getString("officeAvailabilityMetricType");
		owlModel = operator.assignIndividual(owlModel, "Metric",
				officeAvailabilityMetric);
		owlModel = operator.assignDataProperty(owlModel,
				officeAvailabilityMetric, "hasType",
				officeAvailabilityMetricType);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				officeAvailabilityMetric, "hasValue", "undefinedValue");
		String meetingRoomAvailabilityMetric = individualProperties
				.getString("meetingRoomAvailabilityMetric");
		String meetingRoomAvailabilityMetricType = individualProperties
				.getString("meetingRoomAvailabilityMetricType");
		owlModel = operator.assignIndividual(owlModel, "Metric",
				meetingRoomAvailabilityMetric);
		owlModel = operator.assignDataProperty(owlModel,
				meetingRoomAvailabilityMetric, "hasType",
				meetingRoomAvailabilityMetricType);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				meetingRoomAvailabilityMetric, "hasValue", "undefinedValue");
		// 6. Availability
		String homeAvailability = individualProperties
				.getString("homeAvailability");
		String homeAvailabilityCharacterizes = personIndividualName;
		String homeAvailabilityHasMetric = homeAvailabilityMetric;
		owlModel = operator.assignIndividual(owlModel, "Availability",
				homeAvailability);
		owlModel = operator.assignObjectProperty(owlModel, homeAvailability,
				"characterizes", homeAvailabilityCharacterizes);
		owlModel = operator.assignObjectProperty(owlModel, homeAvailability,
				"hasMetric", homeAvailabilityHasMetric);
		String officeAvailability = individualProperties
				.getString("officeAvailability");
		String officeAvailabilityCharacterizes = personIndividualName;
		String officeAvailabilityHasMetric = officeAvailabilityMetric;
		owlModel = operator.assignIndividual(owlModel, "Availability",
				officeAvailability);
		owlModel = operator.assignObjectProperty(owlModel, officeAvailability,
				"characterizes", officeAvailabilityCharacterizes);
		owlModel = operator.assignObjectProperty(owlModel, officeAvailability,
				"hasMetric", officeAvailabilityHasMetric);
		String meetingRoomAvailability = individualProperties
				.getString("meetingRoomAvailability");
		String meetingRoomAvailabilityCharacterizes = personIndividualName;
		String meetingRoomAvailabilityHasMetric = meetingRoomAvailabilityMetric;
		owlModel = operator.assignIndividual(owlModel, "Availability",
				meetingRoomAvailability);
		owlModel = operator.assignObjectProperty(owlModel,
				meetingRoomAvailability, "characterizes",
				meetingRoomAvailabilityCharacterizes);
		owlModel = operator.assignObjectProperty(owlModel,
				meetingRoomAvailability, "hasMetric",
				meetingRoomAvailabilityHasMetric);
		// 9. DeducedActivity
		String nowDeducedActivity = individualProperties
				.getString("nowDeducedActivity");
		owlModel = operator.assignIndividual(owlModel, "DeducedActivity",
				nowDeducedActivity);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				nowDeducedActivity, "hasDescription", "undefinedValue");

	}

	public void setTimeSensor(TimeSensor timeSensor) {
		this.timeSensor = timeSensor;
	}

	public void register(String type) {
		// TODO: write code for the other types of sensors...
		// based on the type of information that is to be received from this
		// sensor, initialize the
		// individuals that will be needed in the owlModel
		// Load from userConfiguration the ID of the PersonEntityInstance
		try {
			this.userProperties = new PropertyResourceBundle(
					new FileInputStream("userConfiguration.properties"));
			this.individualProperties = new PropertyResourceBundle(
					new FileInputStream("individualConfiguration.properties"));
			if (type.equals("Location")) {
				System.out.println("Location sensor added");
				// adds information about both Absolute and Relative location
				// meaning that the sensor can add information about the current
				// absoulute location
				// and information (longitude, latitude, altitude) about the
				// Relative locations

				// the sensor itself is not responsible for mapping between
				// Relative and Absolute location,
				// this is the rsponsibility of the ContextReasoner

				// Metrics
				String nowLongitudeMetric = individualProperties
						.getString("nowLongitudeMetric");
				String nowLongitudeMetricType = individualProperties
						.getString("nowLongitudeMetricType");
				String nowLongitudeMetricUnit = individualProperties
						.getString("nowLongitudeMetricUnit");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						nowLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						nowLongitudeMetric, "hasType", nowLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						nowLongitudeMetric, "hasUnit", nowLongitudeMetricUnit);
				this.owlModel = this.operator.assignDataProperty(this.owlModel,
						nowLongitudeMetric, "hasValue", "undefinedValue");
				String homeLongitudeMetric = individualProperties
						.getString("homeLongitudeMetric");
				String homeLongitudeMetricType = individualProperties
						.getString("homeLongitudeMetricType");
				String homeLongitudeMetricUnit = individualProperties
						.getString("homeLongitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String homeLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						homeLongitudeMetric);
				owlModel = operator
						.assignDataProperty(owlModel, homeLongitudeMetric,
								"hasType", homeLongitudeMetricType);
				owlModel = operator
						.assignDataProperty(owlModel, homeLongitudeMetric,
								"hasUnit", homeLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						homeLongitudeMetric, "hasValue",
						homeLongitudeMetricValue);
				String officeLongitudeMetric = individualProperties
						.getString("officeLongitudeMetric");
				String officeLongitudeMetricType = individualProperties
						.getString("officeLongitudeMetricType");
				String officeLongitudeMetricUnit = individualProperties
						.getString("officeLongitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String officeLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						officeLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasType",
						officeLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasUnit",
						officeLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasValue",
						officeLongitudeMetricValue);
				String meetingRoomLongitudeMetric = individualProperties
						.getString("meetingRoomLongitudeMetric");
				String meetingRoomLongitudeMetricType = individualProperties
						.getString("meetingRoomLongitudeMetricType");
				String meetingRoomLongitudeMetricUnit = individualProperties
						.getString("meetingRoomLongitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String meetingRoomLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						meetingRoomLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasType",
						meetingRoomLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasUnit",
						meetingRoomLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasValue",
						meetingRoomLongitudeMetricValue);
				String restaurantLongitudeMetric = individualProperties
						.getString("restaurantLongitudeMetric");
				String restaurantLongitudeMetricType = individualProperties
						.getString("restaurantLongitudeMetricType");
				String restaurantLongitudeMetricUnit = individualProperties
						.getString("restaurantLongitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String restaurantLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						restaurantLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasType",
						restaurantLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasUnit",
						restaurantLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasValue",
						restaurantLongitudeMetricValue);
				String nowLatitudeMetric = individualProperties
						.getString("nowLatitudeMetric");
				String nowLatitudeMetricType = individualProperties
						.getString("nowLatitudeMetricType");
				String nowLatitudeMetricUnit = individualProperties
						.getString("nowLatitudeMetricUnit");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						nowLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						nowLatitudeMetric, "hasType", nowLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						nowLatitudeMetric, "hasUnit", nowLatitudeMetricUnit);
				this.owlModel = this.operator.assignDataProperty(this.owlModel,
						nowLatitudeMetric, "hasValue", "undefinedValue");
				String homeLatitudeMetric = individualProperties
						.getString("homeLatitudeMetric");
				String homeLatitudeMetricType = individualProperties
						.getString("homeLatitudeMetricType");
				String homeLatitudeMetricUnit = individualProperties
						.getString("homeLatitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String homeLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						homeLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						homeLatitudeMetric, "hasType", homeLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						homeLatitudeMetric, "hasUnit", homeLatitudeMetricUnit);
				owlModel = operator
						.assignDataProperty(owlModel, homeLatitudeMetric,
								"hasValue", homeLatitudeMetricValue);
				String officeLatitudeMetric = individualProperties
						.getString("officeLatitudeMetric");
				String officeLatitudeMetricType = individualProperties
						.getString("officeLatitudeMetricType");
				String officeLatitudeMetricUnit = individualProperties
						.getString("officeLatitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String officeLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						officeLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasType",
						officeLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasUnit",
						officeLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasValue",
						officeLatitudeMetricValue);
				String meetingRoomLatitudeMetric = individualProperties
						.getString("meetingRoomLatitudeMetric");
				String meetingRoomLatitudeMetricType = individualProperties
						.getString("meetingRoomLatitudeMetricType");
				String meetingRoomLatitudeMetricUnit = individualProperties
						.getString("meetingRoomLatitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String meetingRoomLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						meetingRoomLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasType",
						meetingRoomLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasUnit",
						meetingRoomLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasValue",
						meetingRoomLatitudeMetricValue);
				String restaurantLatitudeMetric = individualProperties
						.getString("restaurantLatitudeMetric");
				String restaurantLatitudeMetricType = individualProperties
						.getString("restaurantLatitudeMetricType");
				String restaurantLatitudeMetricUnit = individualProperties
						.getString("restaurantLatitudeMetricUnit");
				// the value will be provided by the sensor when it is updating
				// relative location
				// information
				String restaurantLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						restaurantLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasType",
						restaurantLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasUnit",
						restaurantLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasValue",
						restaurantLatitudeMetricValue);
				// 4. Latitude
				String nowLatitude = individualProperties
						.getString("nowLatitude");
				String nowLatitudeCharacterizes = personIndividualName;
				String nowLatitudehasMetric = nowLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						nowLatitude);
				owlModel = operator.assignObjectProperty(owlModel, nowLatitude,
						"characterizes", nowLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel, nowLatitude,
						"hasMetric", nowLatitudehasMetric);
				String officeLatitude = individualProperties
						.getString("officeLatitude");
				String officeLatitudeCharacterizes = personIndividualName;
				String officeLatitudehasMetric = officeLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						officeLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLatitude, "characterizes",
						officeLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLatitude, "hasMetric", officeLatitudehasMetric);
				String meetingRoomLatitude = individualProperties
						.getString("meetingRoomLatitude");
				String meetingRoomLatitudeCharacterizes = personIndividualName;
				String meetingRoomLatitudehasMetric = meetingRoomLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						meetingRoomLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLatitude, "characterizes",
						meetingRoomLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLatitude, "hasMetric",
						meetingRoomLatitudehasMetric);
				String homeLatitude = individualProperties
						.getString("homeLatitude");
				String homeLatitudeCharacterizes = personIndividualName;
				String homeLatitudehasMetric = homeLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						homeLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLatitude, "characterizes",
						homeLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLatitude, "hasMetric", homeLatitudehasMetric);
				String restaurantLatitude = individualProperties
						.getString("restaurantLatitude");
				String restaurantLatitudeCharacterizes = personIndividualName;
				String restaurantLatitudehasMetric = restaurantLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						restaurantLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLatitude, "characterizes",
						restaurantLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLatitude, "hasMetric",
						restaurantLatitudehasMetric);
				// 5. Longitude
				String nowLongitude = individualProperties
						.getString("nowLongitude");
				String nowLongitudeCharacterizes = personIndividualName;
				String nowLongitudehasMetric = nowLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						nowLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						nowLongitude, "characterizes",
						nowLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						nowLongitude, "hasMetric", nowLongitudehasMetric);
				String officeLongitude = individualProperties
						.getString("officeLongitude");
				String officeLongitudeCharacterizes = personIndividualName;
				String officeLongitudehasMetric = officeLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						officeLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLongitude, "characterizes",
						officeLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLongitude, "hasMetric", officeLongitudehasMetric);
				String meetingRoomLongitude = individualProperties
						.getString("meetingRoomLongitude");
				String meetingRoomLongitudeCharacterizes = personIndividualName;
				String meetingRoomLongitudehasMetric = meetingRoomLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						meetingRoomLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLongitude, "characterizes",
						meetingRoomLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLongitude, "hasMetric",
						meetingRoomLongitudehasMetric);
				String homeLongitude = individualProperties
						.getString("homeLongitude");
				String homeLongitudeCharacterizes = personIndividualName;
				String homeLongitudehasMetric = homeLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						homeLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLongitude, "characterizes",
						homeLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLongitude, "hasMetric", homeLongitudehasMetric);
				String restaurantLongitude = individualProperties
						.getString("restaurantLongitude");
				String restaurantLongitudeCharacterizes = personIndividualName;
				String restaurantLongitudehasMetric = restaurantLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						restaurantLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLongitude, "characterizes",
						restaurantLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLongitude, "hasMetric",
						restaurantLongitudehasMetric);
				// 7. Relative locations
				String homeRelativeLocation = individualProperties
						.getString("homeRelativeLocation");
				String homeRelativeLocationHasAddress = "undefinedValue";
				String homeRelativeLocationCharacterizes = personIndividualName;
				String homeRelativeLocationHasLatitude = homeLatitude;
				String homeRelativeLocationHasLongitude = homeLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						homeRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "characterizes",
						homeRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						homeRelativeLocation, "hasAddress",
						homeRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "hasLongitude",
						homeRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "hasLatitude",
						homeRelativeLocationHasLatitude);
				String officeRelativeLocation = individualProperties
						.getString("officeRelativeLocation");
				String officeRelativeLocationHasAddress = "undefinedValue";
				String officeRelativeLocationCharacterizes = personIndividualName;
				String officeRelativeLocationHasLatitude = officeLatitude;
				String officeRelativeLocationHasLongitude = officeLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						officeRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "characterizes",
						officeRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						officeRelativeLocation, "hasAddress",
						officeRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "hasLongitude",
						officeRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "hasLatitude",
						officeRelativeLocationHasLatitude);
				String meetingRoomRelativeLocation = individualProperties
						.getString("meetingRoom");
				String meetingRoomRelativeLocationCharacterizes = personIndividualName;
				String meetingRoomRelativeLocationHasLatitude = meetingRoomLatitude;
				String meetingRoomRelativeLocationHasLongitude = meetingRoomLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						meetingRoomRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "characterizes",
						meetingRoomRelativeLocationCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "hasLongitude",
						meetingRoomRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "hasLatitude",
						meetingRoomRelativeLocationHasLatitude);
				String restaurantRelativeLocation = individualProperties
						.getString("restaurantRelativeLocation");
				String restaurantRelativeLocationHasAddress = "undefinedValue";
				String restaurantRelativeLocationCharacterizes = personIndividualName;
				String restaurantRelativeLocationHasLatitude = restaurantLatitude;
				String restaurantRelativeLocationHasLongitude = restaurantLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						restaurantRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "characterizes",
						restaurantRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantRelativeLocation, "hasAddress",
						restaurantRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "hasLongitude",
						restaurantRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "hasLatitude",
						restaurantRelativeLocationHasLatitude);
				// 8. Presence
				String homePresence = individualProperties
						.getString("homePresence");
				String homePresenceCharacterizes = personIndividualName;
				String homePresenceHasLocation = homeRelativeLocation;
				String homePresenceHasAvailability = this.individualProperties
						.getString("homeAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						homePresence);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "characterizes",
						homePresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "hasLocation", homePresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "hasAvailability",
						homePresenceHasAvailability);
				String officePresence = individualProperties
						.getString("officePresence");
				String officePresenceCharacterizes = personIndividualName;
				String officePresenceHasLocation = officeRelativeLocation;
				String officePresenceHasAvailability = this.individualProperties
						.getString("officeAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						officePresence);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "characterizes",
						officePresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "hasLocation",
						officePresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "hasAvailability",
						officePresenceHasAvailability);
				String meetingRoomPresence = individualProperties
						.getString("meetingRoomPresence");
				String meetingRoomPresenceCharacterizes = personIndividualName;
				String meetingRoomPresenceHasLocation = meetingRoomRelativeLocation;
				String meetingRoomPresenceHasAvailability = this.individualProperties
						.getString("meetingRoomAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						meetingRoomPresence);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "characterizes",
						meetingRoomPresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "hasLocation",
						meetingRoomPresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "hasAvailability",
						meetingRoomPresenceHasAvailability);
			} else if (type.equals("AbsoluteLocation")) {
				// adds information about only the Absolute location
				System.out.println("AbsoluteLocation sensor added");
				// no information about any Relative locations are received
				// since no information about Relative locations are added,
				// reasoning about presence is
				// not possible, since it is characterized by a relative
				// location

				// Metrics
				String nowLongitudeMetric = individualProperties
						.getString("nowLongitudeMetric");
				String nowLongitudeMetricType = individualProperties
						.getString("nowLongitudeMetricType");
				String nowLongitudeMetricUnit = individualProperties
						.getString("nowLongitudeMetricUnit");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						nowLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						nowLongitudeMetric, "hasType", nowLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						nowLongitudeMetric, "hasUnit", nowLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						nowLongitudeMetric, "hasValue", "undefinedValue");
				String nowLatitudeMetric = individualProperties
						.getString("nowLatitudeMetric");
				String nowLatitudeMetricType = individualProperties
						.getString("nowLatitudeMetricType");
				String nowLatitudeMetricUnit = individualProperties
						.getString("nowLatitudeMetricUnit");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						nowLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						nowLatitudeMetric, "hasType", nowLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						nowLatitudeMetric, "hasUnit", nowLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						nowLatitudeMetric, "hasValue", "undefinedValue");
				// Metrics
				String homeLongitudeMetric = individualProperties
						.getString("homeLongitudeMetric");
				String homeLongitudeMetricType = individualProperties
						.getString("homeLongitudeMetricType");
				String homeLongitudeMetricUnit = individualProperties
						.getString("homeLongitudeMetricUnit");
				String homeLongitudeMetricValue = individualProperties
						.getString("homeLongitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						homeLongitudeMetric);
				owlModel = operator
						.assignDataProperty(owlModel, homeLongitudeMetric,
								"hasType", homeLongitudeMetricType);
				owlModel = operator
						.assignDataProperty(owlModel, homeLongitudeMetric,
								"hasUnit", homeLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						homeLongitudeMetric, "hasValue",
						homeLongitudeMetricValue);
				String officeLongitudeMetric = individualProperties
						.getString("officeLongitudeMetric");
				String officeLongitudeMetricType = individualProperties
						.getString("officeLongitudeMetricType");
				String officeLongitudeMetricUnit = individualProperties
						.getString("officeLongitudeMetricUnit");
				String officeLongitudeMetricValue = individualProperties
						.getString("officeLongitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						officeLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasType",
						officeLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasUnit",
						officeLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasValue",
						officeLongitudeMetricValue);
				String meetingRoomLongitudeMetric = individualProperties
						.getString("meetingRoomLongitudeMetric");
				String meetingRoomLongitudeMetricType = individualProperties
						.getString("meetingRoomLongitudeMetricType");
				String meetingRoomLongitudeMetricUnit = individualProperties
						.getString("meetingRoomLongitudeMetricUnit");
				String meetingRoomLongitudeMetricValue = individualProperties
						.getString("meetingRoomLongitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						meetingRoomLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasType",
						meetingRoomLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasUnit",
						meetingRoomLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasValue",
						meetingRoomLongitudeMetricValue);
				String restaurantLongitudeMetric = individualProperties
						.getString("restaurantLongitudeMetric");
				String restaurantLongitudeMetricType = individualProperties
						.getString("restaurantLongitudeMetricType");
				String restaurantLongitudeMetricUnit = individualProperties
						.getString("restaurantLongitudeMetricUnit");
				String restaurantLongitudeMetricValue = individualProperties
						.getString("restaurantLongitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						restaurantLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasType",
						restaurantLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasUnit",
						restaurantLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasValue",
						restaurantLongitudeMetricValue);
				String homeLatitudeMetric = individualProperties
						.getString("homeLatitudeMetric");
				String homeLatitudeMetricType = individualProperties
						.getString("homeLatitudeMetricType");
				String homeLatitudeMetricUnit = individualProperties
						.getString("homeLatitudeMetricUnit");
				String homeLatitudeMetricValue = individualProperties
						.getString("homeLatitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						homeLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						homeLatitudeMetric, "hasType", homeLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						homeLatitudeMetric, "hasUnit", homeLatitudeMetricUnit);
				owlModel = operator
						.assignDataProperty(owlModel, homeLatitudeMetric,
								"hasValue", homeLatitudeMetricValue);
				String officeLatitudeMetric = individualProperties
						.getString("officeLatitudeMetric");
				String officeLatitudeMetricType = individualProperties
						.getString("officeLatitudeMetricType");
				String officeLatitudeMetricUnit = individualProperties
						.getString("officeLatitudeMetricUnit");
				String officeLatitudeMetricValue = individualProperties
						.getString("officeLatitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						officeLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasType",
						officeLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasUnit",
						officeLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasValue",
						officeLatitudeMetricValue);
				String meetingRoomLatitudeMetric = individualProperties
						.getString("meetingRoomLatitudeMetric");
				String meetingRoomLatitudeMetricType = individualProperties
						.getString("meetingRoomLatitudeMetricType");
				String meetingRoomLatitudeMetricUnit = individualProperties
						.getString("meetingRoomLatitudeMetricUnit");
				String meetingRoomLatitudeMetricValue = individualProperties
						.getString("meetingRoomLatitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						meetingRoomLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasType",
						meetingRoomLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasUnit",
						meetingRoomLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasValue",
						meetingRoomLatitudeMetricValue);
				String restaurantLatitudeMetric = individualProperties
						.getString("restaurantLatitudeMetric");
				String restaurantLatitudeMetricType = individualProperties
						.getString("restaurantLatitudeMetricType");
				String restaurantLatitudeMetricUnit = individualProperties
						.getString("restaurantLatitudeMetricUnit");
				String restaurantLatitudeMetricValue = individualProperties
						.getString("restaurantLatitudeMetricValue");
				owlModel = operator.assignIndividual(owlModel, "Metric",
						restaurantLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasType",
						restaurantLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasUnit",
						restaurantLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasValue",
						restaurantLatitudeMetricValue);
				// 4. Latitude
				String nowLatitude = individualProperties
						.getString("nowLatitude");
				String nowLatitudeCharacterizes = personIndividualName;
				String nowLatitudehasMetric = nowLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						nowLatitude);
				owlModel = operator.assignObjectProperty(owlModel, nowLatitude,
						"characterizes", nowLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel, nowLatitude,
						"hasMetric", nowLatitudehasMetric);
				String officeLatitude = individualProperties
						.getString("officeLatitude");
				String officeLatitudeCharacterizes = personIndividualName;
				String officeLatitudehasMetric = officeLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						officeLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLatitude, "characterizes",
						officeLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLatitude, "hasMetric", officeLatitudehasMetric);
				String meetingRoomLatitude = individualProperties
						.getString("meetingRoomLatitude");
				String meetingRoomLatitudeCharacterizes = personIndividualName;
				String meetingRoomLatitudehasMetric = meetingRoomLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						meetingRoomLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLatitude, "characterizes",
						meetingRoomLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLatitude, "hasMetric",
						meetingRoomLatitudehasMetric);
				String homeLatitude = individualProperties
						.getString("homeLatitude");
				String homeLatitudeCharacterizes = personIndividualName;
				String homeLatitudehasMetric = homeLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						homeLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLatitude, "characterizes",
						homeLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLatitude, "hasMetric", homeLatitudehasMetric);
				String restaurantLatitude = individualProperties
						.getString("restaurantLatitude");
				String restaurantLatitudeCharacterizes = personIndividualName;
				String restaurantLatitudehasMetric = restaurantLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						restaurantLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLatitude, "characterizes",
						restaurantLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLatitude, "hasMetric",
						restaurantLatitudehasMetric);
				// 5. Longitude
				String nowLongitude = individualProperties
						.getString("nowLongitude");
				String nowLongitudeCharacterizes = personIndividualName;
				String nowLongitudehasMetric = nowLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						nowLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						nowLongitude, "characterizes",
						nowLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						nowLongitude, "hasMetric", nowLongitudehasMetric);
				String officeLongitude = individualProperties
						.getString("officeLongitude");
				String officeLongitudeCharacterizes = personIndividualName;
				String officeLongitudehasMetric = officeLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						officeLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLongitude, "characterizes",
						officeLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLongitude, "hasMetric", officeLongitudehasMetric);
				String meetingRoomLongitude = individualProperties
						.getString("meetingRoomLongitude");
				String meetingRoomLongitudeCharacterizes = personIndividualName;
				String meetingRoomLongitudehasMetric = meetingRoomLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						meetingRoomLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLongitude, "characterizes",
						meetingRoomLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLongitude, "hasMetric",
						meetingRoomLongitudehasMetric);
				String homeLongitude = individualProperties
						.getString("homeLongitude");
				String homeLongitudeCharacterizes = personIndividualName;
				String homeLongitudehasMetric = homeLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						homeLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLongitude, "characterizes",
						homeLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLongitude, "hasMetric", homeLongitudehasMetric);
				String restaurantLongitude = individualProperties
						.getString("restaurantLongitude");
				String restaurantLongitudeCharacterizes = personIndividualName;
				String restaurantLongitudehasMetric = restaurantLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						restaurantLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLongitude, "characterizes",
						restaurantLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLongitude, "hasMetric",
						restaurantLongitudehasMetric);
				// 7. Relative locations
				String homeRelativeLocation = individualProperties
						.getString("homeRelativeLocation");
				String homeRelativeLocationHasAddress = individualProperties
						.getString("homeRelativeLocationHasAddress");
				String homeRelativeLocationCharacterizes = personIndividualName;
				String homeRelativeLocationHasLatitude = homeLatitude;
				String homeRelativeLocationHasLongitude = homeLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						homeRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "characterizes",
						homeRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						homeRelativeLocation, "hasAddress",
						homeRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "hasLongitude",
						homeRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "hasLatitude",
						homeRelativeLocationHasLatitude);
				String officeRelativeLocation = individualProperties
						.getString("officeRelativeLocation");
				String officeRelativeLocationHasAddress = individualProperties
						.getString("officeRelativeLocationHasAddress");
				String officeRelativeLocationCharacterizes = personIndividualName;
				String officeRelativeLocationHasLatitude = officeLatitude;
				String officeRelativeLocationHasLongitude = officeLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						officeRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "characterizes",
						officeRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						officeRelativeLocation, "hasAddress",
						officeRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "hasLongitude",
						officeRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "hasLatitude",
						officeRelativeLocationHasLatitude);
				String meetingRoomRelativeLocation = individualProperties
						.getString("meetingRoom");
				String meetingRoomRelativeLocationCharacterizes = personIndividualName;
				String meetingRoomRelativeLocationHasLatitude = meetingRoomLatitude;
				String meetingRoomRelativeLocationHasLongitude = meetingRoomLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						meetingRoomRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "characterizes",
						meetingRoomRelativeLocationCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "hasLongitude",
						meetingRoomRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "hasLatitude",
						meetingRoomRelativeLocationHasLatitude);
				String restaurantRelativeLocation = individualProperties
						.getString("restaurantRelativeLocation");
				String restaurantRelativeLocationHasAddress = individualProperties
						.getString("restaurantRelativeLocationHasAddress");
				String restaurantRelativeLocationCharacterizes = personIndividualName;
				String restaurantRelativeLocationHasLatitude = restaurantLatitude;
				String restaurantRelativeLocationHasLongitude = restaurantLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						restaurantRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "characterizes",
						restaurantRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantRelativeLocation, "hasAddress",
						restaurantRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "hasLongitude",
						restaurantRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "hasLatitude",
						restaurantRelativeLocationHasLatitude);
				// 8. Presence
				String homePresence = individualProperties
						.getString("homePresence");
				String homePresenceCharacterizes = personIndividualName;
				String homePresenceHasLocation = homeRelativeLocation;
				String homePresenceHasAvailability = this.individualProperties
						.getString("homeAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						homePresence);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "characterizes",
						homePresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "hasLocation", homePresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "hasAvailability",
						homePresenceHasAvailability);
				String officePresence = individualProperties
						.getString("officePresence");
				String officePresenceCharacterizes = personIndividualName;
				String officePresenceHasLocation = officeRelativeLocation;
				String officePresenceHasAvailability = this.individualProperties
						.getString("officeAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						officePresence);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "characterizes",
						officePresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "hasLocation",
						officePresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "hasAvailability",
						officePresenceHasAvailability);
				String meetingRoomPresence = individualProperties
						.getString("meetingRoomPresence");
				String meetingRoomPresenceCharacterizes = personIndividualName;
				String meetingRoomPresenceHasLocation = meetingRoomRelativeLocation;
				String meetingRoomPresenceHasAvailability = this.individualProperties
						.getString("meetingRoomAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						meetingRoomPresence);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "characterizes",
						meetingRoomPresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "hasLocation",
						meetingRoomPresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "hasAvailability",
						meetingRoomPresenceHasAvailability);
			} else if (type.equals("RelativeLocation")) {
				System.out.println("RelativeLocation sensor added");
				// adds information about only the Relative location meaning
				// that no information
				// about the current location is received from this sensor
				// only the coordinates for the relative locations that are
				// defined will be added in
				// the owl model
				// to be able to reason about presence and availability -
				// another sensor that informs
				// the theReasoner about the current location
				// (now_absolute_long/lat) is needed...

				// Metrics
				String homeLongitudeMetric = individualProperties
						.getString("homeLongitudeMetric");
				String homeLongitudeMetricType = individualProperties
						.getString("homeLongitudeMetricType");
				String homeLongitudeMetricUnit = individualProperties
						.getString("homeLongitudeMetricUnit");
				String homeLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						homeLongitudeMetric);
				owlModel = operator
						.assignDataProperty(owlModel, homeLongitudeMetric,
								"hasType", homeLongitudeMetricType);
				owlModel = operator
						.assignDataProperty(owlModel, homeLongitudeMetric,
								"hasUnit", homeLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						homeLongitudeMetric, "hasValue",
						homeLongitudeMetricValue);
				String officeLongitudeMetric = individualProperties
						.getString("officeLongitudeMetric");
				String officeLongitudeMetricType = individualProperties
						.getString("officeLongitudeMetricType");
				String officeLongitudeMetricUnit = individualProperties
						.getString("officeLongitudeMetricUnit");
				String officeLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						officeLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasType",
						officeLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasUnit",
						officeLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						officeLongitudeMetric, "hasValue",
						officeLongitudeMetricValue);
				String meetingRoomLongitudeMetric = individualProperties
						.getString("meetingRoomLongitudeMetric");
				String meetingRoomLongitudeMetricType = individualProperties
						.getString("meetingRoomLongitudeMetricType");
				String meetingRoomLongitudeMetricUnit = individualProperties
						.getString("meetingRoomLongitudeMetricUnit");
				String meetingRoomLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						meetingRoomLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasType",
						meetingRoomLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasUnit",
						meetingRoomLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLongitudeMetric, "hasValue",
						meetingRoomLongitudeMetricValue);
				String restaurantLongitudeMetric = individualProperties
						.getString("restaurantLongitudeMetric");
				String restaurantLongitudeMetricType = individualProperties
						.getString("restaurantLongitudeMetricType");
				String restaurantLongitudeMetricUnit = individualProperties
						.getString("restaurantLongitudeMetricUnit");
				String restaurantLongitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						restaurantLongitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasType",
						restaurantLongitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasUnit",
						restaurantLongitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLongitudeMetric, "hasValue",
						restaurantLongitudeMetricValue);
				String homeLatitudeMetric = individualProperties
						.getString("homeLatitudeMetric");
				String homeLatitudeMetricType = individualProperties
						.getString("homeLatitudeMetricType");
				String homeLatitudeMetricUnit = individualProperties
						.getString("homeLatitudeMetricUnit");
				String homeLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						homeLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						homeLatitudeMetric, "hasType", homeLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						homeLatitudeMetric, "hasUnit", homeLatitudeMetricUnit);
				owlModel = operator
						.assignDataProperty(owlModel, homeLatitudeMetric,
								"hasValue", homeLatitudeMetricValue);
				String officeLatitudeMetric = individualProperties
						.getString("officeLatitudeMetric");
				String officeLatitudeMetricType = individualProperties
						.getString("officeLatitudeMetricType");
				String officeLatitudeMetricUnit = individualProperties
						.getString("officeLatitudeMetricUnit");
				String officeLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						officeLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasType",
						officeLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasUnit",
						officeLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						officeLatitudeMetric, "hasValue",
						officeLatitudeMetricValue);
				String meetingRoomLatitudeMetric = individualProperties
						.getString("meetingRoomLatitudeMetric");
				String meetingRoomLatitudeMetricType = individualProperties
						.getString("meetingRoomLatitudeMetricType");
				String meetingRoomLatitudeMetricUnit = individualProperties
						.getString("meetingRoomLatitudeMetricUnit");
				String meetingRoomLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						meetingRoomLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasType",
						meetingRoomLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasUnit",
						meetingRoomLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						meetingRoomLatitudeMetric, "hasValue",
						meetingRoomLatitudeMetricValue);
				String restaurantLatitudeMetric = individualProperties
						.getString("restaurantLatitudeMetric");
				String restaurantLatitudeMetricType = individualProperties
						.getString("restaurantLatitudeMetricType");
				String restaurantLatitudeMetricUnit = individualProperties
						.getString("restaurantLatitudeMetricUnit");
				String restaurantLatitudeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						restaurantLatitudeMetric);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasType",
						restaurantLatitudeMetricType);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasUnit",
						restaurantLatitudeMetricUnit);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantLatitudeMetric, "hasValue",
						restaurantLatitudeMetricValue);
				// 4. Latitude
				String officeLatitude = individualProperties
						.getString("officeLatitude");
				String officeLatitudeCharacterizes = personIndividualName;
				String officeLatitudehasMetric = officeLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						officeLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLatitude, "characterizes",
						officeLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLatitude, "hasMetric", officeLatitudehasMetric);
				String meetingRoomLatitude = individualProperties
						.getString("meetingRoomLatitude");
				String meetingRoomLatitudeCharacterizes = personIndividualName;
				String meetingRoomLatitudehasMetric = meetingRoomLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						meetingRoomLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLatitude, "characterizes",
						meetingRoomLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLatitude, "hasMetric",
						meetingRoomLatitudehasMetric);
				String homeLatitude = individualProperties
						.getString("homeLatitude");
				String homeLatitudeCharacterizes = personIndividualName;
				String homeLatitudehasMetric = homeLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						homeLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLatitude, "characterizes",
						homeLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLatitude, "hasMetric", homeLatitudehasMetric);
				String restaurantLatitude = individualProperties
						.getString("restaurantLatitude");
				String restaurantLatitudeCharacterizes = personIndividualName;
				String restaurantLatitudehasMetric = restaurantLatitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Latitude",
						restaurantLatitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLatitude, "characterizes",
						restaurantLatitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLatitude, "hasMetric",
						restaurantLatitudehasMetric);
				// 5. Longitude
				String officeLongitude = individualProperties
						.getString("officeLongitude");
				String officeLongitudeCharacterizes = personIndividualName;
				String officeLongitudehasMetric = officeLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						officeLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLongitude, "characterizes",
						officeLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officeLongitude, "hasMetric", officeLongitudehasMetric);
				String meetingRoomLongitude = individualProperties
						.getString("meetingRoomLongitude");
				String meetingRoomLongitudeCharacterizes = personIndividualName;
				String meetingRoomLongitudehasMetric = meetingRoomLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						meetingRoomLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLongitude, "characterizes",
						meetingRoomLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomLongitude, "hasMetric",
						meetingRoomLongitudehasMetric);
				String homeLongitude = individualProperties
						.getString("homeLongitude");
				String homeLongitudeCharacterizes = personIndividualName;
				String homeLongitudehasMetric = homeLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						homeLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLongitude, "characterizes",
						homeLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homeLongitude, "hasMetric", homeLongitudehasMetric);
				String restaurantLongitude = individualProperties
						.getString("restaurantLongitude");
				String restaurantLongitudeCharacterizes = personIndividualName;
				String restaurantLongitudehasMetric = restaurantLongitudeMetric;
				owlModel = operator.assignIndividual(owlModel, "Longitude",
						restaurantLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLongitude, "characterizes",
						restaurantLongitudeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantLongitude, "hasMetric",
						restaurantLongitudehasMetric);
				// 7. Relative locations
				String homeRelativeLocation = individualProperties
						.getString("homeRelativeLocation");
				String homeRelativeLocationHasAddress = "undefinedValue";
				String homeRelativeLocationCharacterizes = personIndividualName;
				String homeRelativeLocationHasLatitude = homeLatitude;
				String homeRelativeLocationHasLongitude = homeLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						homeRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "characterizes",
						homeRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						homeRelativeLocation, "hasAddress",
						homeRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "hasLongitude",
						homeRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						homeRelativeLocation, "hasLatitude",
						homeRelativeLocationHasLatitude);
				String officeRelativeLocation = individualProperties
						.getString("officeRelativeLocation");
				String officeRelativeLocationHasAddress = "undefinedValue";
				String officeRelativeLocationCharacterizes = personIndividualName;
				String officeRelativeLocationHasLatitude = officeLatitude;
				String officeRelativeLocationHasLongitude = officeLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						officeRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "characterizes",
						officeRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						officeRelativeLocation, "hasAddress",
						officeRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "hasLongitude",
						officeRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						officeRelativeLocation, "hasLatitude",
						officeRelativeLocationHasLatitude);
				String meetingRoomRelativeLocation = individualProperties
						.getString("meetingRoom");
				String meetingRoomRelativeLocationCharacterizes = personIndividualName;
				String meetingRoomRelativeLocationHasLatitude = meetingRoomLatitude;
				String meetingRoomRelativeLocationHasLongitude = meetingRoomLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						meetingRoomRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "characterizes",
						meetingRoomRelativeLocationCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "hasLongitude",
						meetingRoomRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomRelativeLocation, "hasLatitude",
						meetingRoomRelativeLocationHasLatitude);
				String restaurantRelativeLocation = individualProperties
						.getString("restaurantRelativeLocation");
				String restaurantRelativeLocationHasAddress = "undefinedValue";
				String restaurantRelativeLocationCharacterizes = personIndividualName;
				String restaurantRelativeLocationHasLatitude = restaurantLatitude;
				String restaurantRelativeLocationHasLongitude = restaurantLongitude;
				owlModel = operator.assignIndividual(owlModel, "Relative",
						restaurantRelativeLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "characterizes",
						restaurantRelativeLocationCharacterizes);
				owlModel = operator.assignDataProperty(owlModel,
						restaurantRelativeLocation, "hasAddress",
						restaurantRelativeLocationHasAddress);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "hasLongitude",
						restaurantRelativeLocationHasLongitude);
				owlModel = operator.assignObjectProperty(owlModel,
						restaurantRelativeLocation, "hasLatitude",
						restaurantRelativeLocationHasLatitude);
				// 8. Presence
				String homePresence = individualProperties
						.getString("homePresence");
				String homePresenceCharacterizes = personIndividualName;
				String homePresenceHasLocation = homeRelativeLocation;
				String homePresenceHasAvailability = this.individualProperties
						.getString("homeAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						homePresence);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "characterizes",
						homePresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "hasLocation", homePresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						homePresence, "hasAvailability",
						homePresenceHasAvailability);
				String officePresence = individualProperties
						.getString("officePresence");
				String officePresenceCharacterizes = personIndividualName;
				String officePresenceHasLocation = officeRelativeLocation;
				String officePresenceHasAvailability = this.individualProperties
						.getString("officeAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						officePresence);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "characterizes",
						officePresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "hasLocation",
						officePresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						officePresence, "hasAvailability",
						officePresenceHasAvailability);
				String meetingRoomPresence = individualProperties
						.getString("meetingRoomPresence");
				String meetingRoomPresenceCharacterizes = personIndividualName;
				String meetingRoomPresenceHasLocation = meetingRoomRelativeLocation;
				String meetingRoomPresenceHasAvailability = this.individualProperties
						.getString("meetingRoomAvailability");
				owlModel = operator.assignIndividual(owlModel, "Presence",
						meetingRoomPresence);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "characterizes",
						meetingRoomPresenceCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "hasLocation",
						meetingRoomPresenceHasLocation);
				owlModel = operator.assignObjectProperty(owlModel,
						meetingRoomPresence, "hasAvailability",
						meetingRoomPresenceHasAvailability);
			} else if (type.equals("Pressure")) {
				// adds information about the Pressure
			} else if (type.equals("LuxConditions")) {
				// adds information about the Lux Conditions
			} else if (type.equals("Temperature")) {
				// adds information about the Temperature
			} else if (type.equals("Humidity")) {
				// adds information about the Humidity
			} else if (type.equals("Time")) {
				// adds information about the Time
				System.out.println("Time sensor added");
				// Metric
				String nowTimeMetric = individualProperties
						.getString("nowTimeMetric");
				String nowTimeMetricType = individualProperties
						.getString("nowTimeMetricType");
				String nowTimeMetricUnit = individualProperties
						.getString("nowTimeMetricUnit");
				String nowTimeMetricValue = "undefinedValue";
				owlModel = operator.assignIndividual(owlModel, "Metric",
						nowTimeMetric);
				owlModel = operator.assignDataProperty(owlModel, nowTimeMetric,
						"hasType", nowTimeMetricType);
				owlModel = operator.assignDataProperty(owlModel, nowTimeMetric,
						"hasUnit", nowTimeMetricUnit);
				this.owlModel = this.operator.assignDataProperty(this.owlModel,
						nowTimeMetric, "hasValue", nowTimeMetricValue);
				// 3. Time
				String nowTime = individualProperties.getString("nowTime");
				String nowTimeCharacterizes = personIndividualName;
				String nowTimeHasMetric = nowTimeMetric;
				owlModel = operator.assignIndividual(owlModel, "Time", nowTime);
				owlModel = operator.assignObjectProperty(owlModel, nowTime,
						"characterizes", nowTimeCharacterizes);
				owlModel = operator.assignObjectProperty(owlModel, nowTime,
						"hasMetric", nowTimeHasMetric);
			} else if (type.equals("Noise")) {
				// adds information about the Noise
			} else if (type.equals("InstantStatus")) {
				// adds information about the Instant status
			} else if (type.equals("ScheduledActivity")) {
				// adds information about the Scheduled Activities
				System.out.println("ScheduledActivity sensor added");
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void store() {
		operator.storeUserInformation(owlModel);
	}

	public void updateInformation(Object contextInformation) {
		Object informationClass = contextInformation.getClass();
		if (contextInformation instanceof ScheduledActivity) {
			this
					.updateScheduledActivity((ScheduledActivity) contextInformation);

		} else if (contextInformation instanceof Location) {
			this.updateLocation((Location) contextInformation);
		} else if (contextInformation instanceof Time) {
			this.updateTime((Time) contextInformation);
		} else if (contextInformation instanceof Relative) {
			this.updateRelative((Relative) contextInformation);
		} else if (contextInformation instanceof InstantStatus) {
			this.updateInstantStatus((InstantStatus) contextInformation);
		} else if (contextInformation instanceof Noise) {
			this.updateNoise((Noise) contextInformation);
		} else if (contextInformation instanceof Pressure) {
			this.updatePressure((Pressure) contextInformation);
		} else if (contextInformation instanceof LuxConditions) {
			this.updateLuxConditions((LuxConditions) contextInformation);
		} else if (contextInformation instanceof Temperature) {
			this.updateTemperature((Temperature) contextInformation);
		} else if (contextInformation instanceof Humidity) {
			this.updateHumidity((Humidity) contextInformation);
		} else {
			Exception e = new Exception(
					"The ContextInformation was not recognized by the ContextReasoner");
			e.printStackTrace();
		}
	}

	private void updateHumidity(Humidity humidity) {
		// TODO Auto-generated method stub

	}

	private void updateTemperature(Temperature temperature) {
		// TODO Auto-generated method stub

	}

	private void updateLuxConditions(LuxConditions conditions) {
		// TODO Auto-generated method stub

	}

	private void updatePressure(Pressure pressure) {
		// TODO Auto-generated method stub

	}

	private void updateNoise(Noise noise) {
		// TODO Auto-generated method stub

	}

	private void updateInstantStatus(InstantStatus status) {
		// TODO Auto-generated method stub

	}

	private void updateRelative(Relative relative) {
		// as for the demonstrator, the location sensor will not be able to add
		// information about the
		// relative locations such as address, altitude, longitude and latitude
		String name = relative.getName();
		String altitudeName = name + "Altitude";
		String longitudeName = name + "Longitude";
		String latitudeName = name + "Latitude";

		String altitudeMetricName = altitudeName + "Metric";
		String longitudeMetricName = longitudeName + "Metric";
		String latitudeMetricName = latitudeName + "Metric";

		String address = ((ArrayList) relative.getHasAddress()).get(0)
				.toString();
		String altitude = ((ArrayList) relative.getHasAltitude()).get(0)
				.toString();
		if (altitude == null) {
			altitude = "undefinedValue";
		}
		String latitude = ((ArrayList) relative.getHasLatitude()).get(0)
				.toString();
		String longitude = ((ArrayList) relative.getHasLongitude()).get(0)
				.toString();
		// add all of this to the model too...
		// Metrics
		this.owlModel = this.operator.assignIndividual(this.owlModel, "Metric",
				altitudeMetricName);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				altitudeMetricName, "hasType", "double");
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				altitudeMetricName, "hasUnit", "meter");
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				altitudeMetricName, "hasValue", altitude);
		this.owlModel = this.operator.assignIndividual(this.owlModel, "Metric",
				longitudeMetricName);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				longitudeMetricName, "hasType", "float");
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				longitudeMetricName, "hasUnit", "DegreeDecimal");
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				longitudeMetricName, "hasValue", longitude);
		this.owlModel = this.operator.assignIndividual(this.owlModel, "Metric",
				latitudeMetricName);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				latitudeMetricName, "hasType", "float");
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				latitudeMetricName, "hasUnit", "DegreeDecimal");
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				latitudeMetricName, "hasValue", latitude);
		// Altitude
		this.owlModel = this.operator.assignIndividual(this.owlModel,
				"Altitude", altitudeName);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				altitudeName, "hasMetric", altitudeMetricName);
		// Longitude
		this.owlModel = this.operator.assignIndividual(this.owlModel,
				"Longitude", longitudeName);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				longitudeName, "hasMetric", longitudeMetricName);
		// Latitude
		this.owlModel = this.operator.assignIndividual(this.owlModel,
				"Altitude", latitudeName);
		this.owlModel = this.operator.assignDataProperty(this.owlModel,
				latitudeName, "hasMetric", latitudeMetricName);
		// Relative
		this.owlModel = this.operator.assignIndividual(this.owlModel,
				"Relative", name);
		this.owlModel = this.operator.assignDataProperty(this.owlModel, name,
				"hasAddress", address);
		this.owlModel = this.operator.assignObjectProperty(this.owlModel, name,
				"hasAltitude", altitudeName);
		this.owlModel = this.operator.assignObjectProperty(this.owlModel, name,
				"hasLongitude", longitudeName);
		this.owlModel = this.operator.assignObjectProperty(this.owlModel, name,
				"hasLatitude", latitudeName);
	}

	private void updateLocation(Location location) {
		// update the model based on the information about the newly received
		// location
		System.out
				.println("updateLocation()!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		Longitude longitude = location.getLongitude();
		Latitude latitude = location.getLatitude();

		Metric nowLongMetric = (Metric) ((ArrayList) longitude.getHasMetric())
				.get(0);
		Metric nowLatMetric = (Metric) ((ArrayList) latitude.getHasMetric())
				.get(0);
		String nowLongMetricValue = nowLongMetric.getHasValue();
		String nowLatMetricValue = nowLatMetric.getHasValue();

		String nowLongMetricName = this.individualProperties
				.getString("nowLongitudeMetric");
		String nowLatMetricName = this.individualProperties
				.getString("nowLatitudeMetric");
		String nowLongModelledValue = this.operator.getDatatypeValue(
				this.owlModel, nowLongMetricName, "hasValue");
		String nowLatModelledValue = this.operator.getDatatypeValue(
				this.owlModel, nowLatMetricName, "hasValue");
		// System.out.println("Modelled Longitude=" + nowLongModelledValue + "
		// VS sensed longitude=" + nowLongMetricValue);
		// System.out.println("Modelled Latitude=" + nowLatModelledValue + " VS
		// sensed latitude=" + nowLatMetricValue);
		// update the sensed latitude and longitude if they are different from
		// the modelled ones
		if ((!(nowLongModelledValue.equals(nowLongMetricValue)))
				|| (!(nowLatModelledValue.equals(nowLatMetricValue)))) {
			this.owlModel = this.operator.modifyDataProperty(this.owlModel,
					nowLongMetricName, "hasValue", nowLongMetricValue);
			this.owlModel = this.operator.modifyDataProperty(this.owlModel,
					nowLatMetricName, "hasValue", nowLatMetricValue);

			// System.out.println("The location was
			// new!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");

			// deduce what the relative location is:
			// the relative location will not be updated in the model, since the
			// model only contains
			// information about what coordinates the different relative
			// locations have. The mapping
			// is done in the code, and only kept locally/virtually to deduce
			// the secondary information
			// that is to be forwarded to the ContextEngine...
			if ((!(nowLatMetricValue.equals("undefined")))
					&& (!(nowLongMetricValue.equals("undefined")))) {
				// System.out.println("The location was NOT undefined");
				String officeLat = this.individualProperties
						.getString("officeLatitudeMetricValue");
				String homeLat = this.individualProperties
						.getString("homeLatitudeMetricValue");
				String meetingRoomLat = this.individualProperties
						.getString("meetingRoomLatitudeMetricValue");
				String restaurantLat = this.individualProperties
						.getString("restaurantLatitudeMetricValue");
				String officeLong = this.individualProperties
						.getString("officeLongitudeMetricValue");
				String homeLong = this.individualProperties
						.getString("homeLongitudeMetricValue");
				String meetingRoomLong = this.individualProperties
						.getString("meetingRoomLongitudeMetricValue");
				String restaurantLong = this.individualProperties
						.getString("restaurantLongitudeMetricValue");
				if (officeLat.equals(nowLatMetricValue)
						&& officeLong.equals(nowLongMetricValue)) {
					// System.out.println("officeLat.equals(nowLatMetricValue)!!!!!!!!!!!!!");

					// location is at the office
					// System.out.println("Location=Office");
					this.location = "office";
					// update this in the model

				} else if (homeLat.equals(nowLatMetricValue)
						&& homeLong.equals(nowLongMetricValue)) {
					// System.out.println("homeLat.equals(nowLatMetricValue)!!!!!!!!!!!!!");

					// the home location in the map corresponds to undefined in
					// the service logic to simplify
					// the simulation
					// location is at home
					// System.out.println("Location=Home");
					// this.location = "home";
					this.location = "nil";

				} else if (meetingRoomLat.equals(nowLatMetricValue)
						&& meetingRoomLong.equals(nowLongMetricValue)) {
					// System.out.println("meetingRoomLat.equals(nowLatMetricValue)!!!!!!!!!!!!!");

					// location is at the meeting room
					// System.out.println("Location=Meeting room");
					this.location = "meetingRoom";

				} else if (restaurantLat.equals(nowLatMetricValue)
						&& restaurantLong.equals(nowLongMetricValue)) {
					// System.out.println("restaurantLat.equals(nowLatMetricValue)!!!!!!!!!!!!!");

					// location is at the restaurant
					// System.out.println("Location=Restaurant");
					this.location = "restaurant";

				} else {
					System.out.println("Location=undefined");
					this.location = "nil";
				}
			}// end if
			else {
				// nowLatitudeMetricValue has not been set yet
				// this should not happen
				System.out
						.println("nowLatitudeMetricValue has not been set yet");
				this.location = "nil";
			}

			// retrieve the time
			String nowTimeMetric = this.individualProperties
					.getString("nowTimeMetric");
			String timeMetricValue = this.operator.getDatatypeValue(
					this.owlModel, nowTimeMetric, "hasValue");
			if (!timeMetricValue.equals("undefined")) {
				// retrieve the scheduled activities
				ArrayList scheduledActivities = this.operator.getIndividuals(
						this.owlModel, "ScheduledActivity");

				if (!(scheduledActivities.get(0).toString().equals("undefined"))) {
					// there exist activities. The question now is whether the
					// activities are scheduled now or later...
					for (int i = 0; i < scheduledActivities.size(); i++) {
						OWLIndividual individual = (OWLIndividual) scheduledActivities
								.get(i);

						// get all the scheduledActivities and compare to see if
						// there
						// is an
						// activity that matches the current time...
						Object beginTimeObject = this.operator
								.getObjectPropertyValue(this.owlModel,
										individual.getName(), "beginsAtTime");
						if (!(beginTimeObject.toString().equals("undefined"))) {
							OWLIndividual beginTime = (OWLIndividual) beginTimeObject;
							Object beginTimeMetricObject = this.operator
									.getObjectPropertyValue(this.owlModel,
											beginTime.getName(), "hasMetric");
							if (!(beginTimeMetricObject.toString()
									.equals("undefined"))) {
								// assuming that if the beginTime is deifned
								// then the endTime is also defined...
								OWLIndividual beginTimeMetric = (OWLIndividual) beginTimeMetricObject;
								String beginTimeMetricValue = this.operator
										.getDatatypeValue(this.owlModel,
												beginTimeMetric.getName(),
												"hasValue").toString();
								OWLIndividual endTime = (OWLIndividual) this.operator
										.getObjectPropertyValue(this.owlModel,
												individual.getName(),
												"isDoneAtTime");
								OWLIndividual endTimeMetric = (OWLIndividual) this.operator
										.getObjectPropertyValue(this.owlModel,
												endTime.getName(), "hasMetric");
								String endTimeMetricValue = this.operator
										.getDatatypeValue(this.owlModel,
												endTimeMetric.getName(),
												"hasValue").toString();
								// System.out.println("The time is now: "
								// + timeMetricValue);
								// System.out.println("Activity " + i + " begins
								// at: "
								// + beginTimeMetricValue);
								// System.out.println("Activity " + i + " ends
								// at: "
								// + endTimeMetricValue);
								if (timeMetricValue
										.equals(beginTimeMetricValue)
										&& timeMetricValue
												.equals(endTimeMetricValue)) {
									// this means that the time is officeTime
									// since the activity ends at the
									// same time as it begins...
									this.time = "officeTime";
									this.activityRegistered = false;
									break;
								} // end if
								if (timeMetricValue
										.equals(beginTimeMetricValue)) {
									// the activity starts and the time goes
									// from
									// OfficeTime to
									// ActivityTime
									// ergo there is an activity that should
									// take place
									// now
									// System.out.println("Time=ActivityTime");
									this.time = "activityTime";
									this.activityRegistered = true;
									break;
								}// end if
								else if (timeMetricValue
										.equals(endTimeMetricValue)) {
									// the activity is about to end and goes
									// from
									// ActivityTime to
									// officeTime
									// ergo there is no activity that should
									// take place
									// now

									// System.out.println("Time=OfficeTime");
									this.time = "officeTime";
									this.activityRegistered = false;
									break;
								}// end if
								else {
									// do nothing since the time is undefined...
									// System.out.println("Time=undefined!!!!!!");
									this.time = "nil";
									this.activityRegistered = false;
								}// end else
							}// end if
							else {
								// this beginTime does not have a
								// beginTimeMetric
								// specified
								// this should not happen
								System.out
										.println("This beginTime does not have a beginTimeMetric");
							}// end else
						}// end if
						else {
							// the begin time is undefined
							// this should not happen
							System.out
									.println("This scheduled activity does not have a beginTime");
						}
					} // end for
				}// end if
				else {
					// there does not exist any scheduled activities
					System.out.println("There does not exist any activities");
					this.activityRegistered = false;
					this.time = "nil";
				}// end else

			} // end if
			else {
				// time has not been set and so there is no reason to check the
				// activities
				System.out.println("time has not been set yet");
				this.time = "nil";
				this.activityRegistered = false;
			} // end else

			// now deduce what the new secondary information is...
			RuleReasoner reasoningRule = new RuleReasoner();
			reasoningRule.inferRule(this.time, this.location,
					this.activityRegistered);
			String timeTest = reasoningRule.getTime();
			String locationTest = reasoningRule.getLocation();
			boolean scheduledActivity = reasoningRule.hasScheduledActivity();
			String deducedActivity = reasoningRule.getDeducedActivity();
			String homeAvailability = reasoningRule.getHomeAvailability();
			String officeAvailability = reasoningRule.getOfficeAvailability();
			String meetingRoomAvailability = reasoningRule
					.getMeetingRoomAvailability();

			System.out.println("The observed time is : " + this.time);
			System.out.println("The observed location is : " + this.location);

			System.out.println("The reasoned time is : " + timeTest);
			System.out.println("The reasoned location is : " + locationTest);
			/*
			 * System.out.println("There is a scheduled activity at the moment : " +
			 * scheduledActivity); System.out.println("The deduced activity is : " +
			 * deducedActivity); System.out.println("The home availability is : " +
			 * homeAvailability); System.out .println("The office availability
			 * is : " + officeAvailability); System.out.println("The meeting
			 * room availability is : " + meetingRoomAvailability);
			 */

			// now polulate the owlmodel with this information and send it to
			// the UserAgent
			String deducedActivityName = this.individualProperties
					.getString("nowDeducedActivity");
			// System.out.println("DeducedActivityName " + deducedActivityName);
			// System.out.println("DeducedActivity " + deducedActivity);
			this.owlModel = this.operator.modifyDataProperty(this.owlModel,
					deducedActivityName, "hasDescription", deducedActivity);

			String homeAvailabilityMetricName = this.individualProperties
					.getString("homeAvailabilityMetric");
			String officeAvailabilityMetricName = this.individualProperties
					.getString("officeAvailabilityMetric");
			String meetingRoomAvailabilityMetricName = this.individualProperties
					.getString("meetingRoomAvailabilityMetric");
			this.owlModel = this.operator.modifyDataProperty(this.owlModel,
					homeAvailabilityMetricName, "hasValue", homeAvailability);
			this.owlModel = this.operator.modifyDataProperty(this.owlModel,
					officeAvailabilityMetricName, "hasValue",
					officeAvailability);
			this.owlModel = this.operator.modifyDataProperty(this.owlModel,
					meetingRoomAvailabilityMetricName, "hasValue",
					meetingRoomAvailability);
			this.myContextEngine.sendMessage(new ContextUpdate(deducedActivity,
					homeAvailability, officeAvailability,
					meetingRoomAvailability), this.myContextEngine
					.getMyActorAddress());

			System.out.println("Done with update Location!!!");
		} // end if
		else {
			// the sensed latitude and longitude is the same as the one that is
			// modelled and so nothing will be done
			System.out
					.println("the sensed latitude and longitude is the same as the one that is modelled and so nothing will be done");
		} // end else

	}

	private void updateScheduledActivity(ScheduledActivity scheduledActivity) {
		// retrieve the name of the activity
		String activityName = scheduledActivity.getName();
		// check whether the newly received information is new
		// that is - if the information received is already contained in the
		// model.
		// if it is not new - ignore the newly received info
		// if it is new - update the model and send an update to UserAgent...
		OWLIndividual owlIndividual = owlModel.getOWLIndividual(activityName);
		if (owlIndividual == null) {
			// System.out.println("updateScheduledActivity()!!!");
			// the activity is new and does not exist in the OWLModel

			// retireve the time at which the activity starts
			Time beginTime = (Time) ((ArrayList) scheduledActivity
					.getBeginsAtTime()).get(0);
			String beginTimeName = beginTime.getName();
			Metric beginTimeMetric = (Metric) (((ArrayList) beginTime
					.getHasMetric()).get(0));
			String beginTimeMetricName = beginTimeMetric.getName();
			String beginTimeMetricType = beginTimeMetric.getHasType();
			String beginTimeMetricUnit = beginTimeMetric.getHasUnit();
			String beginTimeMetricValue = beginTimeMetric.getHasValue();
			// retireve the time at which the activity ends
			Time endTime = (Time) ((ArrayList) scheduledActivity
					.getIsDoneAtTime()).get(0);
			String endTimeName = endTime.getName();
			Metric endTimeMetric = (Metric) (((ArrayList) endTime
					.getHasMetric()).get(0));
			String endTimeMetricName = endTimeMetric.getName();
			String endTimeMetricType = endTimeMetric.getHasType();
			String endTimeMetricUnit = endTimeMetric.getHasUnit();
			String endTimeMetricValue = endTimeMetric.getHasValue();
			// retireve the summary of the activity
			ArrayList summaryCollection = (ArrayList) scheduledActivity
					.getHasSummary();
			String summary = (String) summaryCollection.get(0);
			// retireve the description of the activity
			ArrayList descriptionCollection = (ArrayList) scheduledActivity
					.getHasDescription();
			String description = (String) descriptionCollection.get(0);
			// retireve the category of the activity
			ArrayList categoryCollection = (ArrayList) scheduledActivity
					.getHasCategory();
			String category = categoryCollection.get(0).toString();
			// retireve the priority of the activity
			ArrayList priorityCollection = (ArrayList) scheduledActivity
					.getHasPriority();
			String priority = (String) priorityCollection.get(0);
			// retireve the participants of the activity
			// could have made it possible to have activity with more than 1
			// person
			// but for the demonstrator, this will not be done
			ArrayList participantCollection = (ArrayList) scheduledActivity
					.getHasParticipant();
			PersonEntity participant = (PersonEntity) participantCollection
					.get(0);
			String personName = participant.getName();
			// retireve the (Relative) location of the activity
			ArrayList locationCollection = (ArrayList) scheduledActivity
					.getHasLocation();
			Relative relativeLocation = (Relative) locationCollection.get(0);
			String locationName = relativeLocation.getName();

			// String activityNumber = activityName.substring(8);

			// assign the Scheduled Activity to the owl model
			this.owlModel = this.operator.assignIndividual(this.owlModel,
					"ScheduledActivity", activityName);
			// assign the time at which the activity starts
			this.owlModel = this.operator.assignIndividual(this.owlModel,
					"Metric", beginTimeMetricName);
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					beginTimeMetricName, "hasType", beginTimeMetricType);
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					beginTimeMetricName, "hasUnit", beginTimeMetricUnit);
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					beginTimeMetricName, "hasValue", beginTimeMetricValue);
			this.owlModel = this.operator.assignIndividual(this.owlModel,
					"Time", beginTimeName);
			this.owlModel = this.operator.assignObjectProperty(this.owlModel,
					beginTimeName, "hasMetric", beginTimeMetricName);
			this.owlModel = this.operator.assignObjectProperty(this.owlModel,
					activityName, "beginsAtTime", beginTimeName);
			// assign the time at which the activity ends
			this.owlModel = this.operator.assignIndividual(this.owlModel,
					"Metric", endTimeMetricName);
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					endTimeMetricName, "hasType", endTimeMetricType);
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					endTimeMetricName, "hasUnit", endTimeMetricUnit);
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					endTimeMetricName, "hasValue", endTimeMetricValue);
			this.owlModel = this.operator.assignIndividual(this.owlModel,
					"Time", endTimeName);
			this.owlModel = this.operator.assignObjectProperty(this.owlModel,
					endTimeName, "hasMetric", endTimeMetricName);
			this.owlModel = this.operator.assignObjectProperty(this.owlModel,
					activityName, "isDoneAtTime", endTimeName);
			// assign the participants of the activity
			if (!this.operator.individualExists(this.owlModel, personName)) {
				// the participant is not yet defined in the model
				this.owlModel = this.operator.assignIndividual(this.owlModel,
						"PersonEntity", personName);
			}
			this.owlModel = this.operator.assignObjectProperty(this.owlModel,
					activityName, "hasParticipant", personName);
			// assign the location of the activity
			// when assigning Object properties, it is important that the range
			// is already
			// defined. when it comes to the location - if it is null, then
			// don't add it in the model
			// since the location is undefined
			if (!locationName.equals("null")) {
				this.owlModel = this.operator.assignObjectProperty(
						this.owlModel, activityName, "hasLocation",
						locationName);
			}
			// assign the summary of the activity
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					activityName, "hasSummary", summary);
			// assign the description of the activity
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					activityName, "hasDescription", description);
			// assign the category of the activity
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					activityName, "hasCategory", category);
			// assign the priority of the activity
			this.owlModel = this.operator.assignDataProperty(this.owlModel,
					activityName, "hasPriority", priority);

			// start timers that notifies this when the activity shall begin and
			// end
			// System.out.println("start the timers...");
			Time beginTimeObject = new Time();
			Metric beginTimeObjectMetric = new Metric();
			beginTimeObjectMetric.setHasValue(beginTimeMetricValue);
			beginTimeObject.addHasMetric(beginTimeObjectMetric);

			this.timeSensor.fetch(beginTimeObject);

			Time endTimeObject = new Time();
			Metric endTimeObjectMetric = new Metric();
			endTimeObjectMetric.setHasValue(endTimeMetricValue);
			endTimeObject.addHasMetric(endTimeObjectMetric);
			this.timeSensor.fetch(endTimeObject);

		} // end if*/
	}

	private void updateTime(Time time) {
		// update the model based on the information about the newly received
		// time
		Metric timeMetric = ((Metric) ((ArrayList) time.getHasMetric()).get(0));
		String timeMetricValue = timeMetric.getHasValue();
		String nowTimeMetric = this.individualProperties
				.getString("nowTimeMetric");
		System.out.println("updateTime()");
		// System.out.println("The time is: " + timeMetricValue);
		this.owlModel = this.operator.modifyDataProperty(this.owlModel,
				nowTimeMetric, "hasValue", timeMetricValue);

		// retrieve the location and the scheduled activities to be able to
		// deduce what the secondary
		// context information is
		String nowLongMetric = this.individualProperties
				.getString("nowLongitudeMetric");
		String nowLatMetric = this.individualProperties
				.getString("nowLatitudeMetric");
		String nowLongMetricValue = this.operator.getDatatypeValue(
				this.owlModel, nowLongMetric, "hasValue");
		String nowLatMetricValue = this.operator.getDatatypeValue(
				this.owlModel, nowLatMetric, "hasValue");
		if (!(nowLatMetricValue.equals("undefined"))) {
			String officeLat = this.individualProperties
					.getString("officeLatitudeMetricValue");
			String homeLat = this.individualProperties
					.getString("homeLatitudeMetricValue");
			String meetingRoomLat = this.individualProperties
					.getString("meetingRoomLatitudeMetricValue");
			String restaurantLat = this.individualProperties
					.getString("restaurantLatitudeMetricValue");
			String officeLong = this.individualProperties
					.getString("officeLongitudeMetricValue");
			String homeLong = this.individualProperties
					.getString("homeLongitudeMetricValue");
			String meetingRoomLong = this.individualProperties
					.getString("meetingRoomLongitudeMetricValue");
			String restaurantLong = this.individualProperties
					.getString("restaurantLongitudeMetricValue");
			if (officeLat.equals(nowLatMetricValue)
					&& officeLong.equals(nowLongMetricValue)) {
				// location is at the office
				// System.out.println("Location=Office");
				this.location = "office";

			} else if (homeLat.equals(nowLatMetricValue)
					&& homeLong.equals(nowLongMetricValue)) {
				// the home location in the map corresponds to undefined in the
				// service logic to simplify
				// the simulation
				// location is at home
				// System.out.println("Location=Home");
				// this.location = "home";
				this.location = "nil";

			} else if (meetingRoomLat.equals(nowLatMetricValue)
					&& meetingRoomLong.equals(nowLongMetricValue)) {
				// location is at the meeting room
				// System.out.println("Location=Meeting room");
				this.location = "meetingRoom";

			} else if (restaurantLat.equals(nowLatMetricValue)
					&& restaurantLong.equals(nowLongMetricValue)) {
				// location is at the restaurant
				// System.out.println("Location=Restaurant");
				this.location = "restaurant";

			} else {
				// System.out.println("Location=undefined");
				this.location = "nil";
			}
		}// end if
		else {
			// nowLatitudeMetricValue has not been set yet
			System.out.println("nowLatitudeMetricValue has not been set yet");
			this.location = "nil";
		}

		ArrayList scheduledActivities = this.operator.getIndividuals(
				this.owlModel, "ScheduledActivity");

		if (!(scheduledActivities.get(0).toString().equals("undefined"))) {
			// there exist activities. The question now is whether the
			// activities are scheduled now or not...
			for (int i = 0; i < scheduledActivities.size(); i++) {
				OWLIndividual individual = (OWLIndividual) scheduledActivities
						.get(i);

				// get all the scheduledActivities and compare to see if there
				// is an
				// activity that matches the current time...
				Object beginTimeObject = this.operator.getObjectPropertyValue(
						this.owlModel, individual.getName(), "beginsAtTime");
				if (!(beginTimeObject.toString().equals("undefined"))) {
					OWLIndividual beginTime = (OWLIndividual) beginTimeObject;
					Object beginTimeMetricObject = this.operator
							.getObjectPropertyValue(this.owlModel, beginTime
									.getName(), "hasMetric");
					if (!(beginTimeMetricObject.toString().equals("undefined"))) {
						OWLIndividual beginTimeMetric = (OWLIndividual) beginTimeMetricObject;
						String beginTimeMetricValue = this.operator
								.getDatatypeValue(this.owlModel,
										beginTimeMetric.getName(), "hasValue")
								.toString();
						OWLIndividual endTime = (OWLIndividual) this.operator
								.getObjectPropertyValue(this.owlModel,
										individual.getName(), "isDoneAtTime");
						OWLIndividual endTimeMetric = (OWLIndividual) this.operator
								.getObjectPropertyValue(this.owlModel, endTime
										.getName(), "hasMetric");
						String endTimeMetricValue = this.operator
								.getDatatypeValue(this.owlModel,
										endTimeMetric.getName(), "hasValue")
								.toString();
						// System.out.println("The time is now: "
						// + timeMetricValue);
						// System.out.println("Activity " + i + " begins at: "
						// + beginTimeMetricValue);
						// System.out.println("Activity " + i + " ends at: "
						// + endTimeMetricValue);
						if (timeMetricValue.equals(beginTimeMetricValue)) {
							// the activity starts and the time goes from
							// OfficeTime to
							// ActivityTime
							// ergo there is an activity that should take place
							// now
							// System.out.println("Time=ActivityTime");
							this.time = "activityTime";
							this.activityRegistered = true;
							if (beginTimeMetricValue.equals(endTimeMetricValue)) {
								// this is the first activity which is for
								// setting the time so that
								// it is defined in the model
								this.time = "officeTime";
								this.activityRegistered = false;
							}// end if
							break;
						}// end if
						else if (timeMetricValue.equals(endTimeMetricValue)) {
							// the activity is about to end and goes from
							// ActivityTime to
							// officeTime
							// ergo there is no activity that should take place
							// now

							// System.out.println("Time=OfficeTime");
							this.time = "officeTime";
							this.activityRegistered = false;
							break;
						}// end if
						else {
							// do nothing since the time is undefined...
							// this shall never happen, because the
							// timeMetricValue is based on the activities
							// begin/end Time
							// System.out.println("Time=undefined, since
							// nowTime!= activityBegin OR nowTime!=
							// activityEnds!!!!!!");
							this.time = "nil";
							this.activityRegistered = false;
						}// end else
					}// end if
					else {
						// this beginTime does not have a beginTimeMetric
						// specified
						System.out
								.println("This beginTime does not have a beginTimeMetric");
					}// end else
				}// end if
				else {
					// the begin time is undefined
					System.out
							.println("This scheduled activity does not have a beginTime");
				}
			} // end for
		}// end if
		else {
			// there does not exist any scheduled activities
			System.out.println("There does not exist any activities");
			// this should never happen!!!!
			this.time = "nil";
			this.activityRegistered = false;
		}// end else

		// now deduce what the new secondary information is...
		RuleReasoner reasoningRule = new RuleReasoner();
		reasoningRule.inferRule(this.time, this.location,
				this.activityRegistered);
		String timeTest = reasoningRule.getTime();
		String locationTest = reasoningRule.getLocation();
		boolean scheduledActivity = reasoningRule.hasScheduledActivity();
		String deducedActivity = reasoningRule.getDeducedActivity();
		String homeAvailability = reasoningRule.getHomeAvailability();
		String officeAvailability = reasoningRule.getOfficeAvailability();
		String meetingRoomAvailability = reasoningRule
				.getMeetingRoomAvailability();

		System.out.println("The observed time is : " + this.time);
		System.out.println("The observed location is : " + this.location);

		System.out.println("The reasoned time is : " + timeTest);
		System.out.println("The reasoned location is : " + locationTest);
		/*
		 * System.out.println("There is a scheduled activity at the moment : " +
		 * scheduledActivity); System.out.println("The deduced activity is : " +
		 * deducedActivity); System.out.println("The home availability is : " +
		 * homeAvailability); System.out .println("The office availability is : " +
		 * officeAvailability); System.out.println("The meeting room
		 * availability is : " + meetingRoomAvailability);
		 */
		// now polulate the owlmodel with this information and send it to the
		// UserAgent
		String deducedActivityName = this.individualProperties
				.getString("nowDeducedActivity");
		// System.out.println("DeducedActivityName " + deducedActivityName);
		// System.out.println("DeducedActivity " + deducedActivity);
		this.owlModel = this.operator.modifyDataProperty(this.owlModel,
				deducedActivityName, "hasDescription", deducedActivity);

		String homeAvailabilityMetricName = this.individualProperties
				.getString("homeAvailabilityMetric");
		String officeAvailabilityMetricName = this.individualProperties
				.getString("officeAvailabilityMetric");
		String meetingRoomAvailabilityMetricName = this.individualProperties
				.getString("meetingRoomAvailabilityMetric");
		this.owlModel = this.operator.modifyDataProperty(this.owlModel,
				homeAvailabilityMetricName, "hasValue", homeAvailability);
		this.owlModel = this.operator.modifyDataProperty(this.owlModel,
				officeAvailabilityMetricName, "hasValue", officeAvailability);
		this.owlModel = this.operator.modifyDataProperty(this.owlModel,
				meetingRoomAvailabilityMetricName, "hasValue",
				meetingRoomAvailability);
		this.myContextEngine.sendMessage(new ContextUpdate(deducedActivity,
				homeAvailability, officeAvailability, meetingRoomAvailability),
				this.myContextEngine.getMyActorAddress());

		System.out.println("Done with update Time!!!");
	}

}
