package no.ntnu.item.contextengine.sensors.locationsensor.locationsensor;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.PropertyResourceBundle;
import java.util.Set;

import no.ntnu.item.contextengine.model.ontologyclasses.Latitude;
import no.ntnu.item.contextengine.model.ontologyclasses.Location;
import no.ntnu.item.contextengine.model.ontologyclasses.Longitude;
import no.ntnu.item.contextengine.model.ontologyclasses.Metric;
import no.ntnu.item.contextengine.model.ontologyclasses.Time;
import no.ntnu.item.contextengine.sensors.locationsensor.signals.LocationSensorRegistered;
import no.ntnu.item.contextengine.sensors.locationsensor.signals.StartFetching;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.FetchLocationReq;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.FetchLocationRes;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.RegisterLocationReqSubscriptions;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.StopLocationReqSubscriptions;
import no.ntnu.item.contextengine.sensors.locationsensor.signals.StartSubscriptions;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.LocationSubscriptionReport;
import no.ntnu.item.contextengine.sensors.locationsensor.signals.StopSubscriptions;
import no.ntnu.item.contextengine.sensors.locationsensor.util.LocationValueObject;
import no.ntnu.item.contextengine.sensors.locationsensor.util.Printer;
import no.ntnu.item.contextengine.sensors.timesensor.TimeReminder;
import no.ntnu.item.contextengine.signals.StartUpContextEngine;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleReleaseMsg;


public class LocationSensorActions extends Object {

	public static void fetch(StartFetching signal, LocationSensorSM asm){
        //send FetchLocationReq (ArrayList users)
		try {
			PropertyResourceBundle myResources = new PropertyResourceBundle(new FileInputStream("defaultContextEngineProperties.properties"));
            // give UserAgent the necessary information it needs
			// - userName, phoneNumber of terminal, status. 
			//   As for the mamCallController 
			//   and mamStatusController, these can be set in init() 
			//   by looking in System actor
			int numberOfSubscribers = new Integer(myResources.getString("numberOfSubscribers")).intValue();
			ArrayList list = new ArrayList();
			for (int i = 1; i <= numberOfSubscribers; i++) {
				list.add(myResources.getString("Subscriber" + i));
			}
			asm.sendMessage(new FetchLocationReq(list), asm.myLocationFeatureAddress);

			//start counting down towards the next time to fetch the location
			asm.hourInterval = signal.hours;
			asm.minuteInterval = signal.minutes;
			asm.secondInterval = signal.seconds;
//			as of for the demonstrator, the second is the only format that will work
			if (asm.hourInterval == 0) {
				if (asm.minuteInterval==0) {
					if (asm.secondInterval !=0 ) {
						new TimeReminder(asm, asm.secondInterval);
					}
					else {
						System.out.println("The second interval was equal to null");
					}
				}
				else {
					System.out.println("The minute interval was NOT equal to null");
				}
			}
			else {
				System.out.println("The hour interval was NOT equal to null");
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void receive(FetchLocationRes signal, LocationSensorSM asm){
		HashMap resultList = signal.resultList;
	    Printer p = new Printer();
	    //p.out("!!!!!!!!!!!!!!!!!!!!!!!!!");
	    //p.out("trying to show results");
	    //p.out("There are " + resultList.size() + " to show");
	    Set mapSet = resultList.keySet();
		Iterator it = mapSet.iterator();
		
		while (it.hasNext()) {
			String key = it.next().toString();
			LocationValueObject value = (LocationValueObject)resultList.get(key);
			float latitude = value.getLatitude();
			float longitude = value.getLongitude();
			
			//p.out("!!!!!!!!!Location Report!!!!!!!!!");
			//p.out("User: " + key + " is located at: ");
			//p.out("Latitude: " + latitude);
			//p.out("Longitude: " + longitude);
			//p.out("");
            //validate is not needed since the sensor is only simulated
			ArrayList locationInformation = new ArrayList();
			locationInformation.add(new Integer((int) latitude));
			locationInformation.add(new Integer((int) longitude));
			Location formattedLocationInformation = (Location)format(asm, locationInformation);
			forward(asm, (Object)formattedLocationInformation);
		}
	}
	
	public static void fetchAgain(LocationSensorSM asm){
		//send FetchLocationReq (ArrayList users)
		try {
			PropertyResourceBundle myResources = new PropertyResourceBundle(new FileInputStream("defaultContextEngineProperties.properties"));
            // give UserAgent the necessary information it needs
			// - userName, phoneNumber of terminal, status. 
			//   As for the mamCallController 
			//   and mamStatusController, these can be set in init() 
			//   by looking in System actor
			int numberOfSubscribers = new Integer(myResources.getString("numberOfSubscribers")).intValue();
			ArrayList list = new ArrayList();
			for (int i = 1; i <= numberOfSubscribers; i++) {
				list.add(myResources.getString("Subscriber" + i));
			}
			asm.sendMessage(new FetchLocationReq(list), asm.myLocationFeatureAddress);

			//start counting down towards the next time to fetch the location
			
			
            //as of for the demonstrator, the second is the only format that will work
			if (asm.hourInterval == 0) {
				if (asm.minuteInterval==0) {
					if (asm.secondInterval !=0 ) {
						new TimeReminder(asm, asm.secondInterval);
					}
					else {
						System.out.println("The second interval was equal to null");
					}
				}
				else {
					System.out.println("The minute interval was NOT equal to null");
				}
			}
			else {
				System.out.println("The hour interval was NOT equal to null");
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void validate(LocationSensorSM asm){
	    // validate will not be implemented for the demonstrator, since this is not the main focus
		// the sensors are only simulated and not real, however if the task was to implement the 
		// sensors as well then validate would have to be implemented...
	}

	public static Object format(LocationSensorSM asm, ArrayList locationInformation){
	    Location finalLocationInformation = new Location();
	    // fill the array with the longitude and latitude objects
	    int latitude = ((Integer)locationInformation.get(0)).intValue();
	    int longitude = ((Integer)locationInformation.get(1)).intValue();
	    
	    Latitude lat = new Latitude();
	    Metric latMetric = new Metric();
	    latMetric.setHasValue(""+latitude);
	    lat.addHasMetric(latMetric);
	    finalLocationInformation.setLatitude(lat);
	    
	    Longitude longi = new Longitude();
	    Metric longiMetric = new Metric();
	    longiMetric.setHasValue(""+longitude);
	    longi.addHasMetric(longiMetric);
	    finalLocationInformation.setLongitude(longi);
	    
	    return finalLocationInformation;
	}

	public static void forward(LocationSensorSM asm, Object formattedLocationInformation){
	    Location location = (Location)formattedLocationInformation;
	    
		asm.myContextReasoner.updateInformation(location);
	}

	public static void startSubscribing(StartSubscriptions signal, LocationSensorSM asm){
//		send FetchLocationReq (ArrayList users)
		try {
			PropertyResourceBundle myResources = new PropertyResourceBundle(new FileInputStream("defaultContextEngineProperties.properties"));
            // give UserAgent the necessary information it needs
			// - userName, phoneNumber of terminal, status. 
			//   As for the mamCallController 
			//   and mamStatusController, these can be set in init() 
			//   by looking in System actor
			int numberOfSubscribers = new Integer(myResources.getString("numberOfSubscribers")).intValue();
			ArrayList list = new ArrayList();
			for (int i = 1; i <= numberOfSubscribers; i++) {
				list.add(myResources.getString("Subscriber" + i));
			}
			asm.sendMessage(new RegisterLocationReqSubscriptions(list), asm.myLocationFeatureAddress);
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void receiveSubscriptionInfo(LocationSubscriptionReport signal, LocationSensorSM asm){
		HashMap resultList = signal.resultList;
	    Set mapSet = resultList.keySet();
		Iterator it = mapSet.iterator();
		while (it.hasNext()) {
			String key = it.next().toString();
			LocationValueObject value = (LocationValueObject)resultList.get(key);
			float latitude = value.getLatitude();
			float longitude = value.getLongitude();
			Printer p = new Printer();
			//p.out("!!!!!!!!!Subscription Report!!!!!!!!!");
			//p.out("User: " + key + " is located at: ");
			//p.out("Latitude: " + latitude);
			//p.out("Longitude: " + longitude);
			//p.out("");
		}
        //validate()
		// format()
		//forward()
	}

	public static void stopSubscribing(StopSubscriptions signal, LocationSensorSM asm){
//		send FetchLocationReq (ArrayList users)
		try {
			PropertyResourceBundle myResources = new PropertyResourceBundle(new FileInputStream("defaultContextEngineProperties.properties"));
            // give UserAgent the necessary information it needs
			// - userName, phoneNumber of terminal, status. 
			//   As for the mamCallController 
			//   and mamStatusController, these can be set in init() 
			//   by looking in System actor
			int numberOfSubscribers = new Integer(myResources.getString("numberOfSubscribers")).intValue();
			ArrayList list = new ArrayList();
			for (int i = 1; i <= numberOfSubscribers; i++) {
				list.add(myResources.getString("Subscriber" + i));
			}
			list = new ArrayList();
			
			
			asm.sendMessage(new StopLocationReqSubscriptions(list), asm.myLocationFeatureAddress);
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void activate(StartUpContextEngine signal, LocationSensorSM asm){
	    // register with the ContextReasoner
		asm.myContextEngine = signal.getSenderRole();
		asm.myContextReasoner = signal.theContextReasoner;
		asm.myContextReasoner.register("AbsoluteLocation");
		asm.sendMessage(new LocationSensorRegistered(true), asm.myContextEngine);
		//System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		//System.out.println("LocationSensor is activating... and have registered with the ContextReasoner");
		//System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		
	}

	public static void shutDownSystem(RoleReleaseMsg signal, LocationSensorSM asm){
	    asm.sendMessage(new RoleReleaseMsg(), asm.myLocationFeatureAddress);
	}




}
