// Copyright (C) 2003-2004 Ericsson AB.


package no.ntnu.item.contextengine.sensors.locationsensor.util;

import com.ericsson.hosasdk.api.TpAddress;
import com.ericsson.hosasdk.api.TpAddressPlan;
import com.ericsson.hosasdk.api.TpAddressPresentation;
import com.ericsson.hosasdk.api.TpAddressScreening;
import com.ericsson.hosasdk.api.mm.TpMobilityError;
import com.ericsson.hosasdk.api.mm.TpMobilityDiagnostic;
import com.ericsson.hosasdk.api.mm.TpLocationRequest;
import com.ericsson.hosasdk.api.mm.TpLocationResponseTime;
import com.ericsson.hosasdk.api.mm.TpLocationPriority;
import com.ericsson.hosasdk.api.mm.TpLocationType;
import com.ericsson.hosasdk.api.mm.TpLocationResponseIndicator;
import com.ericsson.hosasdk.api.mm.TpLocationRequest;
import com.ericsson.hosasdk.api.mm.TpMobilityStopAssignmentData;
import com.ericsson.hosasdk.api.mm.TpMobilityStopScope;
import com.ericsson.hosasdk.api.mm.TpUserLocationExtended;
import com.ericsson.hosasdk.api.mm.ul.IpUserLocation;
import com.ericsson.hosasdk.api.mm.ul.IpAppUserLocation;
import com.ericsson.hosasdk.api.mm.ul.IpAppUserLocationAdapter;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.PropertyResourceBundle;

import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.nrglocationfeature.NRGLocationFeatureSM;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.LocationReceived;
import no.ntnu.item.contextengine.sensors.locationsensor.locationfetcher.signals.LocationSubscriptionReport;

import se.ericsson.eto.norarc.ejbactorframe.ActorSM;
import se.ericsson.eto.norarc.ejbframe.ActorAddress;

/**
 * This class is responsible for all Ericsson Network Resource Gateway interation regarding
 * User Location needed for this application
 * 
 * <code>Modified by Hkon Vestmoen to fit into the Master's thesis "Context Based Call Control" spring 2006.</code>
 * 
 */

public class NRGLocationProcessor extends IpAppUserLocationAdapter
    implements IpAppUserLocation
{
    private IpUserLocation itsULManager;
    private NRGLocationFeatureSM itsParent;
    private int subscriptionAssignmentID;

    /**
     * Creates a new instance.
     * @param anULManager a User Location manager
     * @param aParent the Feature that will receive location updates
     */

    public NRGLocationProcessor(IpUserLocation anULManager, NRGLocationFeatureSM aParent)
    {
        this.itsULManager = anULManager;
        this.itsParent = aParent;
        System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
        System.out.println("myParent is : " + aParent);
        System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
    }

    /**
     * Requests a location update for a set of users.
     * Location updates will be returned asynchronously,
     * using Feature.locationReceived.
     * @param aUserList a set of users.
     */

    public void requestLocation(ArrayList userList) {
        TpAddress[] users = new TpAddress[userList.size()];
        for (int i = 0; i != users.length; i++)
        {
            users[i] = createE164Address((String)userList.get(i));
        }

        TpLocationResponseTime responseTime = new TpLocationResponseTime(TpLocationResponseIndicator.P_M_NO_DELAY,
            -1); // timer value, not applicable
        float accuracy = 100f; // in meters
        boolean altitudeRequested = false;
        String locationMethod = "NETWORK";
        TpLocationRequest request = new TpLocationRequest(accuracy,
            responseTime, altitudeRequested,
            TpLocationType.P_M_CURRENT, TpLocationPriority.P_M_NORMAL,
            locationMethod);

        itsULManager.extendedLocationReportReq(this, users, request);
    }
    
    /**
     * Registers periodic location reports for given users with the NRG.
     * @param users the list of subscribers
     */
    public void registerLocationRequestSubscriptions (ArrayList userList) {
    	TpAddress[] users = new TpAddress[userList.size()];
        for (int i = 0; i != users.length; i++)
        {
            users[i] = createE164Address((String)userList.get(i));
        }

        TpLocationResponseTime responseTime = new TpLocationResponseTime(TpLocationResponseIndicator.P_M_NO_DELAY,
            -1); // timer value, not applicable
        float accuracy = 100f; // in meters
        boolean altitudeRequested = false;
        String locationMethod = "NETWORK";
        TpLocationRequest request = new TpLocationRequest(accuracy,
            responseTime, altitudeRequested,
            TpLocationType.P_M_CURRENT, TpLocationPriority.P_M_NORMAL,
            locationMethod);
    	
        //reportingInterval is defined as a org.csapi.TpDuration in the HOSA specification. This data type is a 
        //TpInt32 representing a time interval in milliseconds. A value of "-1" defines infinite duration and 
        //a value of "-2" represents a default duration.
        int reportingInterval = -2;
        try {
			PropertyResourceBundle myResources = new PropertyResourceBundle(new FileInputStream("defaultProperties.properties"));
			reportingInterval = new Integer (myResources.getString("reportingInterval")).intValue();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        
        //register the location report subscriptions with the SCS
    	this.subscriptionAssignmentID = this.itsULManager.periodicLocationReportingStartReq(this, users, request, reportingInterval);
    }
    
    /**
     * Invoked by IpUserLocation when a new report is sent.
     */
    public void periodicLocationReport (int assignmentID, TpUserLocationExtended [] reports) {
    	if (this.subscriptionAssignmentID == assignmentID) {
    		// forward the locationReports
    		
    		HashMap locationResult = new HashMap();
            for (int i = 0; i != reports.length; i++) {
                String user = reports[i].UserID.AddrString;
                if (reports[i].StatusCode == TpMobilityError.P_M_OK) {
                    float latitude = reports[i].Locations[0].GeographicalPosition.Latitude;
                    float longitude = reports[i].Locations[0].GeographicalPosition.Longitude;
                     LocationValueObject loc = new LocationValueObject(latitude, longitude);
                    locationResult.put(user, loc);
                }//end if
            } //end for
            LocationSubscriptionReport lsr = new LocationSubscriptionReport(locationResult);
            this.itsParent.sendMessage(lsr, this.itsParent.getMyActorAddress());
    	}
    }
    
    /**
     * Stops the location request subscriptions.
     * @param users the subscribers
     */
    public void stopLocationRequestSubscriptions (ArrayList userList) {
    	TpMobilityStopScope mobilityScope = new TpMobilityStopScope();
    	TpAddress[] users = new TpAddress[userList.size()];
        for (int i = 0; i != users.length; i++)
        {
            users[i] = createE164Address((String)userList.get(i));
        }
        // stop the subscriptions
    	TpMobilityStopAssignmentData stopRequest = new TpMobilityStopAssignmentData(this.subscriptionAssignmentID, mobilityScope, users);
    	this.itsULManager.periodicLocationReportingStop(stopRequest);
    }

    /**
     * Invoked by Ericsson Network Resource Gateway to present the application with location reports.
     * The geographical position for each user for which location information
     * is available is reported to the callback.
     */

    public void extendedLocationReportRes(int anAssignmentId, TpUserLocationExtended[] reports) {
        HashMap locationResult = new HashMap();
        for (int i = 0; i != reports.length; i++) {
            String user = reports[i].UserID.AddrString;
            if (reports[i].StatusCode == TpMobilityError.P_M_OK) {
                float latitude = reports[i].Locations[0].GeographicalPosition.Latitude;
                float longitude = reports[i].Locations[0].GeographicalPosition.Longitude;
                 LocationValueObject loc = new LocationValueObject(latitude, longitude);
                locationResult.put(user, loc);
            }//end if
        } //end for
        LocationReceived locRcvdMsg = new LocationReceived(locationResult);
        System.out.println("sending Location received to NRGLocationFetcher");
        this.itsParent.sendMessage(locRcvdMsg, this.itsParent.getMyActorAddress());
    }

    private static TpAddress createE164Address(String aNumber)
    {
        return new TpAddress(TpAddressPlan.P_ADDRESS_PLAN_E164,
            aNumber, "",
            TpAddressPresentation.P_ADDRESS_PRESENTATION_UNDEFINED,
            TpAddressScreening.P_ADDRESS_SCREENING_UNDEFINED, "");
    }
}
