package no.ntnu.item.contextengine.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.PropertyResourceBundle;

import com.hp.hpl.jena.util.FileUtils;

import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.jena.JenaOWLModel;
import edu.stanford.smi.protegex.owl.model.OWLDatatypeProperty;
import edu.stanford.smi.protegex.owl.model.OWLIndividual;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.OWLObjectProperty;
import edu.stanford.smi.protegex.owl.model.RDFIndividual;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLDatatypeProperty;

public class OntologyOperator {

	/**
	 * Fetches the own ontology model structure that is the same for all User Agents.
	 * @param fileName the file containing the structure.
	 * @return the object containing the model of the structure.
	 */

	public JenaOWLModel fetchStructure(String fileName) {
		JenaOWLModel owlModel = null;
		try {
			owlModel = ProtegeOWL
					.createJenaOWLModelFromInputStream(new FileInputStream(
							new File(fileName)));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return owlModel;
	}

	public boolean individualExists(JenaOWLModel owlModel, String individualName) {
		boolean exists = true;
		OWLIndividual individual = owlModel.getOWLIndividual(individualName);
		if (individual == null) {
			exists = false;
		}
		return exists;
	}

	/**
	 * Creates an individual of the specified OWL class.
	 * 
	 * @param owlModel -
	 *            the owl instance for this userAgent
	 * @param owlClassName -
	 *            the name of the OWL class to be assigned an individual.
	 * @param individualName -
	 *            the name of the OWL individual to be created
	 * @return the updated owl instance for this user agent
	 */
	public JenaOWLModel assignIndividual(JenaOWLModel owlModel,
			String owlClassName, String individualName) {
		//first create the individual in the model
		OWLNamedClass theOwlClass = owlModel.getOWLNamedClass(owlClassName);
		// System.out.println("Assigning individual");
		// System.out.println("Found the owl class " +
		// theOwlClass.getBrowserText());
		try {
			theOwlClass.createOWLIndividual(individualName);
			// return the populated model
			return owlModel;

		} catch (IllegalArgumentException e) {
			//e.printStackTrace();
			return owlModel;
		}

	}

	/**
	 * Assigns a data property to the specified individual.
	 * 
	 * @param owlModel -
	 *            the owl instance for this userAgent
	 * @param individualName -
	 *            the name of the individual to be assigned a data property
	 * @param dataPropertyName -
	 *            the name of the data property to use
	 * @param dataPropertyValue -
	 *            the range value of the data property
	 * @return the updated owl instance fot this user agent
	 */
	public JenaOWLModel assignDataProperty(JenaOWLModel owlModel,
			String individualName, String dataPropertyName,
			String dataPropertyValue) {
		OWLIndividual owlIndividual = owlModel.getOWLIndividual(individualName);
		// System.out.println("Assigning data property");
		//System.out.println("Found the owl individual " +
		//owlIndividual.getBrowserText());

		OWLDatatypeProperty dataProperty = owlModel
				.getOWLDatatypeProperty(dataPropertyName);
		//System.out.println("Found the owl data property " +
		//dataProperty.getBrowserText());

		owlIndividual.addPropertyValue(dataProperty, dataPropertyValue);
		return owlModel;
	}

	/**
	 * Modifies the data property value of the specified individual.
	 * The data property value is only modified if the provided value is different from the one 
	 * stored in the model.
	 * 
	 * @param owlModel -
	 *            the owl instance for this userAgent
	 * @param individualName -
	 *            the name of the individual to be assigned a data property
	 * @param dataPropertyName -
	 *            the name of the data property to use
	 * @param dataPropertyValue -
	 *            the range value of the data property
	 * @return the updated owl instance fot this user agent
	 */
	public JenaOWLModel modifyDataProperty(JenaOWLModel owlModel,
			String individualName, String dataPropertyName,
			String dataPropertyValue) {
		OWLIndividual owlIndividual = owlModel.getOWLIndividual(individualName);

		OWLDatatypeProperty dataProperty = owlModel
				.getOWLDatatypeProperty(dataPropertyName);

		if (!owlIndividual.hasPropertyValue(dataProperty, dataPropertyValue)) {
			//this means that the value in the model is different from the newly received one
			//since this is the case, the value needs to be updated in the model so that it
			//corresponds to the one observed by the sensors
			//System.out.println("Found, found!!");

			owlIndividual.setPropertyValue(dataProperty, dataPropertyValue);

			// could also store this in the file

		}

		return owlModel;
	}

	/**
	 * Assigns an object property to the specified individual
	 * 
	 * @param owlModel -
	 *            the owl instance for this userAgent
	 * @param individualName -
	 *            the name of the individual to be assigned an object property
	 * @param objectPropertyName -
	 *            the name of the object property to use
	 * @param objectPropertyRange -
	 *            the range individual of the object property
	 * @return the updated owl instance for this user agent
	 */
	public JenaOWLModel assignObjectProperty(JenaOWLModel owlModel,
			String individualName, String objectPropertyName,
			String objectPropertyRange) {
		OWLIndividual owlIndividual = owlModel.getOWLIndividual(individualName);
		// System.out.println("Assigning object property");
		// System.out.println("Found the owl individual domain "
		// + owlIndividual.getBrowserText());

		// System.out.println();
		// System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		// System.out.println("Trying to find the object property "
		// + objectPropertyName);
		OWLObjectProperty objectProperty = owlModel
				.getOWLObjectProperty(objectPropertyName);
		if (objectProperty == null) {
			// System.out.println("Object property was NOT found!!!!");
		} else {
			// System.out.println("Object property was found!!!!");
		}
		/*
		 * System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		 * System.out.println();
		 * 
		 * System.out.println();
		 * System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		 * System.out.println("Trying to find the owl individual range " +
		 * objectPropertyRange);
		 */
		OWLIndividual rangeIndividual = owlModel
				.getOWLIndividual(objectPropertyRange);
		if (rangeIndividual == null) {
			// System.out.println("Owl individual range was NOT found!!!!");
		} else {
			// System.out.println("Owl individual range was found!!!!");
		}
		// System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
		// System.out.println();

		owlIndividual.addPropertyValue(objectProperty, rangeIndividual);
		return owlModel;
	}

	/**
	 * Stores the information about the user on disk.
	 * 
	 * @param owlModel
	 *            the owl instance containing the context information for this
	 *            user agent
	 */
	public void storeUserInformation(JenaOWLModel owlModel) {
		try {
			PropertyResourceBundle myResources = new PropertyResourceBundle(
					new FileInputStream("userConfiguration.properties"));
			String personIndividualID = myResources.getString("personID");
			String fileName = personIndividualID + ".owl";

			Collection errors = new ArrayList();
			owlModel.save(new File(fileName).toURI(), FileUtils.langXMLAbbrev,
					errors);
			// System.out.println("File saved with " + errors.size() + "
			// errors.");
			// owlModel.save(new File(fileName).toURI());
			// System.out.println("File saved");
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public ArrayList getIndividuals(JenaOWLModel owlModel, String className) {
		ArrayList individuals = null;

		OWLNamedClass theClass = owlModel.getOWLNamedClass(className);
		// the boolean variable indicates if all the instances of subclasses also shall be returned
		Collection individualsObject = theClass.getInstances(false);
		//System.out.println("Found the individuals and the class is called " + individualsObject.getClass().getName());
		if (individualsObject == null) {
			ArrayList a = new ArrayList();
			a.add("undefined");
			return a;
		} else {
			ArrayList a = new ArrayList(individualsObject);
			return a;
		}
	}

	public Object getObjectPropertyValue(JenaOWLModel owlModel,
			String individualName, String objectPropertyName) {
		Object objectPropertyValue = null;
		OWLIndividual individual = owlModel.getOWLIndividual(individualName);
		OWLObjectProperty objectProperty = owlModel
				.getOWLObjectProperty(objectPropertyName);
		objectPropertyValue = individual.getPropertyValue(objectProperty);
		if (objectPropertyValue == null) {
			return "undefined";
		} else {
			return objectPropertyValue;
		}

	}

	public String getDatatypeValue(JenaOWLModel owlModel,
			String individualName, String dataPropertyName) {
		String value = null;
		OWLIndividual owlIndividual = owlModel.getOWLIndividual(individualName);
		//System.out.print("Found the individual: " +owlIndividual.getName());
		//System.out.print("Trying to find the data property: " + dataPropertyName);
		OWLDatatypeProperty dataProperty = owlModel
				.getOWLDatatypeProperty(dataPropertyName);

		// System.out.print("Found the data property: " +dataProperty.getName());
		//System.out.print("Trying to find the value of the data property: " +dataProperty.getName());
		Object valueObject = owlIndividual.getPropertyValue(dataProperty);
		if (valueObject == null) {
			// meaning that the value of the dataproperty has not yet been set
			return "undefined";

		}

		else if (valueObject.toString().equals("undefinedValue")) {
			return "undefined";
		} else {
			return valueObject.toString();
		}
	}
}
