package no.ntnu.item.contextengine.sensors.activitysensor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.PropertyResourceBundle;
import java.util.StringTokenizer;

import edu.stanford.smi.protege.model.FrameID;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.jena.JenaOWLModel;

import no.ntnu.item.contextengine.controller.ContextReasoner;
import no.ntnu.item.contextengine.model.ontologyclasses.Metric;
import no.ntnu.item.contextengine.model.ontologyclasses.PersonEntity;
import no.ntnu.item.contextengine.model.ontologyclasses.Relative;
import no.ntnu.item.contextengine.model.ontologyclasses.ScheduledActivity;
import no.ntnu.item.contextengine.model.ontologyclasses.Time;
import no.ntnu.item.contextengine.sensors.ContextSensor;

public class ActivitySensor implements ContextSensor {

	private PropertyResourceBundle activityProperties = null;

	private ContextReasoner contextReasoner = null;

	public ActivitySensor(ContextReasoner contextReasoner) {
		this.contextReasoner = contextReasoner;
	}

	public void registerWithReasoner() {
		this.contextReasoner.register("ScheduledActivity");
	}

	public void fetch(Object guidanceData) {
		try {
			this.activityProperties = new PropertyResourceBundle(
					new FileInputStream("userScheduledActivities.properties"));

			int numberOfActivities = new Integer(this.activityProperties
					.getString("numberOfMeetings")).intValue();
			String summary;
			String description;
			String begins;
			String isDone;
			String location;
			String participant;
			String category;
			String priority;
			// for each activity
			for (int i = 1; i <= numberOfActivities; i++) {
				// obtain all the information that is to be filled out about the activity
				//System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!");
				//System.out.println("Report for activity number: " +i);
				summary = this.activityProperties.getString(i + "Summary");
				description = this.activityProperties.getString(i
						+ "Description");
				begins = this.activityProperties.getString(i
						+ "BeginsAtTimeMetricValue");
				isDone = this.activityProperties.getString(i
						+ "IsDoneAtTimeMetricValue");
				location = this.activityProperties.getString(i + "Location");
				participant = this.activityProperties.getString(i
						+ "Participant");
				category = this.activityProperties.getString(i + "Category");
				priority = this.activityProperties.getString(i + "Priority");
				/*System.out.println("Summary: " + summary);
				 System.out.println("Description: " + description);
				 System.out.println("Starts at: " + begins);
				 System.out.println("Is done at: " + isDone);
				 System.out.println("Located at: " + location);
				 System.out.println("Has participant: " + participant);
				 System.out.println("Category: " + category);
				 System.out.println("Priority: " + priority);
				 System.out.println();*/

				//validate the information that needs to be validated
				summary = validate(summary).toString();
				description = validate(description).toString();
				begins = validate(begins).toString();
				isDone = validate(isDone).toString();
				location = validate(location).toString();
				participant = validate(participant).toString();
				category = validate(category).toString();
				priority = validate(priority).toString();

				/*System.out.println("After validation");
				 System.out.println("Summary: " + summary);
				 System.out.println("Description: " + description);
				 System.out.println("Starts at: " + begins);
				 System.out.println("Is done at: " + isDone);
				 System.out.println("Located at: " + location);
				 System.out.println("Has participant: " + participant);
				 System.out.println("Category: " + category);
				 System.out.println("Priority: " + priority);*/

				//format the information that needs to be formated
				Collection activityInformation = new ArrayList();
				Integer activityNumber = new Integer(i);

				activityInformation.add(activityNumber);
				activityInformation.add(summary);
				activityInformation.add(description);
				activityInformation.add(begins);
				activityInformation.add(isDone);
				activityInformation.add(location);
				activityInformation.add(participant);
				activityInformation.add(category);
				activityInformation.add(priority);

				ScheduledActivity scheduledActivity = (ScheduledActivity) format(activityInformation);
				//System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!");

				// forward the information to ContextReasoner
				forward(scheduledActivity);

			} // end for
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void forward(Object contextData) {
		// send the data to the ContextReasoner
		ScheduledActivity contextDataForwardable = (ScheduledActivity) contextData;
		this.contextReasoner.updateInformation(contextDataForwardable);
	}

	public Object validate(Object data) {
		String validatedData = (String) data;
		// now validate the data
		// no data can be equal to null, so this must be checked
		// besides that, the data can be anything. 
		// Recognizing data is done in the ContextReasoner (e.g. recognizing a location as a 
		// relative location that the User has defined)
		if (data.equals("null")) {
			validatedData = "invalid";
		}
		return (Object) validatedData;
	}

	public Object format(Collection activityInformation) {
		// format the Strings into value objects that can be handled by the ContextReasoner and 
		// that are in accordance with the OWL classes defined in the model

		// retrieve the information from the collection
		ArrayList activityInfo = (ArrayList) activityInformation;
		int activityNumber = ((Integer) activityInfo.get(0)).intValue();
		String summaryValue = (String) activityInfo.get(1);
		String descriptionValue = (String) activityInfo.get(2);
		String beginsValue = (String) activityInfo.get(3);
		String isDoneValue = (String) activityInfo.get(4);
		String locationValue = (String) activityInfo.get(5);
		String participantValue = (String) activityInfo.get(6);
		String categoryValue = (String) activityInfo.get(7);
		String priorityValue = (String) activityInfo.get(8);

		// initialize and name the activity object
		ScheduledActivity scheduledActivity = new ScheduledActivity();
		scheduledActivity.setName("Activity" + activityNumber);

		//set the 4 datatype properties of the scheduled activity
		scheduledActivity.addHasSummary(summaryValue);
		scheduledActivity.addHasDescription(descriptionValue);
		scheduledActivity.addHasCategory(categoryValue);
		scheduledActivity.addHasPriority(priorityValue);

		//set the name of the participant
		Collection participantCollection = new ArrayList();
		PersonEntity participantEntity = new PersonEntity();
		participantEntity.setName(participantValue);
		scheduledActivity.addHasParticipant(participantEntity);

		//set the start time for the activity
		Time beginsTime = new Time();
		beginsTime.setName("Activity" + activityNumber + "TimeBegins");
		Metric activityBeginsTimeMetric = new Metric();
		activityBeginsTimeMetric.setName("Activity" + activityNumber
				+ "TimeBeginsMetric");
		activityBeginsTimeMetric.setHasType("String");
		activityBeginsTimeMetric.setHasUnit("YearMonthDayHourMinuteSecond");
		activityBeginsTimeMetric.setHasValue(beginsValue);
		beginsTime.addHasMetric(activityBeginsTimeMetric);
		scheduledActivity.addBeginsAtTime(beginsTime);

		//set the end time for the activity
		Time endsTime = new Time();
		endsTime.setName("Activity" + activityNumber + "TimeEnds");
		Metric activityEndsTimeMetric = new Metric();
		activityEndsTimeMetric.setName("Activity" + activityNumber
				+ "TimeEndsMetric");
		activityEndsTimeMetric.setHasType("String");
		activityEndsTimeMetric.setHasUnit("YearMonthDayHourMinuteSecond");
		activityEndsTimeMetric.setHasValue(isDoneValue);
		endsTime.addHasMetric(activityEndsTimeMetric);
		scheduledActivity.addIsDoneAtTime(endsTime);

		// set the location (RELATIVE) for the activity
		// since this Relative object is already defined in the ontology, the name is the only 
		// one that needs to be set
		Relative locationEntity = new Relative();
		locationEntity.setName(locationValue);
		scheduledActivity.addHasLocation(locationEntity);

		return scheduledActivity;

	}
}
