package no.ntnu.item.serviceframe.actors.policymanagement.policymanager;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;

import se.ericsson.eto.norarc.ejbframe.ActorAddress;
import se.ericsson.eto.norarc.ejbframe.TraceConstants;

import no.ntnu.item.serviceframe.auxiliar.agentprofile.AgentProfile;
import no.ntnu.item.serviceframe.auxiliar.agentprofile.UserProfile;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.Node;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAge;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAnd;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAtomicPredicate;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAvailabilityEntity;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAvailabilityUser;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeCallType;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeDeducedActivity;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeDevice;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeFalse;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeLocation;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeNoAnswer;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeNot;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeOr;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeScreeningRoleList;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeScreeningUserList;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeStartCondition;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeSubscription;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTime;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTimeBtw;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTimeDate;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTimeDay;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTrue;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.ParseException;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.CollaborationPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.FeatureSelectionPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.OnRRFSPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.RoleBindingPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.sessiondescription.SessionDescription;
import no.ntnu.item.serviceframe.signals.agentprofile.ProfileLoadRequestMsg;
import no.ntnu.item.serviceframe.signals.agentprofile.ProfileMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.CollaborationPolicyListReplyMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.CollaborationPolicyListReqMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.OnNonAGPolicyListReplyMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.OnNonAGPolicyListReqMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.OnRRPolicyListReplyMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.OnRRPolicyListReqMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.OnUnsRBPolicyListReplyMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.OnUnsRBPolicyListReqMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.RoleBindingPolicyListReplyMsg;
import no.ntnu.item.serviceframe.signals.getpolicylist.RoleBindingPolicyListReqMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckNOKMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOKMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOnGnotAReqMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOnRRReqMsg;

public class PolicyManagerActions extends Object {
	public static Calendar rightNow = Calendar.getInstance();

	public static ArrayList currentArrayList;

	//	+++++++++++ AUXILIARY METHODS ++++++++++++++++++++

	/*
	 * This method checks if all Role Binding policies are satisfied. If so, 
	 * it returns "null". Otherwise, it returns the reason (i.e. the negation of the condition not satisfied)
	 * 
	 */
	private static Node checkRoleBindingPolicies(Collection collection,
			AgentProfile agentProfile, SessionDescription sessionDesc) {
		//System.out.println("Checking the Role Binding policies");
		Node result = null;
		if (collection != null) {

			RoleBindingPolicyObject onRB;
			Node condition;

			for (Iterator p = collection.iterator(); p.hasNext();) {

				onRB = (RoleBindingPolicyObject) p.next();
				condition = onRB.getCondition();

				// All policies must be satisfied
				if (evaluateCondition(condition, null, agentProfile,
						sessionDesc) == false) {
					//Now we negate the condition. The negated condition will be the reason why the policy was not satisfied
					PCNodeNot not = new PCNodeNot(4422); //We pass any number
					not.jjtAddChild(condition.jjtGetChild(0), 0);
					result = not;
					break;
				}

			}
		}
		return result;
	}

	/*
	 * This method checks if there exists any Feature-Selection policy  that is satisfied  
	 * in the input policy list. If so, it returns an ArrayList containing  
	 * the type of the alternative role to be played (first position) and a set of parameters 
	 * that may be needed by the role (second position). Otherwise, it returns "null"
	 * 
	 */
	private static ArrayList checkFeatureSelectionPolicies(
			Collection collection, AgentProfile agentProfile,
			SessionDescription sessionDesc, Node selectionReason) {
		//System.out.println("CHECKING Feature Selection policies...");
		ArrayList result = null;

		if (collection == null) System.out.println("No onRR policies");
		if (collection != null) {
			System.out.println("There are onRR policies");
			//System.out.println("CHECKING "+(array.get(0)).toString());

			Node condition;
			//OnRRFSPolicyObject onRR;
			FeatureSelectionPolicyObject fsPO;

			for (Iterator p = collection.iterator(); p.hasNext();) {

				fsPO = (FeatureSelectionPolicyObject) p.next();
				condition = fsPO.getCondition();
				if (evaluateCondition(condition, selectionReason, agentProfile,
						sessionDesc) == true) {
					result = new ArrayList();
					result.add(fsPO.getAlternativeRole());
					result.add(fsPO.getParameters());
					break;
				}
			}
		}
		return result;
	}

	/*
	 * This method checks if all the Collaboration policies are satisfied
	 * If so, it returns "null". Otherwise, it returns the reason 
	 * (i.e. the negation of the condition not satisfied)
	 * Right now it is not generic enough (e.g. only "peerUser" is passed as input
	 * parameter, but other input parameter may be needed to check certain policies
	 * Parameter-passing should be made generic (TO BE DONE!)
	 */
	private static Node checkCollaborationPolicies(Collection collection,
			AgentProfile agentProfile, SessionDescription sessionDesc) {
		//System.out.println("CHECKING the Collaboration policies");
		Node result = null;

		if (collection != null) {

			CollaborationPolicyObject collabObj;
			Node condition;

			for (Iterator p = collection.iterator(); p.hasNext();) {

				collabObj = (CollaborationPolicyObject) p.next();
				condition = collabObj.getCondition();

				if (evaluateCondition(condition, null, agentProfile,
						sessionDesc) == false) {
					//Now we negate the condition. The negated condition will be the reason why the policy was not satisfied
					PCNodeNot not = new PCNodeNot(2244); //We pass any number
					not.jjtAddChild(condition.jjtGetChild(0), 0);
					result = not;
					break;
				}
			}
		}
		return result;
	}

	private static boolean evaluateCondition(Node condition,
			Node selectionReason, AgentProfile agentProfile,
			SessionDescription sessionDesc) {
		boolean result = false;

		if (condition instanceof PCNodeStartCondition) {
			//System.out.println("We have a ROOT");
			result = evaluateCondition(condition.jjtGetChild(0),
					selectionReason, agentProfile, sessionDesc);
			//System.out.println("Result ROOT is: "+result);
		} else if (condition instanceof PCNodeAnd) {
			//System.out.println("We have an AND");
			result = true;
			for (int i = 0; i < condition.jjtGetNumChildren(); i++) {
				//All predicates must be true
				if (evaluateCondition(condition.jjtGetChild(i),
						selectionReason, agentProfile, sessionDesc) == false) {
					result = false;
					break;
				}
			}
			//System.out.println("Result AND is: "+result);
		} else if (condition instanceof PCNodeOr) {
			//System.out.println("We have an OR");
			result = false;
			for (int i = 0; i < condition.jjtGetNumChildren(); i++) {
				//At least one predicate has to be true
				if (evaluateCondition(condition.jjtGetChild(i),
						selectionReason, agentProfile, sessionDesc) == true) {
					result = true;
					break;
				}
			}
			//System.out.println("Result OR is: "+result);
		} else if (condition instanceof PCNodeNot) {
			//System.out.println("We have a NOT");
			result = !evaluateCondition(condition.jjtGetChild(0),
					selectionReason, agentProfile, sessionDesc);
			//System.out.println("Result NOT is: "+result);
		} else if (condition instanceof PCNodeAtomicPredicate) {
			//System.out.println("We have an AtomicPredicate");
			result = evaluateAtomicPredicate(condition.jjtGetChild(0),
					selectionReason, agentProfile, sessionDesc);
			//System.out.println("Result AtomicPredicate is: "+result);
		}

		return result;
	}

	/*
	 * This method is used to evaluate atomic predicates
	 * 
	 * Note that we may associate policies with different types of agents.
	 * For each type of agent the checks to be performed may be different (or in other words,
	 * the predicates used to define policies for different agents may be different). 
	 * Therefore there should be some way to specialize this method for each kind of agent. One 
	 * possibility is to have an inner actor to the PolicyManager implementing this method.
	 * Then, we may used specialized inner actors for each kind af agent.
	 * TO BE DONE
	 */
	private static boolean evaluateAtomicPredicate(Node predicate,
			Node selectionReason, AgentProfile agentProfile,
			SessionDescription sessionDesc) {

//		 Right now, the selectionReason parameter is only used for OnNonAchievedGoal policies (it has to be
		// decided yet if it is useful to use it also with OnUnsuccessfulRoleBinding policies).
		// If 'selectionReason' is not 'null' we are checking an OnNonAchievedGoal policy. Therefore
		// we check first the condition against the selectionReason
		if (selectionReason != null) {

			if (predicate instanceof PCNodeTrue) {
				//System.out.println("We have a PCNodeTrue");
				return true;
			} else if (predicate instanceof PCNodeFalse) {
				//System.out.println("We have a PCNodeFalse");
				return false;
			} else if (predicate instanceof PCNodeNoAnswer) {
				//System.out.println("We have a PCNodeNoAnswer");
				if (selectionReason instanceof PCNodeNoAnswer)
					return true;
				else
					return false;
			}

			// TO BE DONE. More checkings to be added here
			
			

		} //The condition of an OnNonAchievedGoal policy may also be expressed in terms of time, location, etc. Thus, we continue checking 

		if (predicate instanceof PCNodeTime) {
			//System.out.println("We have a PCNodeTime");
			PCNodeTime t = (PCNodeTime) predicate;
			if (t.getOperator().equalsIgnoreCase("=")) {
				if (rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) {
					return true;
				} else {
					return false;
				}
			} else if (t.getOperator().equalsIgnoreCase("!=")) {
				if ((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour())
						&& (rightNow.get(Calendar.MINUTE) == t.getMinutes())) {
					return false;
				}
				return true;
			} else if (t.getOperator().equalsIgnoreCase("<")) {
				if ((rightNow.get(Calendar.HOUR_OF_DAY) < t.getHour())
						|| ((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow
								.get(Calendar.MINUTE) < t.getMinutes()))) {
					return true;
				}
				return false;
			} else if (t.getOperator().equalsIgnoreCase(">")) {
				if ((rightNow.get(Calendar.HOUR_OF_DAY) > t.getHour())
						|| ((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow
								.get(Calendar.MINUTE) > t.getMinutes()))) {
					return true;
				}
				return false;
			} else if (t.getOperator().equalsIgnoreCase("<=")) {
				if ((rightNow.get(Calendar.HOUR_OF_DAY) < t.getHour())
						|| ((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow
								.get(Calendar.MINUTE) <= t.getMinutes()))) {
					return true;
				}
				return false;
			} else if (t.getOperator().equalsIgnoreCase(">=")) {
				if ((rightNow.get(Calendar.HOUR_OF_DAY) > t.getHour())
						|| ((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow
								.get(Calendar.MINUTE) >= t.getMinutes()))) {
					return true;
				}
				return false;
			}
		} else if (predicate instanceof PCNodeTimeBtw) {
			//System.out.println("We have a PCNodeTimeBtw");
			PCNodeTimeBtw t = (PCNodeTimeBtw) predicate;
			if (rightNow.get(Calendar.HOUR_OF_DAY) >= t.getStartHour()
					&& rightNow.get(Calendar.HOUR_OF_DAY) <= t.getEndHour()) {
				if (rightNow.get(Calendar.MINUTE) >= t.getStartMinutes()
						&& rightNow.get(Calendar.MINUTE) <= t.getEndMinutes()) {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		} else if (predicate instanceof PCNodeTimeDay) {
			//System.out.println("We have a PCNodeTimeDay");
			PCNodeTimeDay t = (PCNodeTimeDay) predicate;
			int weekday = 0;
			if (t.getWeekday().equalsIgnoreCase("monday")) {
				weekday = 2;
			} else if (t.getWeekday().equalsIgnoreCase("tuesday")) {
				weekday = 3;
			} else if (t.getWeekday().equalsIgnoreCase("wednesday")) {
				weekday = 4;
			} else if (t.getWeekday().equalsIgnoreCase("thursday")) {
				weekday = 5;
			} else if (t.getWeekday().equalsIgnoreCase("friday")) {
				weekday = 6;
			} else if (t.getWeekday().equalsIgnoreCase("saturday")) {
				weekday = 7;
			} else if (t.getWeekday().equalsIgnoreCase("sunday")) {
				weekday = 8;
			}
			int day = rightNow.get(Calendar.DAY_OF_WEEK);
			if (weekday == day) {
				return true;
			}
			return false;
		} else if (predicate instanceof PCNodeTimeDate) {
			//System.out.println("We have a PCNodeTimeDate");
			PCNodeTimeDate t = (PCNodeTimeDate) predicate;
			if (t.getYear() == rightNow.get(Calendar.YEAR)) {
				if (t.getMonth() == rightNow.get(Calendar.MONTH)) {
					if (t.getDay() == rightNow.get(Calendar.DAY_OF_MONTH)) {
						return true;
					}
				}
			}
			return false;
		} else if (predicate instanceof PCNodeAge) {
			//System.out.println("We have a PCNodeAge");
			PCNodeAge t = (PCNodeAge) predicate;
			// CHECK INTENDED FOR A USERAGENT -> WE USE USERPROFILE
			if (t.getAge() == (((UserProfile) agentProfile).getAge())) {
				return true;
			}
			return false;
		} else if (predicate instanceof PCNodeSubscription) {
			//System.out.println("We have a PCNodeSubscription");
			PCNodeSubscription t = (PCNodeSubscription) predicate;
			//for(int j = 0; j < userProfile.getSubscriptions().size(); j++){
			//	if(value.equalsIgnoreCase((String) userProfile.getSubscriptions().get(j))){
			//		return true;
			//	}
			//}
			// CHECK INTENDED FOR A USERAGENT -> WE USE USERPROFILE
			if (((UserProfile) agentProfile).getSubscriptions().contains(
					t.getSubscription())) {
				return true;
			}
			return false;
		} else if (predicate instanceof PCNodeScreeningRoleList) {
			//System.out.println("We have a PCNodeScreeningRoleList");
			//TO BE DONE
			//We should check if the screening list contains any role already participating
			//in the session or any role that has been invited to it. If so, return false.
			return true;
		} else if (predicate instanceof PCNodeScreeningUserList) {
			//System.out.println("We have a PCNodeScreeningUserList");
			//OBS! A condition based on a screening list can be seen as
			//a condition of type "NOT listOfUsers". Therefore, we return "false" (i.e. "NOT true")
			//if any user participating in the session is on the screening list
			PCNodeScreeningUserList sl = (PCNodeScreeningUserList) predicate;
			if (sl.getUserList().contains(sessionDesc.contactedUserName)
					|| sl.getUserList().contains(sessionDesc.initiatorUserName))
				return false;
			else
				return true;
		} else if (predicate instanceof PCNodeAvailabilityUser) {
			System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			System.out.println("We have a PCNodeAvailabilityUser");
			PCNodeAvailabilityUser a = (PCNodeAvailabilityUser) predicate;
			//We don't look at the user id. We assume the availability condition applies to the owner of the profile
			//If necessary, change this.
			UserProfile userProfile = (UserProfile) agentProfile;
			String homeAvailabilityValue = userProfile.getHomeAvailability();
			String officeAvailabilityValue = userProfile
					.getOfficeAvailability();
			String meetingRoomAvailabilityValue = userProfile
					.getMeetingRoomAvailability();
			String predicateHomeAvailability = a.getHomeAvailability();
			String predicateOfficeAvailability = a.getOfficeAvailability();
			String predicateMeetingRoomAvailability = a
					.getMeetingRoomAvailability();
			System.out.println("Predicate homeAvailability="
					+ predicateHomeAvailability + " officeavailability="
					+ predicateOfficeAvailability
					+ " meetingRoomAvailability="
					+ predicateMeetingRoomAvailability);
			System.out.println("User profile homeAvailability="
					+ homeAvailabilityValue + " officeavailability="
					+ officeAvailabilityValue
					+ " meetingRoomAvailability="
					+ meetingRoomAvailabilityValue);
			if (homeAvailabilityValue == null) {
				homeAvailabilityValue = "UNDEFINED";
			}
			if (officeAvailabilityValue == null) {
				officeAvailabilityValue = "UNDEFINED";
			}
			if (meetingRoomAvailabilityValue == null) {
				meetingRoomAvailabilityValue = "UNDEFINED";
			}
			if (((homeAvailabilityValue.equals(predicateHomeAvailability))
					&& (officeAvailabilityValue.equals(predicateOfficeAvailability)) 
					&& (meetingRoomAvailabilityValue.equals(predicateMeetingRoomAvailability)))) {
				System.out.println("The AVAILABILITY predicate and the profile MATCHED!!!!!!!!!!!!!!!!!!!!!");
				System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return true;
			} else {
				System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return false;
			}
		} else if (predicate instanceof PCNodeDeducedActivity) {
			System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			System.out.println("We have a PCNodeDeducedActivity");
			PCNodeDeducedActivity a = (PCNodeDeducedActivity) predicate;
			UserProfile userProfile = (UserProfile) agentProfile;
			String deducedActivityValue = userProfile.getDeducedActivity();
			System.out.println("Predicate deduced activity: " + a.getValue());
			System.out.println("User profile deduced activity: " + deducedActivityValue);
			if (deducedActivityValue == null) {
				deducedActivityValue = "UNDEFINED";
			}
			if (deducedActivityValue.equals(a.getValue())) {
				System.out.println("The DeducedActivity predicate and the profile MATCHED!!!!!!!!!!!!!!!!!!!!!");
				System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return true;
			} else {
				System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return false;
			}
			
		} else if (predicate instanceof PCNodeCallType) {
			System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			System.out.println("We have a PCNodeCallType");
			PCNodeCallType a = (PCNodeCallType) predicate;
			String predicateCallType = a.getType();
			String sessionCallType = sessionDesc.getCallType();
			
			System.out.println("Predicate call type: " + predicateCallType);
			System.out.println("Session call type: " + sessionCallType);
			if (sessionCallType == null) {
				sessionCallType= "UNDEFINED";
			}
			if (sessionCallType.equals(predicateCallType)) {
				System.out.println("The callType predicate and the profile MATCHED!!!!!!!!!!!!!!!!!!!!!");
				System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return true;
			} else {
				System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
				return false;
			}
		} else if (predicate instanceof PCNodeAvailabilityEntity) {
			//System.out.println("We have a PCNodeAvailabilityEntity");
			PCNodeAvailabilityEntity a = (PCNodeAvailabilityEntity) predicate;
			if (((UserProfile) agentProfile).getTerminalStatus(a.getEntityID()) == a
					.getStatus()) {
				return true;
			}
			return false;
		} else if (predicate instanceof PCNodeLocation) {
			//System.out.println("We have a PCNodeLocation");
			PCNodeLocation l = (PCNodeLocation) predicate;
			if (l.getLocation().equalsIgnoreCase(
					((UserProfile) agentProfile).getLocation()))
				return true;
			else
				return false;
		} else if (predicate instanceof PCNodeDevice) {
			//System.out.println("We have a PCNodeNoDevice");
			PCNodeDevice d = (PCNodeDevice) predicate;
			//TO BE DONE
			//Not clear how to deal with this predicate
			//To be compared with sessionDescription info?
			return true;
		} else if (predicate instanceof PCNodeTrue) {
			//System.out.println("We have a PCNodeTrue");
			return true;
		} else if (predicate instanceof PCNodeFalse) {
			//System.out.println("We have a PCNodeFalse");
			return false;
		}

		return false;
	}

	//    +++++++++ END OF AUXILIARY METHODS ++++++++++++++++++++

	/*
	 * source vertex: waitForData
	 * target vertex: waitForOnRRArrayList
	 */
	public static void getOnRRArrayList(PolicyCheckOnRRReqMsg signal,
			PolicyManagerSM asm) {
		//System.out.println("PolicyManager: Fetching the OnRR arraylist");
		//StatusMessageMsg smm = new StatusMessageMsg("Checking OnRoleRequest Feature-selection Policy for: " + asm.savedPolicyCheckRequest.RoleRequest.getRoleType());
		//asm.sendMessage(smm, asm.savedPolicyCheckRequest.getSenderRole());
		asm.checkRequestor = signal.getSenderRole();
		asm.currentSessionDescription = signal.sessionDescription;
		asm.currentSubjectRole = signal.roleType;
		//asm.sendMessage(new OnRRPolicyListReqMsg(signal.roleType), "policyStorePort");
		System.out.println("///Getting onRR policies for:" +signal.roleType);
		asm.sendMessage(new OnRRPolicyListReqMsg(signal.roleType),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));

	}

	/*
	 * source vertex: waitForOnRRArrayList
	 * target vertex: waitForRoleBindingArrayList
	 */
	public static void getRoleBindingArrayList(OnRRPolicyListReplyMsg signal,
			PolicyManagerSM asm) {
		// Before trying to bind the requested role (and, therefore, before checking its
		// RoleBinding policies), we check if there is any 
		// OnRoleRequest Feature-Selection policy that holds
		//System.out.println("Checking OnRoleRequest policies");

		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework,
				asm.trace.getTraceHeader()
						+ "Checking OnRoleRequest F.S. Policies for role <<"
						+ signal.role + ">>");

		asm.altRoleAndParameters = checkFeatureSelectionPolicies(signal.onrr,
				asm.agentProfile, asm.currentSessionDescription, null);

		if (asm.altRoleAndParameters != null) {
			asm.currentSubjectRole = (String) asm.altRoleAndParameters.get(0);
			//System.out.println("OnRoleRequest policy found. We will try to bind an alternative role");
			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "One OnRoleRequest F.S. Policy was satisfied. We will try to bind the role <<"
									+ asm.currentSubjectRole
									+ ">> instead of <<" + signal.role + ">>");
		} else {
			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "No OnRoleRequest F.S. Policies were found or no one was satisfied.");
		}

		//System.out.println("Fetching the Role Binding arraylist");
		//asm.sendMessage(new RoleBindingPolicyListReqMsg(asm.currentSubjectRole), "policyStorePort");
		asm.sendMessage(
				new RoleBindingPolicyListReqMsg(asm.currentSubjectRole),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));//"policyStorePort");
	}

	/*
	 * source vertex: waitForInvArrayList
	 * target vertex: checkRoleBindingCanBeAccepted
	 */
	public static void processRoleBindingArrayList(
			RoleBindingPolicyListReplyMsg signal, PolicyManagerSM asm) {
		asm.savedRoleBindingArrayListMsg = (RoleBindingPolicyListReplyMsg) signal;

		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework,
				asm.trace.getTraceHeader()
						+ "Checking RoleBinding Policies for role <<"
						+ signal.role + ">>");

		asm.roleBindingResult = checkRoleBindingPolicies(signal.rolebinding,
				asm.agentProfile, asm.currentSessionDescription);
	}

	public static boolean roleBindingAccepted(
			RoleBindingPolicyListReplyMsg signal, PolicyManagerSM asm) {
		return (asm.roleBindingResult == null);
	}

	/*
	 * source vertex: checkRoleBindingCanBeAccepted
	 * target vertex: waitForCollaborationArrayListReply
	 */
	public static void getCollaborationArrayList(
			RoleBindingPolicyListReplyMsg signal, PolicyManagerSM asm) {
		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework,
				asm.trace.getTraceHeader()
						+ "All RoleBinding Policies for role <<" + signal.role
						+ ">> were satisfied. Continue with binding process");

		//StatusMessageMsg sm = new StatusMessageMsg("Checking Collaboration Policy for: "+ signal.role);
		//asm.sendMessage(sm, asm.savedPolicyCheckRequest.getSenderRole());
		//System.out.println("Fetching the Collaboration arraylist");
		//asm.sendMessage(new CollaborationPolicyListReqMsg(signal.role), "policyStorePort");
		asm.sendMessage(new CollaborationPolicyListReqMsg(signal.role),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));//"policyStorePort");
	}

	public static boolean roleBindingNotAccepted(
			RoleBindingPolicyListReplyMsg signal, PolicyManagerSM asm) {
		return (asm.roleBindingResult != null);
	}

	/*
	 * source vertex: checkRoleBindingCanBeAccepted
	 * target vertex: waitForOnUnsArrayListReply
	 */
	public static void getOnUnsRoleBindingFSArrayList(
			RoleBindingPolicyListReplyMsg signal, PolicyManagerSM asm) {

		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework,
				asm.trace.getTraceHeader()
						+ "One of the RoleBinding Policies for role <<"
						+ signal.role
						+ ">> was not satisfied. The role cannot be bound!");

		//System.out.println("Fetching the OnUnsuccessfulRoleBinding arraylist");

		asm.altRoleAndParameters = null;

		//asm.sendMessage(new OnUnsRBPolicyListReqMsg(signal.role), "policyStorePort");
		asm.sendMessage(new OnUnsRBPolicyListReqMsg(signal.role),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));//"policyStorePort");
	}

	/*
	 * source vertex: waitForCollaborationArrayListReply
	 * target vertex: idle
	 */
	/*    public static void sendPolicyCheckResult(CollaborationArrayListReplyMsg signal, PolicyManagerSM asm){
	 CollaborationArrayListReplyMsg msg = (CollaborationArrayListReplyMsg)signal;
	 boolean a = checkRuntimePolicy(signal.collaboration, currentUserProfieLoadConfirmMsg);

	 if(a == true){
	 asm.sendMessage(new PolicyCheckAccepted(), asm.savedPolicyCheckRequest.getSenderRole());
	 }
	 else if(a == false){
	 asm.sendMessage(new PolicyCheckDenied(), asm.savedPolicyCheckRequest.getSenderRole());
	 }
	 }
	 */

	/*
	 * source vertex: waitForOnUnsArrayListReply
	 * target vertex: checkOnUns
	 */
	public static void processOnUnsArrayList(OnUnsRBPolicyListReplyMsg signal,
			PolicyManagerSM asm) {
		asm.trace
				.traceOut(
						TraceConstants.tlDebug,
						TraceConstants.tcFramework,
						asm.trace.getTraceHeader()
								+ "Checking OnUnsuccessfulRoleBinding F.S. Policies for role <<"
								+ signal.role + ">>");

		//asm.altRoleAndParameters = checkFeatureSelectionPolicies(signal.onuns, asm.agentProfile, asm.currentSessionDescription, asm.roleBindingResult);
		asm.altRoleAndParameters = checkFeatureSelectionPolicies(signal.onuns,
				asm.agentProfile, asm.currentSessionDescription, null);

		if (asm.altRoleAndParameters != null) {
			asm.currentSubjectRole = (String) asm.altRoleAndParameters.get(0);

			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "One OnUnsuccessfulRoleBinding F.S. Policy was satisfied. We will try to bind the role <<"
									+ asm.currentSubjectRole
									+ ">> instead of <<" + signal.role + ">>");
		} else {
			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "No OnUnsuccessfulRoleBinding F.S. Policies were found or no one was satisfied.");
		}
	}

	public static boolean altRoleOnUnsRBExist(OnUnsRBPolicyListReplyMsg signal,
			PolicyManagerSM asm) {
		return (asm.altRoleAndParameters != null);
	}

	/*
	 * source vertex: checkOnUns
	 * target vertex: waitForInvArrayList
	 */
	public static void getRoleBindingArrayList2(
			OnUnsRBPolicyListReplyMsg signal, PolicyManagerSM asm) {
		// TODO edit action statements
		//System.out.println("Fetching the RoleBinding arraylist on Unsuccessful role binding");
		//StatusMessageMsg smm = new StatusMessageMsg("Checking RoleBinding Policy on UnsuccessfulRoleBinding for: "+signal.role);
		//asm.sendMessage(smm, asm.savedPolicyCheckRequest.getSenderRole());
		asm.sendMessage(
				new RoleBindingPolicyListReqMsg(asm.currentSubjectRole),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));//"policyStorePort");
	}

	public static boolean altRoleOnUnsRBNotExist(
			OnUnsRBPolicyListReplyMsg signal, PolicyManagerSM asm) {
		return (asm.altRoleAndParameters == null);
	}

	public static void requestProfileData(PolicyManagerSM asm) {
		//The PolicyManager subscribes itself with the UserProfileEditor actor  
		//to receive an updated version of the userprofile whenever the latter is modified
		//asm.sendMessage(new ProfileLoadRequestMsg(true), "profileEditorPort");
		asm.sendMessage(new ProfileLoadRequestMsg(true), new ActorAddress(
				asm.context.getParentAddress().getActorID()
						+ "/myProfileEditor", "ProfileEditor"));

	}

	public static void updateAgentProfile(ProfileMsg signal, PolicyManagerSM asm) {
		asm.agentProfile = signal.profile;
	}

	public static void processCollabArrayList(
			CollaborationPolicyListReplyMsg signal, PolicyManagerSM asm) {
		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework,
				asm.trace.getTraceHeader()
						+ "Checking Collaboration Policies for role <<"
						+ signal.role + ">>");

		asm.roleBindingResult = checkCollaborationPolicies(
				signal.collaboration, asm.agentProfile,
				asm.currentSessionDescription);
	}

	public static boolean collabPoliciesOK(
			CollaborationPolicyListReplyMsg signal, PolicyManagerSM asm) {
		if (asm.roleBindingResult == null)
			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "All Collaboration Policies were satisfied. The role <<"
									+ signal.role + ">> will be bound");

		return (asm.roleBindingResult == null);
	}

	public static void sendPolicyCheckNOK(PolicyManagerSM asm) {
		asm.sendMessage(new PolicyCheckNOKMsg(), asm.checkRequestor);
	}

	public static boolean collabPoliciesNOK(
			CollaborationPolicyListReplyMsg signal, PolicyManagerSM asm) {
		if (asm.roleBindingResult != null)
			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "One of the Collaboration Policies was not satisfied.");

		return (asm.roleBindingResult != null);
	}

	public static void sendPolicyCheckOK(PolicyManagerSM asm) {
		Hashtable parameters = null;
		if (asm.altRoleAndParameters != null)
			parameters = (Hashtable) asm.altRoleAndParameters.get(1);

		asm.sendMessage(
				new PolicyCheckOKMsg(asm.currentSubjectRole, parameters),
				asm.checkRequestor);
	}

	public static void processOnNonAchArray(OnNonAGPolicyListReplyMsg signal,
			PolicyManagerSM asm) {
		Node reason;
		asm.altRoleAndParameters = null;
		try {
			reason = asm.policyConditionParser
					.parseCondition(asm.savedPolicyCheckOnGnotAReqMsg.reason);

			asm.trace
					.traceOut(
							TraceConstants.tlDebug,
							TraceConstants.tcFramework,
							asm.trace.getTraceHeader()
									+ "Checking OnNonAchievedGoal F.S. Policies for role <<"
									+ signal.role + ">>");

			asm.altRoleAndParameters = checkFeatureSelectionPolicies(
					signal.onNon, asm.agentProfile,
					asm.currentSessionDescription, reason);

			if (asm.altRoleAndParameters != null) {
				asm.currentSubjectRole = (String) asm.altRoleAndParameters
						.get(0);

				asm.trace
						.traceOut(
								TraceConstants.tlDebug,
								TraceConstants.tcFramework,
								asm.trace.getTraceHeader()
										+ "One OnNonAchievedRoleGoal F.S. Policy was satisfied. We will try to bind the role <<"
										+ asm.currentSubjectRole
										+ ">> instead of <<" + signal.role
										+ ">>");
			} else {
				asm.trace
						.traceOut(
								TraceConstants.tlDebug,
								TraceConstants.tcFramework,
								asm.trace.getTraceHeader()
										+ "No OnNonAchievedRoleGoal F.S. Policies were found or no one was satisfied.");
			}

		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void getOnNonAchGoalFSArrayList(
			PolicyCheckOnGnotAReqMsg signal, PolicyManagerSM asm) {
		//System.out.println("Fetching the OnNonAchievedRoleGoal arraylist");
		//StatusMessageMsg smm = new StatusMessageMsg("Checking OnNonAchievedGoal Feature-selection Policies for: "+signal.subjectRole);
		//asm.sendMessage(smm, asm.savedPolicyCheckRequest.getSenderRole());

		asm.savedPolicyCheckOnGnotAReqMsg = signal;
		asm.currentSessionDescription = signal.sessionDescription;
		asm.checkRequestor = signal.getSenderRole();

		asm.currentSubjectRole = signal.roleType;
		//asm.sendMessage(new OnNonAGPolicyListReqMsg(signal.roleType), "policyStorePort");
		asm.sendMessage(new OnNonAGPolicyListReqMsg(signal.roleType),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));//"policyStorePort");
	}

	public static void getRoleBindingArrayListOnNonAchG(PolicyManagerSM asm) {
		//System.out.println("Fetching the Role Binding arraylist for NonAchievedGoal");
		//asm.sendMessage(new RoleBindingPolicyListReqMsg(asm.currentSubjectRole), "policyStorePort");
		asm.sendMessage(
				new RoleBindingPolicyListReqMsg(asm.currentSubjectRole),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));//"policyStorePort");
	}

	public static boolean foundOnNonAchFS(OnNonAGPolicyListReplyMsg signal,
			PolicyManagerSM asm) {
		return (asm.altRoleAndParameters != null);
	}

	public static boolean notFoundOnNonAchFS(OnNonAGPolicyListReplyMsg signal,
			PolicyManagerSM asm) {
		return (asm.altRoleAndParameters == null);
	}

	public static void getOnUnsRoleBindingFSArrayList2(
			CollaborationPolicyListReplyMsg signal, PolicyManagerSM asm) {
		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework,
				asm.trace.getTraceHeader()
						+ "One of the Collaboration Policies for role <<"
						+ signal.role
						+ ">> was not satisfied. The role cannot be bound!");

		//System.out.println("Fetching the OnUnsuccessfulRoleBinding arraylist");

		asm.altRoleAndParameters = null;

		asm.sendMessage(new OnUnsRBPolicyListReqMsg(signal.role),
				new ActorAddress(asm.context.getParentAddress().getActorID()
						+ "/myPolicyStore", "PolicyStore"));
	}

}
