package no.ntnu.item.serviceframe.actors.serviceregistry.serviceregistry;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.ntnu.item.serviceframe.auxiliar.servicedescription.ServiceDescription;
import no.ntnu.item.serviceframe.signals.serviceregistry.LookupServicesMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.LookupServicesResultMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.RegServiceAckMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.RegServiceMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.ServiceRegistrationNotificationAckMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.ServiceRegistrationNotificationMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.ServiceRegistrationNotificationReqMsg;
import se.ericsson.eto.norarc.ejbframe.ActorAddress;


public class ServiceRegistryActions extends Object {

	public static void registerService(RegServiceMsg signal, ServiceRegistrySM asm){
	    asm.serviceList.add(signal.serviceDescription);
	    asm.sendMessage(new RegServiceAckMsg(), signal.getSenderRole());
	    
    	// Notify all subscribers about the new available service
	    ArrayList newServices = new ArrayList();
	    newServices.add(signal.serviceDescription);
	    for (int i=0; i<asm.notificationSubscribers.size(); i++) {
    		asm.sendMessage(new ServiceRegistrationNotificationMsg(newServices), (ActorAddress)asm.notificationSubscribers.get(i));
        }
	}

	public static void sendLookupResult(LookupServicesMsg signal, ServiceRegistrySM asm){
	    ArrayList sl = new ArrayList();
		if (signal.keyword.equalsIgnoreCase("*")) {
	    	for (int i = 0; i<asm.serviceList.size(); i++) {
	    		sl.add(((ServiceDescription)asm.serviceList.get(i)).clone());
	    	}
	    }
		else {
            // use regular expression on actor types
            Pattern pType = Pattern.compile("*"+signal.keyword+"*", Pattern.CASE_INSENSITIVE); // the pattern search string
            Matcher m = null;
            for (int i = 0; i < asm.serviceList.size(); i++) {
                ServiceDescription d = (ServiceDescription) asm.serviceList.get(i);

                //Try first to match the serviceID
                m = pType.matcher(d.serviceID);
                if (m.matches()) {
                    sl.add(d.clone());
                }
                else { //Try to find the keyword on the description
                    m = pType.matcher(d.description);
                    if (m.matches()) {
                        sl.add(d.clone());
                    }
                }
                	
            }
		}
		
		asm.sendMessage(new LookupServicesResultMsg(sl), signal.getSenderRole());
	    	
	}

	public static void processSubscription(ServiceRegistrationNotificationReqMsg signal, ServiceRegistrySM asm){
	    asm.notificationSubscribers.add(signal.address);
	    asm.sendMessage(new ServiceRegistrationNotificationAckMsg(), signal.getSenderRole());
	}



}
