package no.ntnu.item.serviceframe.actors.policymanagement.policymanager;


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Hashtable;

import se.ericsson.eto.norarc.ejbframe.ActorAddress;
import se.ericsson.eto.norarc.ejbframe.TraceConstants;

import no.ntnu.item.serviceframe.auxiliar.agentprofile.AgentProfile;
import no.ntnu.item.serviceframe.auxiliar.agentprofile.UserProfile;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.Node;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAge;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAnd;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAtomicPredicate;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAvailabilityEntity;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeAvailabilityUser;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeDevice;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeFalse;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeLocation;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeNoAnswer;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeNot;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeOr;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeScreeningRoleList;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeScreeningUserList;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeStartCondition;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeSubscription;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTime;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTimeBtw;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTimeDate;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTimeDay;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.PCNodeTrue;
import no.ntnu.item.serviceframe.auxiliar.policyconditionparser.ParseException;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.CollaborationPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.OnRRFSPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.policyobjects.RoleBindingPolicyObject;
import no.ntnu.item.serviceframe.auxiliar.sessiondescription.SessionDescription;
import no.ntnu.item.serviceframe.signals.agentprofile.ProfileLoadRequestMsg;
import no.ntnu.item.serviceframe.signals.agentprofile.ProfileMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.CollaborationArrayListReplyMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.CollaborationArrayListReqMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.OnNonAGArrayListReplyMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.OnNonAGArrayListReqMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.OnRRArrayListReplyMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.OnRRArrayListReqMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.OnUnsRBArrayListReplyMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.OnUnsRBArrayListReqMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.RoleBindingArrayListReplyMsg;
import no.ntnu.item.serviceframe.signals.getarraylist.RoleBindingArrayListReqMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckNOKMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOKMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOnGnotAReqMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOnRRReqMsg;
public class PolicyManagerActions extends Object {
	public static Calendar rightNow = Calendar.getInstance();
	public static ArrayList currentArrayList;


//	+++++++++++ AUXILIARY METHODS ++++++++++++++++++++

    /*
     * This method checks if all Role Binding policies are satisfied. If so, 
     * it returns "null". Otherwise, it returns the reason (i.e. the negation of the condition not satisfied)
     * 
     */
	private static Node checkRoleBindingPolicies(ArrayList array, AgentProfile agentProfile, SessionDescription sessionDesc){
    	//System.out.println("Checking the Role Binding policies");
    	Node result = null;
    	if(array != null){
    		
    		RoleBindingPolicyObject onRB;
    		Node condition;
    		
    		for (int i = 0; i<array.size(); i++) {
    			
    			onRB = (RoleBindingPolicyObject)array.get(i);
    			condition = onRB.getCondition();
    			
    			// All policies must be satisfied
    			if (evaluateCondition(condition, null, agentProfile, sessionDesc) == false) {
    				//Now we negate the condition. The negated condition will be the reason why the policy was not satisfied
    				PCNodeNot not = new PCNodeNot(4422); //We pass any number
    				not.jjtAddChild(condition.jjtGetChild(0),-1);
    				result = not;
    				break;
    			}
    				
    		}
    	}
    	return result;
    }

	/*
     * This method checks if there exists any Feature-Selection policy in the ipout list of 
     * list of policies that is satisfied. If so, it returns an ArrayList containing  
     * the type of the alternative role to be played (first position) and a set of parameters 
     * that may be needed by the role (second position). Otherwise, it returns "null"
     * 
     */
	private static ArrayList checkFeatureSelectionPolicies(ArrayList array, AgentProfile agentProfile, SessionDescription sessionDesc, Node selectionReason){
    	//System.out.println("CHECKING Feature Selection policies...");
		ArrayList result = null;

    	if(array != null){
        	//System.out.println("CHECKING "+(array.get(0)).toString());
    	
    		Node condition;
    		OnRRFSPolicyObject onRR;
    		
    		for (int i = 0; i<array.size(); i++) {
    			onRR = (OnRRFSPolicyObject)array.get(i);
    			condition = onRR.getCondition();
    			if (evaluateCondition(condition, selectionReason, agentProfile, sessionDesc) == true){
    				result = new ArrayList();
    				result.add(onRR.getAlternativeRole());
    				result.add(onRR.getParameters());
    				break;
    			}
    		}
    	}
   		return result;
    }


	/*
     * This method checks if all the Collaboration policies are satisfied
     * If so, it returns "null". Otherwise, it returns the reason 
     * (i.e. the negation of the condition not satisfied)
     * Right now it is not generic enough (e.g. only "peerUser" is passed as input
     * parameter, but other input parameter may be needed to check certain policies
     * Parameter-passing should be made generic (TO BE DONE!)
     */
	private static Node checkCollaborationPolicies(ArrayList array, AgentProfile agentProfile, SessionDescription sessionDesc){
    	//System.out.println("CHECKING the Collaboration policies");
    	Node result = null;
    	
    	if(array != null){
    		
    		CollaborationPolicyObject collabObj;
    		Node condition;
    		
    		for (int i = 0; i<array.size(); i++) {
    			
    			collabObj = (CollaborationPolicyObject)array.get(i);
    			condition = collabObj.getCondition();
    			
    			if (evaluateCondition(condition, null, agentProfile, sessionDesc) == false){
    				//Now we negate the condition. The negated condition will be the reason why the policy was not satisfied
    				PCNodeNot not = new PCNodeNot(2244); //We pass any number
    				not.jjtAddChild(condition.jjtGetChild(0),-1);
    				result = not;
    				break;
    			}
    		}
    	}
    	return result;
    }

    private static boolean evaluateCondition(Node condition, Node selectionReason, AgentProfile agentProfile, SessionDescription sessionDesc)
    {
    	boolean result=false; 
   	
    	if (condition instanceof PCNodeStartCondition) {
    		//System.out.println("We have a ROOT");
    		result = evaluateCondition(condition.jjtGetChild(0), selectionReason, agentProfile, sessionDesc);
    		//System.out.println("Result ROOT is: "+result);
    	} 
    	else if (condition instanceof PCNodeAnd) {
    		//System.out.println("We have an AND");
    		result = true;
    		for (int i=0; i<condition.jjtGetNumChildren();i++) {
    			//All predicates must be true
    			if (evaluateCondition(condition.jjtGetChild(i), selectionReason, agentProfile, sessionDesc)==false) {
    				result = false;
    				break;
    			}
    		}
    		//System.out.println("Result AND is: "+result);
    	} 
    	else if (condition instanceof PCNodeOr) {
    		//System.out.println("We have an OR");
    		result = false;
    		for (int i=0; i<condition.jjtGetNumChildren();i++) {
    			//At least one predicate has to be true
    			if (evaluateCondition(condition.jjtGetChild(i), selectionReason, agentProfile, sessionDesc)==true) {
    				result = true;
    				break;
    			}
    		}
    		//System.out.println("Result OR is: "+result);
    	}
    	else if (condition instanceof PCNodeNot) {
    		//System.out.println("We have a NOT");
    		result = !evaluateCondition(condition.jjtGetChild(0), selectionReason, agentProfile, sessionDesc);
    		//System.out.println("Result NOT is: "+result);
    	} 
    	else if (condition instanceof PCNodeAtomicPredicate) {
    		//System.out.println("We have an AtomicPredicate");
    		result = evaluateAtomicPredicate(condition.jjtGetChild(0), selectionReason, agentProfile, sessionDesc);
    		//System.out.println("Result AtomicPredicate is: "+result);
    	}
    	
    	
    	return result;
   	}

/*
 * This method is used to avaluate atomic predicates
 * TO BE DONE
 * Right now it is though for checking policies associated with a UserAgent
 * However, for each type of agent the checks may be different. Therefore there
 * should be some way to specialize this method for each kind of agent. One 
 * possibility is to have an inner actor to the PolicyManager implementing this method.
 * Then, we may used specialized inner actors for each kind af agent
 */
    private static boolean evaluateAtomicPredicate(Node predicate, Node selectionReason, AgentProfile agentProfile, SessionDescription sessionDesc){

    	if (selectionReason!=null) {
    		//We deal with an OnNonAchievedGoal policy, so we check first the condition against the selectionReason
        	
    		if (predicate instanceof PCNodeTrue) {
        		//System.out.println("We have a PCNodeTrue");
        		return true;
        	} 
        	else if (predicate instanceof PCNodeFalse) {
            	//System.out.println("We have a PCNodeFalse");
            	return false;
            }
        	else if (predicate instanceof PCNodeNoAnswer) {
        		//System.out.println("We have a PCNodeNoAnswer");
        		if (selectionReason instanceof PCNodeNoAnswer)
        			return true;
        		else
        			return false;
        	}
    	} //The condition of a OnNonAchievedGoal policy may also be expressed in terms of time, location, etc. Thus, we continue checking 
    	

    	if (predicate instanceof PCNodeTime) {
    		//System.out.println("We have a PCNodeTime");
    		PCNodeTime t = (PCNodeTime)predicate;
    		if (t.getOperator().equalsIgnoreCase("=")){
    			if (rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()){
    				return true;
    			}
    			else{
    				return false;
    			}
    		}
    		else if(t.getOperator().equalsIgnoreCase("!=")){
    			if( (rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow.get(Calendar.MINUTE) == t.getMinutes()) ){
    				return false;
    			}
    			return true;
    		}
    		else if (t.getOperator().equalsIgnoreCase("<")){
    			if ( (rightNow.get(Calendar.HOUR_OF_DAY) < t.getHour()) ||
    					((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow.get(Calendar.MINUTE) < t.getMinutes())) ){
    				return true;
    			}
    			return false;
    		}
    		else if (t.getOperator().equalsIgnoreCase(">")){
    			if ( (rightNow.get(Calendar.HOUR_OF_DAY) > t.getHour()) ||
    					((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow.get(Calendar.MINUTE) > t.getMinutes())) ){
    				return true;
    			}
    			return false;
    		}
    		else if (t.getOperator().equalsIgnoreCase("<=")){
    			if ( (rightNow.get(Calendar.HOUR_OF_DAY) < t.getHour()) ||
    					((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow.get(Calendar.MINUTE) <= t.getMinutes())) ){
    				return true;
    			}
    			return false;
    		}
    		else if(t.getOperator().equalsIgnoreCase(">=")){
    			if ( (rightNow.get(Calendar.HOUR_OF_DAY) > t.getHour()) ||
    					((rightNow.get(Calendar.HOUR_OF_DAY) == t.getHour()) && (rightNow.get(Calendar.MINUTE) >= t.getMinutes())) ){
    				return true;
    			}
    			return false;
    		}
    	}
    	else if (predicate instanceof PCNodeTimeBtw) {
    		//System.out.println("We have a PCNodeTimeBtw");
    		PCNodeTimeBtw t = (PCNodeTimeBtw)predicate;
    		if (rightNow.get(Calendar.HOUR_OF_DAY) >= t.getStartHour() && rightNow.get(Calendar.HOUR_OF_DAY) <= t.getEndHour()){
    			if(rightNow.get(Calendar.MINUTE)>= t.getStartMinutes() && rightNow.get(Calendar.MINUTE) <= t.getEndMinutes()){
    				return true;
    			}
    			else{
    				return false;
    			}
    		}
    		else{
    			return false;
    		}
    	}
    	else if (predicate instanceof PCNodeTimeDay) {
    		//System.out.println("We have a PCNodeTimeDay");
    		PCNodeTimeDay t = (PCNodeTimeDay)predicate;
    		int weekday = 0;
    		if (t.getWeekday().equalsIgnoreCase("monday")){
    			weekday = 2;
    		}
    		else if (t.getWeekday().equalsIgnoreCase("tuesday")){
    			weekday = 3;
    		}
    		else  if (t.getWeekday().equalsIgnoreCase("wednesday")){
    			weekday = 4;
    		}
    		else  if (t.getWeekday().equalsIgnoreCase("thursday")){
    			weekday = 5;
    		}
    		else  if (t.getWeekday().equalsIgnoreCase("friday")){
    			weekday = 6;
    		}
    		else  if (t.getWeekday().equalsIgnoreCase("saturday")){
    			weekday = 7;
    		}
    		else  if (t.getWeekday().equalsIgnoreCase("sunday")){
    			weekday = 8;
    		}
    		int day = rightNow.get(Calendar.DAY_OF_WEEK);
    		if (weekday == day){
    			return true;
    		}
    		return false;
    	}
    	else if (predicate instanceof PCNodeTimeDate) {
    		//System.out.println("We have a PCNodeTimeDate");
    		PCNodeTimeDate t = (PCNodeTimeDate)predicate;
    		if (t.getYear() == rightNow.get(Calendar.YEAR)){
    			if (t.getMonth() == rightNow.get(Calendar.MONTH)){
    				if (t.getDay() == rightNow.get(Calendar.DAY_OF_MONTH)){
    					return true;
    				}
    			}
    		}
    		return false;
    	}
    	else if (predicate instanceof PCNodeAge) {
    		//System.out.println("We have a PCNodeAge");
    		PCNodeAge t = (PCNodeAge)predicate;
    		// CHECK INTENDED FOR A USERAGENT -> WE USE USERPROFILE
    		if (t.getAge() == (((UserProfile) agentProfile).getAge())){
    			return true;
    		}
    		return false;
    	}
    	else if (predicate instanceof PCNodeSubscription) {
    		//System.out.println("We have a PCNodeSubscription");
    		PCNodeSubscription t = (PCNodeSubscription)predicate;
    		//for(int j = 0; j < userProfile.getSubscriptions().size(); j++){
    		//	if(value.equalsIgnoreCase((String) userProfile.getSubscriptions().get(j))){
    		//		return true;
    		//	}
    		//}
    		// CHECK INTENDED FOR A USERAGENT -> WE USE USERPROFILE
    		if (((UserProfile) agentProfile).getSubscriptions().contains(t.getSubscription())){
    			return true;
    		}
    		return false;
    	}
    	else if (predicate instanceof PCNodeScreeningRoleList) {
    		//System.out.println("We have a PCNodeScreeningRoleList");
    		//TO BE DONE
    		//We should check if the screening list contains any role already participating
    		//in the session or any role that has been invited to it. If so, return false.
    		return true;
    	} 
    	else if (predicate instanceof PCNodeScreeningUserList) {
    		//System.out.println("We have a PCNodeScreeningUserList");
    		//OBS! A condition containing based on a screening list can be seen as
    		//a condition of type "NOT listOfUsers". Therefore, we return "false" ("NOT true")
    		//if any user participating in the session is on the screening list
    		PCNodeScreeningUserList sl = (PCNodeScreeningUserList)predicate;
    		if (sl.getUserList().contains(sessionDesc.contactedUserName) || sl.getUserList().contains(sessionDesc.initiatorUserName))
    			return false;
    		else
    			return true;
    	} 
    	else if (predicate instanceof PCNodeAvailabilityUser) {
    		//System.out.println("We have a PCNodeAvailabilityUser");
    		PCNodeAvailabilityUser a = (PCNodeAvailabilityUser)predicate;
    		//We don't look at the user id. We assume the availability condition applies to the owner of the profile
    		//If necessary, change this.
    		if ( ((UserProfile) agentProfile).getUserAvailability() == a.getUserStatus()){ 
    			return true;
    		}
    		return false;
    	} 
    	else if (predicate instanceof PCNodeAvailabilityEntity) {
    		//System.out.println("We have a PCNodeAvailabilityEntity");
    		PCNodeAvailabilityEntity a = (PCNodeAvailabilityEntity)predicate;
    		if ( ((UserProfile) agentProfile).getTerminalStatus(a.getEntityId()) == a.getEntityStatus()){ 
    			return true;
    		}
    		return false;
    	} 
    	else if (predicate instanceof PCNodeLocation) {
    		//System.out.println("We have a PCNodeLocation");
    		PCNodeLocation l = (PCNodeLocation)predicate;
    		if (l.getLocation().equalsIgnoreCase(((UserProfile) agentProfile).getLocation()))
    			return true;
    		else
    			return false;
    	}
    	else if (predicate instanceof PCNodeDevice) {
    		//System.out.println("We have a PCNodeNoDevice");
    		PCNodeDevice d = (PCNodeDevice)predicate;
    		//TO BE DONE
    		//Not clear how to deal with this predicate
    		//To be compared with sessionDescription info?
    		return true;
    	}
    	else if (predicate instanceof PCNodeTrue) {
    		//System.out.println("We have a PCNodeTrue");
    		return true;
    	} 
    	else if (predicate instanceof PCNodeFalse) {
    		//System.out.println("We have a PCNodeFalse");
    		return false;
    	}
    	
    	return false;
    }
    
	
	
//+++++++++ END OF AUXILIARY METHODS ++++++++++++++++++++




    /*
     * source vertex: waitForData
     * target vertex: waitForOnRRArrayList
     */
    public static void getOnRRArrayList(PolicyCheckOnRRReqMsg signal, PolicyManagerSM asm){
    	//System.out.println("PolicyManager: Fetching the OnRR arraylist");
    	//StatusMessageMsg smm = new StatusMessageMsg("Checking OnRoleRequest Feature-selection Policy for: " + asm.savedPolicyCheckRequest.RoleRequest.getRoleType());
    	//asm.sendMessage(smm, asm.savedPolicyCheckRequest.getSenderRole());
    	asm.checkRequestor=signal.getSenderRole();
    	asm.currentSessionDescription = signal.sessionDescription;
    	asm.currentSubjectRole = signal.roleType;
    	asm.sendMessage(new OnRRArrayListReqMsg(signal.roleType), "policyStorePort");
    	//asm.sendMessage(new OnRRArrayListReqMsg(signal.roleType), new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));
    	
    }



    /*
     * source vertex: waitForOnRRArrayList
     * target vertex: waitForRoleBindingArrayList
     */
    public static void getRoleBindingArrayList(OnRRArrayListReplyMsg signal, PolicyManagerSM asm){
    	// Before trying to bind the requested role, we check if there is any 
    	// OnRoleRequest Feature-Selection policy that holds
    	//System.out.println("Checking OnRoleRequest policies");
    	
    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "Checking OnRoleRequest F.S. Policies for role <<"+signal.role+">>");
    	
    	asm.altRoleAndParameters = checkFeatureSelectionPolicies(signal.onrr, asm.agentProfile, asm.currentSessionDescription, null);
    	
    	if(asm.altRoleAndParameters != null){
			asm.currentSubjectRole = (String) asm.altRoleAndParameters.get(0);
	    	//System.out.println("OnRoleRequest policy found. We will try to bind an alternative role");
	    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
	                + "One OnRoleRequest F.S. Policy was satisfied. We will try to bind the role <<"+asm.currentSubjectRole+">> instead of <<"+signal.role+">>");
    	}
    	else {
	    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
	                + "No OnRoleRequest F.S. Policies were found or no one was satisfied.");
    	}
    		
    	
    	//System.out.println("Fetching the Role Binding arraylist");
    	asm.sendMessage(new RoleBindingArrayListReqMsg(asm.currentSubjectRole), "policyStorePort");
    	//asm.sendMessage(new RoleBindingArrayListReqMsg(asm.currentSubjectRole), new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));//"policyStorePort");
    }


    /*
     * source vertex: waitForInvArrayList
     * target vertex: checkRoleBindingCanBeAccepted
     */
    public static void processRoleBindingArrayList(RoleBindingArrayListReplyMsg signal, PolicyManagerSM asm){
    	asm.savedRoleBindingArrayListMsg = (RoleBindingArrayListReplyMsg)signal;

    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "Checking RoleBinding Policies for role <<"+signal.role+">>");
    	
    	asm.roleBindingResult = checkRoleBindingPolicies(signal.rolebinding, asm.agentProfile, asm.currentSessionDescription);
    }

    public static boolean roleBindingAccepted(RoleBindingArrayListReplyMsg signal, PolicyManagerSM asm){
    	return (asm.roleBindingResult == null);
    }

    /*
     * source vertex: checkRoleBindingCanBeAccepted
     * target vertex: waitForCollaborationArrayListReply
     */
    public static void getCollaborationArrayList(RoleBindingArrayListReplyMsg signal, PolicyManagerSM asm){
    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "All RoleBinding Policies for role <<"+signal.role+">> were satisfied. Continue with binding process");

    	//StatusMessageMsg sm = new StatusMessageMsg("Checking Collaboration Policy for: "+ signal.role);
    	//asm.sendMessage(sm, asm.savedPolicyCheckRequest.getSenderRole());
		//System.out.println("Fetching the Collaboration arraylist");
		asm.sendMessage(new CollaborationArrayListReqMsg(signal.role), "policyStorePort");
		//asm.sendMessage(new CollaborationArrayListReqMsg(signal.role), new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));//"policyStorePort");
    }

    public static boolean roleBindingNotAccepted(RoleBindingArrayListReplyMsg signal, PolicyManagerSM asm){
    	return (asm.roleBindingResult != null);
    }

    /*
     * source vertex: checkRoleBindingCanBeAccepted
     * target vertex: waitForOnUnsArrayListReply
     */
    public static void getOnUnsRoleBindingFSArrayList(RoleBindingArrayListReplyMsg signal, PolicyManagerSM asm){

    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "One of the RoleBinding Policies for role <<"+signal.role+">> was not satisfied. The role cannot be bound!");

    	//System.out.println("Fetching the OnUnsuccessfulRoleBinding arraylist");
    	
    	asm.altRoleAndParameters = null;
    	
    	asm.sendMessage(new OnUnsRBArrayListReqMsg(signal.role), "policyStorePort");
    	//asm.sendMessage(new OnUnsRBArrayListReqMsg(signal.role), new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));//"policyStorePort");
    }

    /*
     * source vertex: waitForCollaborationArrayListReply
     * target vertex: idle
     */
/*    public static void sendPolicyCheckResult(CollaborationArrayListReplyMsg signal, PolicyManagerSM asm){
    	CollaborationArrayListReplyMsg msg = (CollaborationArrayListReplyMsg)signal;
    	boolean a = checkRuntimePolicy(signal.collaboration, currentUserProfieLoadConfirmMsg);

    	if(a == true){
    		asm.sendMessage(new PolicyCheckAccepted(), asm.savedPolicyCheckRequest.getSenderRole());
    	}
    	else if(a == false){
    		asm.sendMessage(new PolicyCheckDenied(), asm.savedPolicyCheckRequest.getSenderRole());
    	}
    }
*/

    /*
     * source vertex: waitForOnUnsArrayListReply
     * target vertex: checkOnUns
     */
    public static void processOnUnsArrayList(OnUnsRBArrayListReplyMsg signal, PolicyManagerSM asm){
    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "Checking OnUnsuccessfulRoleBinding F.S. Policies for role <<"+signal.role+">>");

    	asm.altRoleAndParameters = checkFeatureSelectionPolicies(signal.arraylist, asm.agentProfile, asm.currentSessionDescription, asm.roleBindingResult);
    	
    	if (asm.altRoleAndParameters!=null) {
    		asm.currentSubjectRole = (String) asm.altRoleAndParameters.get(0);

    		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
	                + "One OnUnsuccessfulRoleBinding F.S. Policy was satisfied. We will try to bind the role <<"+asm.currentSubjectRole+">> instead of <<"+signal.role+">>");
    	} 
    	else {
	    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
	                + "No OnUnsuccessfulRoleBinding F.S. Policies were found or no one was satisfied.");
    	}
    }

    public static boolean altRoleOnUnsRBExist(OnUnsRBArrayListReplyMsg signal, PolicyManagerSM asm){
    	return (asm.altRoleAndParameters != null);
    }

    /*
     * source vertex: checkOnUns
     * target vertex: waitForInvArrayList
     */
    public static void getRoleBindingArrayList2(OnUnsRBArrayListReplyMsg signal, PolicyManagerSM asm){
        // TODO edit action statements
    	//System.out.println("Fetching the RoleBinding arraylist on Unsuccessful role binding");
    	//StatusMessageMsg smm = new StatusMessageMsg("Checking RoleBinding Policy on UnsuccessfulRoleBinding for: "+signal.role);
    	//asm.sendMessage(smm, asm.savedPolicyCheckRequest.getSenderRole());
		asm.sendMessage(new RoleBindingArrayListReqMsg(asm.currentSubjectRole) , new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));//"policyStorePort");
    }

    public static boolean altRoleOnUnsRBNotExist(OnUnsRBArrayListReplyMsg signal, PolicyManagerSM asm){
    	return (asm.altRoleAndParameters == null);
    }



    public static void requestProfileData(PolicyManagerSM asm){
		//The PolicyManager subscribes itself with the UserProfileEditor actor  
		//to receive an updated version of the userprofile whenever the latter is modified
		asm.sendMessage(new ProfileLoadRequestMsg(true), "profileEditorPort");
    	//asm.sendMessage(new ProfileLoadRequestMsg(true), new ActorAddress(asm.context.getParentAddress().getActorID()+"/profileEditor", "ProfileEditor"));
    	
	}

	public static void updateAgentProfile(ProfileMsg signal, PolicyManagerSM asm){
	    asm.agentProfile = signal.profile;
	}

	public static void processCollabArrayList(CollaborationArrayListReplyMsg signal, PolicyManagerSM asm){
    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "Checking Collaboration Policies for role <<"+signal.role+">>");

    	asm.roleBindingResult = checkCollaborationPolicies(signal.collaboration, asm.agentProfile, asm.currentSessionDescription);
	}

	public static boolean collabPoliciesOK(CollaborationArrayListReplyMsg signal, PolicyManagerSM asm){
	    if (asm.roleBindingResult == null)
	    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "All Collaboration Policies were satisfied. The role <<"+signal.role+">> will be bound");
		
		return (asm.roleBindingResult == null);
	}

	public static void sendPolicyCheckNOK(PolicyManagerSM asm){
    	asm.sendMessage(new PolicyCheckNOKMsg(), asm.checkRequestor);
	}

	public static boolean collabPoliciesNOK(CollaborationArrayListReplyMsg signal, PolicyManagerSM asm){
	    if (asm.roleBindingResult != null)
	    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "One of the Collaboration Policies was not satisfied.");

	    return (asm.roleBindingResult != null);
	}

	public static void sendPolicyCheckOK(PolicyManagerSM asm){
    	Hashtable parameters = null;
    	if (asm.altRoleAndParameters != null)
    		parameters = (Hashtable)asm.altRoleAndParameters.get(1);
    	
   		asm.sendMessage(new PolicyCheckOKMsg(asm.currentSubjectRole, parameters), asm.checkRequestor);
	}

	public static void processOnNonAchArray(OnNonAGArrayListReplyMsg signal, PolicyManagerSM asm){
    	Node reason;
    	asm.altRoleAndParameters = null;
		try {
			reason = asm.policyConditionParser.parseCondition(asm.savedPolicyCheckOnGnotAReqMsg.reason);

	    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
	                + "Checking OnNonAchievedRoleGoal F.S. Policies for role <<"+signal.role+">>");

	    	asm.altRoleAndParameters = checkFeatureSelectionPolicies(signal.onNon, asm.agentProfile, asm.currentSessionDescription, reason);
	    	
	    	if (asm.altRoleAndParameters!=null) {
	    		asm.currentSubjectRole = (String) asm.altRoleAndParameters.get(0);

	    		asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
		                + "One OnNonAchievedRoleGoal F.S. Policy was satisfied. We will try to bind the role <<"+asm.currentSubjectRole+">> instead of <<"+signal.role+">>");
	    	} 
	    	else {
		    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
		                + "No OnNonAchievedRoleGoal F.S. Policies were found or no one was satisfied.");
	    	}
	    		
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void getOnNonAchGoalFSArrayList(PolicyCheckOnGnotAReqMsg signal, PolicyManagerSM asm){
    	//System.out.println("Fetching the OnNonAchievedRoleGoal arraylist");
    	//StatusMessageMsg smm = new StatusMessageMsg("Checking OnNonAchievedGoal Feature-selection Policies for: "+signal.subjectRole);
    	//asm.sendMessage(smm, asm.savedPolicyCheckRequest.getSenderRole());

    	asm.savedPolicyCheckOnGnotAReqMsg=signal;
    	asm.currentSessionDescription = signal.sessionDescription;
    	asm.checkRequestor=signal.getSenderRole();
    	
    	asm.currentSubjectRole = signal.roleType;
    	asm.sendMessage(new OnNonAGArrayListReqMsg(signal.roleType), "policyStorePort");
    	//asm.sendMessage(new OnNonAGArrayListReqMsg(signal.roleType), new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));//"policyStorePort");
	}

	public static void getRoleBindingArrayListOnNonAchG(PolicyManagerSM asm){
    	//System.out.println("Fetching the Role Binding arraylist for NonAchievedGoal");
    	asm.sendMessage(new RoleBindingArrayListReqMsg(asm.currentSubjectRole), "policyStorePort");
    	//asm.sendMessage(new RoleBindingArrayListReqMsg(asm.currentSubjectRole), new ActorAddress(asm.context.getParentAddress().getActorID()+"/policyStore", "PolicyStore"));//"policyStorePort");
	}

	public static boolean foundOnNonAchFS(OnNonAGArrayListReplyMsg signal, PolicyManagerSM asm){
		return (asm.altRoleAndParameters!=null);
	}

	public static boolean notFoundOnNonAchFS(OnNonAGArrayListReplyMsg signal, PolicyManagerSM asm){
		return (asm.altRoleAndParameters==null);
	}

	public static void getOnUnsRoleBindingFSArrayList2(CollaborationArrayListReplyMsg signal, PolicyManagerSM asm){
    	asm.trace.traceOut(TraceConstants.tlDebug, TraceConstants.tcFramework, asm.trace.getTraceHeader()
                + "One of the Collaboration Policies for role <<"+signal.role+">> was not satisfied. The role cannot be bound!");

    	//System.out.println("Fetching the OnUnsuccessfulRoleBinding arraylist");
    	
    	asm.altRoleAndParameters = null;
    	
    	asm.sendMessage(new OnUnsRBArrayListReqMsg(signal.role), "policyStorePort");
	}









	
}
