package no.ntnu.item.serviceframe.actors.terminalagent.terminalclientsession;

import java.util.ArrayList;
import java.util.Hashtable;

import no.ntnu.item.serviceframe.auxiliar.sessiondescription.SessionDescription;
import no.ntnu.item.serviceframe.signals.terminalclientlogon.LogonErr;
import no.ntnu.item.serviceframe.signals.terminalclientlogon.LogonReq;
import no.ntnu.item.serviceframe.signals.terminalclientlogon.LogonRes;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleConfirmMsg;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleRequestMsg;
import se.ericsson.eto.norarc.ejbframe.ActorAddress;
import no.ntnu.item.serviceframe.signals.useragentauthentication.UserAuthenticationReq;
import no.ntnu.item.serviceframe.signals.useragentauthentication.UserAuthenticationValid;
import no.ntnu.item.serviceframe.signals.useragentauthentication.UserAuthenticationInvalid;
import no.ntnu.item.serviceframe.signals.terminalclientlogout.LogoutReq;
import no.ntnu.item.serviceframe.signals.handshake.HelloMsg;
import no.ntnu.item.serviceframe.signals.servicesession.ServiceSessionReqMsg;
import no.ntnu.item.serviceframe.signals.handshake.HelloHelloMsg;
import no.ntnu.item.serviceframe.signals.servicesession.ServiceSessionConfMsg;
import no.ntnu.item.serviceframe.signals.servicesession.ServiceSessionDeniedMsg;
import no.ntnu.item.serviceframe.signals.init.InitConfMsg;
import no.ntnu.item.serviceframe.signals.init.InitMsg;
import no.ntnu.item.serviceframe.signals.knownroles.KnownRolesReqMsg;
import messages.LookUpActorResultMsg;
import messages.SearchAliasMsg;
import se.ericsson.eto.norarc.ejbactorframe.messages.StartPlayingMsg;
import no.ntnu.item.serviceframe.signals.knownroles.KnownRolesResultMsg;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleDeniedMsg;


public class TerminalClientSessionActions extends Object {


	
	public static void requestAuthenticationToUA(RoleConfirmMsg signal, TerminalClientSessionSM asm){
		asm.userterminalsessionAddress = signal.getSenderRole();
		asm.sendMessage( new UserAuthenticationReq(
				asm.username,
				asm.password),
				asm.userterminalsessionAddress);
	    }

	public static void sendLogonRes(UserAuthenticationValid signal, TerminalClientSessionSM asm){
		asm.sendMessage(new LogonRes(), asm.myTerminal);
    }

	public static void sendLogonErr(UserAuthenticationInvalid signal, TerminalClientSessionSM asm){
		asm.sendMessage(new LogonErr(), asm.myTerminal);
	    }

	public static void sendLogOutToUA(LogoutReq signal, TerminalClientSessionSM asm){
	        // TODO edit action statements
	    }

	public static void forwardHelloToUCS(HelloMsg signal, TerminalClientSessionSM asm){
		signal.setReceiverRole(asm.userterminalsessionAddress);
		signal.setSenderRole(asm.getMyActorAddress());
		asm.sendMessage(signal);
    }

	public static void replyWithSSD(ServiceSessionReqMsg signal, TerminalClientSessionSM asm){
	        asm.sendMessage(new ServiceSessionDeniedMsg("ServiceID has not been specified"), asm.myTerminal);
	    }

	public static void saveSSR(ServiceSessionReqMsg signal, TerminalClientSessionSM asm){
     
	    }

	public static void forwardSSR(ServiceSessionReqMsg signal, TerminalClientSessionSM asm){
		//If we don't know which role we have to request, 
		//add ActorType to roleRequestors 
		//Expecting only one serviceID

		//OBA! use actorType of the actor who should play the role (here, TerminalAgent)
		String key = (String) signal.serviceIDs.get(0)+"."+asm.context.getParentAddress().getActorType();
		String roleID = (String) asm.knownRoles.get(key);
		if(roleID==null){
			if (signal.roleRequestors == null) signal.roleRequestors = new ArrayList();
			signal.roleRequestors.add(asm.context.getParentAddress().getActorType());
		}
		
		String terminalID = asm.context.getParentAddress().getActorID().substring(asm.context.getParentAddress().getActorID().lastIndexOf('/') + 1);
		signal.sessionDescription = new SessionDescription(asm.username, terminalID, null, null);
		//sending the message
		signal.setReceiverRole(asm.userterminalsessionAddress);
		signal.setSenderRole(asm.getMyActorAddress());
		asm.sendMessage(signal);
    }

	public static boolean SSRvalid(ServiceSessionReqMsg signal, TerminalClientSessionSM asm){
		return (signal.serviceIDs.size()>0);
	    }

	public static boolean SSRnotValid(ServiceSessionReqMsg signal, TerminalClientSessionSM asm){
		return (signal.serviceIDs.size()==0);
	    }

	public static void forwardHelloHello(HelloHelloMsg signal, TerminalClientSessionSM asm){
		//System.out.println("TerminalAddress: "+ asm.myTerminal);
		signal.setReceiverRole(asm.myTerminal);
		signal.setSenderRole(asm.getMyActorAddress());
		asm.sendMessage(signal);
    }

	public static void saveSSC(ServiceSessionConfMsg signal, TerminalClientSessionSM asm){
		asm.savedServiceSessionConfMsg = signal;
		//check if we know the role to request
		//expectiong only one serviceID
		String serviceID = (String) signal.serviceIDs.get(0);
		String key = serviceID+"."+asm.context.getParentAddress().getActorType();
		asm.roleToPlay = (String) asm.knownRoles.get(key);
		if(asm.roleToPlay == null) {
			//no role known. Check incomming roleProposals.
			Hashtable rp = signal.roleProposals;
			asm.roleToPlay = (String) rp.get(key);
		}
    }

	public static void sendSSDToTerminal(TerminalClientSessionSM asm){
		asm.sendMessage( new ServiceSessionDeniedMsg("Do not know the role to play for the requested service"), asm.myTerminal);
	   }

	public static void createRequestedServiceRole(ServiceSessionConfMsg signal, TerminalClientSessionSM asm){
		asm.sendMessage( new RoleRequestMsg(asm.roleToPlay.toLowerCase(), asm.roleToPlay), asm.context.getParentAddress());
	    }

	public static boolean SSCvalid(ServiceSessionConfMsg signal, TerminalClientSessionSM asm){
		return (asm.roleToPlay != null);
	    }

	public static boolean SSCnotValid(ServiceSessionConfMsg signal, TerminalClientSessionSM asm){
		return (asm.roleToPlay == null);
	    }

	public static void forwardSSD(ServiceSessionDeniedMsg signal, TerminalClientSessionSM asm){
	        signal.setReceiverRole(asm.myTerminal);
	        signal.setSenderRole(asm.getMyActorAddress());
	        asm.sendMessage(signal);
	    }

	public static void createRequestedServiceSessionRole(RoleConfirmMsg signal, TerminalClientSessionSM asm){
		asm.sendMessage(new RoleRequestMsg(
				(signal.getRoleRequest().getRoleType()+"Session").toLowerCase(),
				signal.getRoleRequest().getRoleType()+"Session"),
				signal.getSenderRole());
	    }

	public static void sendInitToCreatedSessionRole(RoleConfirmMsg signal, TerminalClientSessionSM asm){
		asm.sendMessage(
				new InitMsg(asm.username, (ActorAddress)asm.savedServiceSessionConfMsg.actorRefs.get((String) asm.savedServiceSessionConfMsg.serviceIDs.get(0))),
				signal.getSenderRole()
				);
	    }

	public static void sendSSCToTerminal(InitConfMsg signal, TerminalClientSessionSM asm){
		asm.savedServiceSessionConfMsg.actorRefs.put((String) asm.savedServiceSessionConfMsg.serviceIDs.get(0), signal.getAddress());
		asm.savedServiceSessionConfMsg.setSenderRole(asm.getMyActorAddress());
		asm.savedServiceSessionConfMsg.setReceiverRole(asm.myTerminal);
		asm.sendMessage(asm.savedServiceSessionConfMsg);
    }

	public static void saveLookupResult(LookUpActorResultMsg signal, TerminalClientSessionSM asm){
	}

	public static void requestUCStoUA(LookUpActorResultMsg signal, TerminalClientSessionSM asm){
		// generating ID for the requested UserTerminalSession
		// change this to something else (like MAC-address?)
		asm.userterminalsessionid =
			"UserTerminalSession" + System.currentTimeMillis();
		ArrayList agents = new ArrayList(signal.getResult());
		asm.sendMessage(
				new RoleRequestMsg(asm.userterminalsessionid, "UserTerminalSession"),
				(ActorAddress) agents.get(0)); //UA addres
	    }

	public static boolean lookupResultValid(LookUpActorResultMsg signal, TerminalClientSessionSM asm){
	        return (signal.getResult().size()>0);
	    }

	public static void sendLogonErrUnknownUser(LookUpActorResultMsg signal, TerminalClientSessionSM asm){
		asm.sendMessage(new LogonErr(), asm.myTerminal);
		asm.myTerminal = null;
	    }

	public static boolean lookupResultNotValid(LookUpActorResultMsg signal, TerminalClientSessionSM asm){
        return (signal.getResult().size()==0);
	    }

	public static void lookupUAaddress(LogonReq signal, TerminalClientSessionSM asm){
		//storing ref to terminal, as well as username and password
		asm.myTerminal = signal.getSenderRole();
		asm.username = signal.getUsername();
		asm.password = signal.getPassword();
		//asm.sendMessage(new SearchAliasMsg(asm.username, "UserAgent"), "nameServerPort");
		asm.sendMessage(new SearchAliasMsg(asm.username, "UserAgent"), new ActorAddress("sys/nameserver","NameServer"));
	    }

	public static void getKnownRoles(StartPlayingMsg signal, TerminalClientSessionSM asm){
	    asm.sendMessage(new KnownRolesReqMsg(), asm.context.getParentAddress());
	}

	public static void saveKnownRoles(KnownRolesResultMsg signal, TerminalClientSessionSM asm){
	    asm.knownRoles = signal.knownroles;
	}

	public static void sendInitToCreatedServiceRole(RoleDeniedMsg signal, TerminalClientSessionSM asm){
		asm.sendMessage(
				new InitMsg(asm.username, (ActorAddress)asm.savedServiceSessionConfMsg.actorRefs.get((String) asm.savedServiceSessionConfMsg.serviceIDs.get(0))),
				signal.getSenderRole()
				);
	}



}
