package no.ntnu.item.serviceframe.actors.useragent.userterminalsession;

import java.util.ArrayList;
import java.util.Hashtable;

import no.ntnu.item.serviceframe.auxiliar.agentprofile.UserProfile;
import no.ntnu.item.serviceframe.auxiliar.servicedescription.ServiceDescription;
import no.ntnu.item.serviceframe.signals.useragentauthentication.UserAuthenticationReq;
import no.ntnu.item.serviceframe.signals.useragentauthentication.UserAuthenticationValid;
import no.ntnu.item.serviceframe.signals.useragentauthentication.UserAuthenticationInvalid;
import no.ntnu.item.serviceframe.signals.agentprofile.ProfileLoadRequestMsg;
import no.ntnu.item.serviceframe.signals.agentprofile.ProfileMsg;
import no.ntnu.item.serviceframe.signals.handshake.HelloHelloMsg;
import no.ntnu.item.serviceframe.signals.handshake.HelloMsg;
import no.ntnu.item.serviceframe.signals.servicesession.ServiceSessionReqMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOnRRReqMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckNOKMsg;
import no.ntnu.item.serviceframe.signals.servicesession.ServiceSessionDeniedMsg;
import no.ntnu.item.serviceframe.signals.policycheck.PolicyCheckOKMsg;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleConfirmMsg;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleRequestMsg;
import se.ericsson.eto.norarc.ejbframe.ActorAddress;
import no.ntnu.item.serviceframe.signals.servicesession.ServiceSessionConfMsg;
import se.ericsson.eto.norarc.ejbactorframe.messages.RoleDeniedMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.LookupServicesMsg;
import no.ntnu.item.serviceframe.signals.serviceregistry.LookupServicesResultMsg;


public class UserTerminalSessionActions extends Object {

	public static boolean authReqValid(ProfileMsg signal, UserTerminalSessionSM asm){
		return (asm.userProfile.getUserName().equals(asm.savedAuthReq.getUsername())
				&& asm.userProfile.getPassword().equals(asm.savedAuthReq.getPassword()));
	    }

	public static boolean authReqNotValid(ProfileMsg signal, UserTerminalSessionSM asm){
		return  !(asm.userProfile.getUserName().equals(asm.savedAuthReq.getUsername())
				&& asm.userProfile.getPassword().equals(asm.savedAuthReq.getPassword()));
	    }

	public static void sendAuthValid(ProfileMsg signal, UserTerminalSessionSM asm){
		asm.sendMessage(new UserAuthenticationValid(), asm.savedAuthReq.getSenderRole());
	    }

	public static void sendAuthNotValid(ProfileMsg signal, UserTerminalSessionSM asm){
		asm.sendMessage(new UserAuthenticationInvalid(), asm.savedAuthReq.getSenderRole());
	    }

	public static void sendPolicyCheckRequest(ServiceSessionReqMsg signal, UserTerminalSessionSM asm){
		asm.savedServiceSessionReqMsg = signal;
		String key = (String) signal.serviceIDs.get(0)+"."+asm.context.getParentAddress().getActorType();
		//asm.sendMessage(new PolicyCheckOnRRReqMsg((String)asm.userProfile.neededRolesForAvailableServices.get(key), signal.sessionDescription), "policyManagerPort");
		asm.sendMessage(new PolicyCheckOnRRReqMsg((String)asm.userProfile.knownRoles.get(key), signal.sessionDescription), new ActorAddress(asm.context.getParentAddress().getActorID()+"/userPolicyManager", "PolicyManager"));
	}

	public static void sendSSD(PolicyCheckNOKMsg signal, UserTerminalSessionSM asm){
		//asm.sendMessage(new ServiceSessionDeniedMsg("According to the system policies access to the requested service is not allowed"), "tcsPort");
		asm.sendMessage(new ServiceSessionDeniedMsg("According to the system policies access to the requested service is not allowed"), asm.terminalClientSessionRef);

	}

	public static void createRequestedServiceRole(PolicyCheckOKMsg signal, UserTerminalSessionSM asm){
		// We assume that the requested service (only one) can be played
		String serviceID = (String) asm.savedServiceSessionReqMsg.serviceIDs.get(0);
		String key = serviceID+"."+asm.context.getParentAddress().getActorType();
		String serviceRole = (String)asm.userProfile.knownRoles.get(key);
		//if(serviceRole==null) {
			//TODO: Could not find role, report error with ServiceSessionDenied
		//	sameState(asm);
		//	return;
		//}
		RoleRequestMsg rr = new RoleRequestMsg(serviceRole.toLowerCase(), serviceRole);
		rr.setReceiverRole(asm.context.getParentAddress());
		asm.sendMessage(rr);
	}

	public static void createRequestedServiceSessionRole(RoleConfirmMsg signal, UserTerminalSessionSM asm){
		// We do not know if the requested service supports session roles.
		// We just try to create one. We will see if we get a RoleConfirmMsg
		// or a RoleDeniedMsg!
		// The type of the session-role is implicitly given by the type 
		// of the role just created: "<roleType>Session"
		String sessionRole = signal.getRoleRequest().getRoleType()+"Session";
		RoleRequestMsg rr = new RoleRequestMsg((sessionRole.toLowerCase())+ System.currentTimeMillis(), sessionRole);
		rr.setReceiverRole(signal.getSenderRole());
		asm.sendMessage(rr);
	}

	public static boolean serviceRoleKnown(ServiceSessionReqMsg signal, UserTerminalSessionSM asm){
		String key = (String) signal.serviceIDs.get(0)+"."+asm.context.getParentAddress().getActorType();
		return ((String)asm.userProfile.knownRoles.get(key)!=null);
	}

	public static void sendSSDonRoleUnknown(ServiceSessionReqMsg signal, UserTerminalSessionSM asm){
	    asm.sendMessage(new ServiceSessionDeniedMsg("Do not know the role to play for the requested service"), signal.getSenderRole());
	}

	public static boolean serviceRoleUnknown(ServiceSessionReqMsg signal, UserTerminalSessionSM asm){
		String key = (String) signal.serviceIDs.get(0)+"."+asm.context.getParentAddress().getActorType();
		return ((String)asm.userProfile.knownRoles.get(key)==null);
	}

	public static void requestUserProfile(UserAuthenticationReq signal, UserTerminalSessionSM asm){
	    asm.savedAuthReq = signal;
	    //asm.sendMessage(new ProfileLoadRequestMsg(false), "userProfileEditorPort");
	    asm.sendMessage(new ProfileLoadRequestMsg(false), new ActorAddress(asm.context.getParentAddress().getActorID()+"/profileEditor", "ProfileEditor"));
	}

	public static void saveUserProfile(ProfileMsg signal, UserTerminalSessionSM asm){
	    asm.userProfile = (UserProfile) signal.profile;
	}

	public static void sendSSC1(RoleConfirmMsg signal, UserTerminalSessionSM asm){
		ArrayList rr = asm.savedServiceSessionReqMsg.roleRequestors;
		ArrayList sIDs = asm.savedServiceSessionReqMsg.serviceIDs;
		Hashtable rp = asm.savedServiceSessionReqMsg.roleProposals;
		if (rp == null) rp = new Hashtable();

		String serviceID;
		String requestingActorType;
		String key;
		String roleType;

		if (rr != null && rr.size()>0) {
			//search the incomming serviceID and roleRequestor list and check if we can provide any of the requested roles
			for(int i=0;i<sIDs.size();i++) {
				for(int j=0;j<rr.size();j++) {
					serviceID = (String) sIDs.get(i);
					requestingActorType = (String) rr.get(j);
					key = serviceID+"."+requestingActorType;
					roleType = (String) asm.userProfile.knownRoles.get(key);
					if(roleType!=null) rp.put(serviceID+"."+requestingActorType, roleType);
				}
			}
		}
		Hashtable actorRefs =new Hashtable();
		actorRefs.put((String)(sIDs.get(0)), signal.getSenderRole());
		
		ServiceSessionConfMsg sscm = new ServiceSessionConfMsg(
				sIDs,
				rr, 
				rp,
				actorRefs
				);

		//The message is sent to the actor that requested the service
		sscm.setReceiverRole(asm.savedServiceSessionReqMsg.getSenderRole());
		asm.sendMessage(sscm);
	}

	public static void sendSSC2(RoleDeniedMsg signal, UserTerminalSessionSM asm){
		//Here we interpret the reception of RoleDeniedMsg as: 
		//"The requested service does not support session roles. It only 
		// consists on a behavior role"
		
		ArrayList rr = asm.savedServiceSessionReqMsg.roleRequestors;
		ArrayList sIDs = asm.savedServiceSessionReqMsg.serviceIDs;
		Hashtable rp = asm.savedServiceSessionReqMsg.roleProposals;
		if (rp == null) rp = new Hashtable();

		String serviceID;
		String requestingActorType;
		String key;
		String roleType;

		if (rr != null && rr.size()>0) {
			//search the incomming serviceID and roleRequestor list and check if we can provide any of the requested roles
			for(int i=0;i<sIDs.size();i++) {
				for(int j=0;j<rr.size();j++) {
					serviceID = (String) sIDs.get(i);
					requestingActorType = (String) rr.get(j);
					key = serviceID+"."+requestingActorType;
					roleType = (String) asm.userProfile.knownRoles.get(key);
					if(roleType!=null) rp.put(serviceID+"."+requestingActorType, roleType);
				}
			}
		}
		Hashtable actorRefs =new Hashtable();
		actorRefs.put((String)(sIDs.get(0)), signal.getSenderRole());
		
		ServiceSessionConfMsg sscm = new ServiceSessionConfMsg(
				sIDs,
				rr, 
				rp,
				actorRefs
				);

		//The message is sent to the actor that requested the service
		sscm.setReceiverRole(asm.savedServiceSessionReqMsg.getSenderRole());
		asm.sendMessage(sscm);
	}

	public static void lookupForAvailableServices(HelloMsg signal, UserTerminalSessionSM asm){
		asm.terminalClientSessionRef = signal.getSenderRole();
		asm.sendMessage(new LookupServicesMsg("*"), new ActorAddress("sys/serviceregistry","ServiceRegistry"));
	}

	public static void sendBackHelloHello(LookupServicesResultMsg signal, UserTerminalSessionSM asm){
		//Store the results in the UserProfile
		for (int i=0; i<signal.serviceList.size();i++){
			asm.userProfile.addAvailableService((ServiceDescription) signal.serviceList.get(i));
			asm.userProfile.addKnownRoles(((ServiceDescription) signal.serviceList.get(i)).rolesToBePlayed);
		}

		asm.sendMessage(new HelloHelloMsg(asm.userProfile.getAvailableServices(), asm.userProfile.getKnownRoles(), null), asm.terminalClientSessionRef);
    }



}
