/*
 * Created on 15.sep.2003
 */
package no.ntnu.item.serviceframe.auxiliar.agentprofile;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import no.ntnu.item.serviceframe.auxiliar.servicedescription.ServiceDescription;

/**
 * The user profile class. Contains information regarding the user,
 * the services the user subscribes to, and the activation of these
 * services.
 *
 * @author Alf Kristian Style
 * @version 1.0, 15.sep.2003
 */
public class UserProfile extends AgentProfile{


    //List of subscribed services. Contains serviceIDs (String)
    private java.util.ArrayList subscriptions = new ArrayList();

    //List of available services. Contains ServiceDescription objects
    public java.util.ArrayList availableServices = new ArrayList();

    // Hashtable with the roles that different actors/agents has to play 
	// in order to participate in a certain service execution
	// This information is stored in the following format:
    // Key: <<serviceID>>.<<container_actor_type>>
	// Value: <<role_type>>
	// That means that an actor with type <<container_actor_type>> has to play
	// a role of type <<role_type>> to participate in the service <<serviceID>>  
    public Hashtable knownRoles = new Hashtable();

    private java.lang.String location;
    private java.lang.String userAvailability;

    //List of screened users. Contains usernames (String)
    private java.util.ArrayList screeningList;

    //Hastbale with the satatus of each terminal
    //Index: TerminalID (String)
    //Value: Status (String)
    private java.util.Hashtable terminalStatus;
	
    //List of terminals. Contains terminalIDs (String)
    private ArrayList terminals = new ArrayList();
	
	private String userName = null;

	private String password = null;

	private String systemPrivileges = "user";

	private String name = null;

	private String title = null;

	private String gender = null;

	private int age = -1;

	private ArrayList interests = new ArrayList();

	private ArrayList emails = new ArrayList();

	private ArrayList phoneNumbers = new ArrayList();

	private ArrayList addresses = new ArrayList();

	private String userGroup= null;

	//private Position position = null;

	//private PositionDefinitions positionDefinitions = null

	private ArrayList calendar = new ArrayList();

	private ArrayList stateDefinitions = new ArrayList();

	private ArrayList buddyList = new ArrayList();

	private ArrayList applicationList = new ArrayList();

	private ArrayList favorites = new ArrayList();

	private ArrayList portalVariables = new ArrayList();

	private Hashtable applicationSpecificData = new Hashtable();

	//private ArrayList currentTerminals = new ArrayList();



	public UserProfile() {
		
	}
	

	public java.lang.String getLocation() {
		return location;
	}

	public void setLocation(java.lang.String location) {
		this.location = location;
	}

	public java.util.ArrayList getSubscriptions() {
		return subscriptions;
	}

	public void setSubscriptions(java.util.ArrayList subscriptions) {
		this.subscriptions = subscriptions;
	}

	public void addSubscription(String subscription) {
		this.subscriptions.add(subscription);
	}

	public void removeSubscription(String subscription) {
		int i = subscriptions.indexOf(subscription);
		if (i > -1) {
			subscriptions.remove(i);
		}
	}

	
	
	
	public java.util.ArrayList getAvailableServices() {
		ArrayList sl = new ArrayList();
		for (int i=0; i<availableServices.size();i++) {
			sl.add(((ServiceDescription)availableServices.get(i)).clone());
		}
		return sl;
	}


	public void setAvailableServices(java.util.ArrayList availableServices) {
		this.availableServices.clear();
		
		for (int i=0; i<availableServices.size();i++) {
			this.availableServices.add(((ServiceDescription)availableServices.get(i)).clone());
		}
		
	}

	public void addAvailableService(ServiceDescription availableService) {
		this.availableServices.add(availableService.clone());
	}

	public void removeAvailableService(ServiceDescription availableService) {
		int i = availableServices.indexOf(availableService);
		if (i > -1) {
			availableServices.remove(i);
		}
	}

	public java.util.Hashtable getKnownRoles() {
		return new Hashtable(knownRoles);
	}


	public void setKnownRoles(
			java.util.Hashtable neededRolesForAvailableServices) {
		this.knownRoles.clear();
		this.knownRoles.putAll(neededRolesForAvailableServices);
	}

	public void addKnownRoles(Hashtable neededRolesForAvailableServices) {
		this.knownRoles.putAll(neededRolesForAvailableServices);
	}

	public void addNeededRoleForAvailableService(String serviceID, String containerActorType, String roleToBePlayed) {
		this.knownRoles.put(serviceID+"."+containerActorType, roleToBePlayed);
	}

	public void removeNeededRoleForAvailableService(String serviceID, String containerActorType) {
		if (knownRoles.containsKey(serviceID+"."+containerActorType)) {
			knownRoles.remove(serviceID+"."+containerActorType);
		}
	}

	
	public java.lang.String getTerminalStatus(String terminalID) {
		return (String) terminalStatus.get(terminalID);
	}

	public void setTerminalStatus(String terminalID, String terminalStatus) {
		this.terminalStatus.put(terminalID, terminalStatus);
	}

	public java.lang.String getUserAvailability() {
		return userAvailability;
	}

	public void setUserAvailability(java.lang.String userAvailability) {
		this.userAvailability = userAvailability;
	}

	public java.util.ArrayList getScreeningList() {
		return screeningList;
	}

	public void setScreeningList(java.util.ArrayList screeningList) {
		this.screeningList = screeningList;
	}

	public void addUserToScreeningList(String screenedUser) {
		this.screeningList.add(screenedUser);
	}

	public void removeUserService(String serviceName){
		for (Iterator iter = applicationList.iterator(); iter.hasNext();) {
			String element = (String)iter.next();
			if (element.equals(serviceName)) {
				iter.remove();
			}
		}
		applicationSpecificData.remove(serviceName);
	}

//	private void updateRoleActivations(RoleActivationRules rules){
//		applicationSpecificData.remove(rules.getRoleType());
//		setRoleActivationRules(rules);
//	}


	/**
	 *
	 * @param list
	 */
//	private void setRoleActivationRules(RoleActivationRules rules) {
//		applicationSpecificData.put(rules.getRoleType(), rules);
//	}

	/**
	 *
	 * @return
	 */
//	public RoleActivationRules getRoleActivationRules(String serviceName) {
//		return (RoleActivationRules)applicationSpecificData.get(serviceName);
//	}




	/**
	 *
	 * @param list
	 */
	public void setApplicationList(ArrayList list) {
		applicationList = list;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getAddresses() {
		return addresses;
	}

	/**
	 *
	 * @return
	 */
	public int getAge() {
		return age;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getApplicationList() {
		return applicationList;
	}



	/**
	 *
	 * @return
	 */
	public ArrayList getBuddyList() {
		return buddyList;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getCalendar() {
		return calendar;
	}


	/**
	 *
	 * @return
	 */
	public ArrayList getEmails() {
		return emails;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getFavorites() {
		return favorites;
	}

	/**
	 *
	 * @return
	 */
	public String getGender() {
		return gender;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getInterests() {
		return interests;
	}

	/**
	 *
	 * @return
	 */
	public String getName() {
		return name;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getPhoneNumbers() {
		return phoneNumbers;
	}

	/**
	 *
	 * @return
	 */
	public ArrayList getPortalVariables() {
		return portalVariables;
	}

	/**
	 *
	 * @return
	 */
//	public Position getPosition() {
//		return position;
//	}


	/**
	 *
	 * @return
	 */
	public ArrayList getStateDefinitions() {
		return stateDefinitions;
	}

	/**
	 *
	 * @return
	 */
	public String getTitle() {
		return title;
	}

	/**
	 *
	 * @return
	 */
	public String getUserGroup() {
		return userGroup;
	}

	/**
	 *
	 * @return
	 */
	public String getUserName() {
		return userName;
	}

	/**
	 *
	 * @param list
	 */
	public void setAddresses(ArrayList list) {
		addresses = list;
	}

	/**
	 *
	 * @param i
	 */
	public void setAge(int i) {
		age = i;
	}


	/**
	 *
	 * @param list
	 */
	public void setBuddyList(ArrayList list) {
		buddyList = list;
	}

	/**
	 *
	 * @param list
	 */
	public void setCalendar(ArrayList list) {
		calendar = list;
	}

	/**
	 *
	 * @param list
	 */
//	public void setCurrentTerminals(LoginTerminalInfo aa) {
//		currentTerminals.add(aa);
//	}

	/**
	 *
	 * @param list
	 */
	public void setEmails(ArrayList list) {
		emails = list;
	}

	/**
	 *
	 * @param list
	 */
	public void setFavorites(ArrayList list) {
		favorites = list;
	}

	/**
	 *
	 * @param string
	 */
	public void setGender(String string) {
		gender = string;
	}

	/**
	 *
	 * @param list
	 */
	public void setInterests(ArrayList list) {
		interests = list;
	}

	/**
	 *
	 * @param string
	 */
	public void setName(String string) {
		name = string;
	}

	/**
	 *
	 * @param list
	 */
	public void setPhoneNumbers(ArrayList list) {
		phoneNumbers = list;
	}

	/**
	 *
	 * @param list
	 */
	public void setPortalVariables(ArrayList list) {
		portalVariables = list;
	}

	/**
	 *
	 * @param position
	 */
//	public void setPosition(Position position) {
//		this.position = position;
//	}


	public ArrayList getTerminals() {
		return terminals;
	}
	
	public void setTerminals(ArrayList list) {
		terminals = list;
	}

	public void addTerminal(String terminal) {
		terminals.add(terminal);
	}

	public void removeTerminal(String terminal) {
		int i = terminals.indexOf(terminal);
		if (i > -1) {
			terminals.remove(i);
		}
		
	}
	
	
	/**
	 *
	 * @param list
	 */
	public void setStateDefinitions(ArrayList list) {
		stateDefinitions = list;
	}

	/**
	 *
	 * @param string
	 */
	public void setTitle(String string) {
		title = string;
	}

	/**
	 *
	 * @param string
	 */
	public void setUserGroup(String string) {
		userGroup = string;
	}

	/**
	 *
	 * @param string
	 */
	public void setUserName(String string) {
		userName = string;
	}

	/**
	 *
	 * @return
	 */
	public String getPassword() {
		return password;
	}

	/**
	 *
	 * @param string
	 */
	public void setPassword(String string) {
		password = string;
	}

	/**
	 *
	 * @return
	 */
	public String getSystemPrivileges() {
		return systemPrivileges;
	}

	/**
	 *
	 * @param string
	 */
	public void setSystemPrivileges(String string) {
		systemPrivileges = string;
	}

	/* (non-Javadoc)
	 * @see useragentsystem.structures.profiles.VisibleProfile#getAllApplicationSpecificData()
	 */
	public Enumeration getAllApplicationSpecificData() {
		return applicationSpecificData.elements();
	}

	/* (non-Javadoc)
	 * @see useragentsystem.structures.profiles.VisibleProfile#getAllApplicationSpecificData()
	 */
	public Hashtable getAllApplicationSpecificDataTable() {
		return applicationSpecificData;
	}

	/* (non-Javadoc)
	 * @see useragentsystem.structures.profiles.VisibleProfile#setApplicationSpecificData(java.util.Hashtable)
	 */
	public void setApplicationSpecificData(Hashtable list) {
		this.applicationSpecificData = list;
	}

//	public RoleActivationProfile getRoleActivationProfile(){
//		return new RoleActivationProfile(this, addresses, age, applicationList, buddyList, calendar, currentTerminals, emails, favorites, gender, interests, name, phoneNumbers, preferredTerminals, stateDefinitions, title, userGroup, userName, position, systemPrivileges, password);
//	}

/*	public void removeCurrentTerminal(ActorAddress aa){
		int i = currentTerminals.indexOf(aa);
		if (i > -1) {
			currentTerminals.remove(i);
		}
	}

	public ArrayList getCurrentTerminals() {
		return currentTerminals;
	}
*/	

}
